/******************************************************************************

                              Copyright (c) 2009
                            Lantiq Deutschland GmbH
                     Am Campeon 3; 85579 Neubiberg, Germany

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

  ****************************************************************************/

/**
   \file board_easy3111.c
   \date 2006-10-01
   \brief Interface implementation for easy3111 board.

   Application needs it to initialize board, prepare structures for this board
   and set mode in which board will work (using PCM, GPIO, ...).
*/

/* ============================= */
/* Includes                      */
/* ============================= */
#include "tapidemo.h"
#include "common.h"
#include "board_easy3201.h"
#include "board_easy3111.h"

/* ============================= */
/* Defines                       */
/* ============================= */

/* Board EASY3111 uses the same function as a board EASY3201 */
#define BOARD_Easy3111_CfgPCMInterface BOARD_Easy3201_CfgPCMInterface
#define BOARD_Easy3211_Reset BOARD_Easy3201_Reset 

#define TD_EASY3111_DEVICE_NUMBER 2 
#define TD_EASY3111_MAX_CHANNEL_NUMBER 8


/* ============================= */
/* Local variable definition     */
/* ============================= */


/** How channels are represented by device, socket.
    Device /dev/vin11 will get status, data from phone channel 0 and
    data channel 0. If at startup phone channel 0 is mapped to data channel 1
    then, phone will send actions to dev/vin11 and data will send actions to
    /dev/vin12.
    -1 means no channel at startup */

/** Max phones on EASY3111 (analog channels)
    It is used only for STARTUP_MAPPING_EASY3111[] table. 
    Number of analog channels used in program is read from drivers 
    using IFX_TAPI_CAP_LIST  */
enum { MAX_PHONES_EASY3111 = 2 };

/** Name of board */
static const char* BOARD_NAME_EASY3111 = "EASY3111";

/** Holds default phone channel to data, pcm channel mappings
    defined at startup. */
static const STARTUP_MAP_TABLE_t STARTUP_MAPPING_EASY3111[MAX_PHONES_EASY3111] =
{
   /* IMORTANT - for this board this is default mapping done by drivers,
      to change this mapping channels(phone and PCM) must be unmapped first */
   /** phone, data, pcm */
   {  0,  0,  0  },
   {  1,  1,  1  }
   /* others are free and can be used later for conferencing */
};

/** Device number to detect */
static IFX_int32_t nDeviceToDetect = 2;

/** Detect boards */
static IFX_boolean_t nDetectDevice = IFX_TRUE;

/* ============================= */
/* Global variable definition    */
/* ============================= */

/* ============================= */
/* Local function declaration    */
/* ============================= */
IFX_return_t BOARD_Easy3111_PrepareBoard(BOARD_t* pBoard, IFX_char_t* pPath);
IFX_return_t BOARD_Easy3111_PrepareNames(BOARD_t* pBoard);
IFX_return_t BOARD_Easy3111_InitSystem(BOARD_t* pBoard);
IFX_return_t BOARD_Easy3111_GetCapabilities(BOARD_t* pBoard,
                                            IFX_char_t* pPath);
IFX_void_t BOARD_Easy3111_RemoveBoard(BOARD_t* pBoard);
IFX_return_t BOARD_Easy3111_Init(BOARD_t* pBoard, PROGRAM_ARG_t* pProgramArg);

/* ============================= */
/* Local function definition     */
/* ============================= */
/**
   Prepare board structure (reserve memory, inicialize, ...).

   \param pBoard - pointer to Board
   \param pPath - path to FW files for download

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remark
*/
IFX_return_t BOARD_Easy3111_PrepareBoard(BOARD_t* pBoard, IFX_char_t* pPath)
{
   IFX_int32_t i = 0;
   TAPIDEMO_DEVICE_t* pDevice = IFX_NULL;
   TAPIDEMO_DEVICE_CPU_t* pCPUDevice = IFX_NULL;

   /* check input arguments */
   if (pBoard == IFX_NULL || pPath == IFX_NULL)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /* First three variables (id, type, sub type) set before this func is called */

   /* File descriptors not open */
   pBoard->nSystem_FD = -1;
   pBoard->nDevCtrl_FD = -1;
   /* Set flag to use system FD */
   pBoard->nUseSys_FD = IFX_TRUE;

   /* allocate memory for CPU device structure */
   pDevice = calloc(1, sizeof(TAPIDEMO_DEVICE_t));
   if (pDevice == IFX_NULL)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("ERR Allocate memory for CPU Device. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /* Init structure for CPU device and add it to the board */
   pDevice->nType = TAPIDEMO_DEV_CPU;
   pCPUDevice = &pDevice->uDevice.stCpu;
   /** Pointer to device dependent structure */
   pCPUDevice->pDevDependStuff = (IFX_void_t *) &DuslicXtDevInitStruct;
#ifdef USE_FILESYSTEM
   /** FW files */
   pCPUDevice->pszDRAM_FW_File = sDRAMFile_DXT;
   pCPUDevice->pszPRAM_FW_File[0] = sPRAMFile_DXT;
   pCPUDevice->pszPRAM_FW_File[1] = sPRAMFile_DXT_Old;
   pCPUDevice->pszBBD_CRAM_File[0] = sBBD_CRAM_File_DXT;
   pCPUDevice->pszBBD_CRAM_File[1] = sBBD_CRAM_File_DXT_Old;
#ifndef VXWORKS
   /* We don't need to set download path. It was done during inititialize main board */
   /* Just check if directory contains required file for extension board*/ 
   if (IFX_SUCCESS != Common_CheckDownloadPath(pPath, pCPUDevice, IFX_TRUE))
   {
      return IFX_ERROR;       
   }
#endif /* VXWORKS */
#endif /* USE_FILESYSTEM */

   if (Common_AddDevice(pBoard, pDevice) == IFX_ERROR)
   {
      TD_OS_MemFree(pDevice);
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("ERR, CPU device not added to the board. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   if (DEVICE_DUSLIC_XT_Register(pCPUDevice) == IFX_ERROR)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("ERR, CPU device not registered. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   if (BOARD_Easy3111_PrepareNames(pBoard) == IFX_ERROR)
      return IFX_ERROR;

   /** Startup mapping table */
   pBoard->pChStartupMapping = STARTUP_MAPPING_EASY3111;

   /* Basic device initialization */
   if (BOARD_Easy3111_InitSystem(pBoard) == IFX_ERROR)
   {
       return IFX_ERROR;
   }

   if (IFX_ERROR == DEVICE_DUSLIC_XT_setupChannel(pBoard, pPath, IFX_TRUE))
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("%s: Unable to initialize extension board.\n"
            "%sDevice node %s is set without extension board being connected\n"
            "%sor initalization error occured.\n",
            pBoard->pszBoardName, pBoard->pIndentBoard, pBoard->pszCtrlDevName,
            pBoard->pIndentBoard));
      TD_OS_DeviceClose(pBoard->nDevCtrl_FD);
      return IFX_ERROR;
   }


   if (IFX_ERROR == BOARD_Easy3111_GetCapabilities(pBoard, pPath))
   {
      return IFX_ERROR;
   }

   if (IFX_ERROR == Common_SetPhonesAndUsedChannlesNumber(pBoard))
   {
      return IFX_ERROR;
   }


   pBoard->nCh_FD = calloc(pBoard->nUsedChannels, sizeof(IFX_int32_t));
   if (pBoard->nMaxCoderCh > 0)
   {
      pBoard->pDataChStat = calloc(pBoard->nMaxCoderCh, sizeof(VOIP_DATA_CH_t));
      pBoard->rgnSockets = calloc(pBoard->nMaxCoderCh, sizeof(IFX_int32_t));
   }

   pBoard->rgoPhones = calloc(pBoard->nMaxPhones, sizeof(PHONE_t));

   if ((pBoard->rgoPhones == IFX_NULL) || (pBoard->nCh_FD == IFX_NULL))
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("ERR Allocate memory to prepare board. "
             "(File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;         
   }

   for (i = 0; i < pBoard->nMaxCoderCh; i++)
   {
       pBoard->rgnSockets[i] = -1;
   }

   for (i = 0; i < pBoard->nUsedChannels; i++)
   {
       pBoard->nCh_FD[i] = -1;
   }

   /* Prepare array of free PCM channels  */
   pBoard->fPCM_ChFree = calloc(pBoard->nMaxPCM_Ch, sizeof(IFX_boolean_t));

   if (pBoard->fPCM_ChFree == IFX_NULL)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("ERR allocate memory for PCM channels. "
            "(File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;         
   }

   /* All PCM channels are free at start */
   for (i = 0; i < pBoard->nMaxPCM_Ch; i++)
   {
       pBoard->fPCM_ChFree[i] = IFX_TRUE;
   }

   if (IFX_SUCCESS != Common_Set_FDs(pBoard))
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Error setting fds. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   return IFX_SUCCESS;
}

/**
   Release board structure (reserve memory, inicalize, ...).

   \param pBoard - pointer to board

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remark Must be called before application exit.
*/
IFX_return_t BOARD_Easy3111_PrepareNames(BOARD_t* pBoard)
{
   IFX_char_t tmp_buf[10] = {0};

   /* check input arguments */
   if (pBoard == IFX_NULL)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }
   /** Device name without numbers, like /dev/vmmc, /dev/vin, ... */
   strncpy(pBoard->pszChDevName, sCH_DEV_NAME_DXT, 
           strlen(sCH_DEV_NAME_DXT));

   /** Device control name numbers, like /dev/vmmc[board id]0, ... */
   strncpy(pBoard->pszCtrlDevName, sCTRL_DEV_NAME_DXT, 
           strlen(sCTRL_DEV_NAME_DXT));
   /*snprintf(tmp_buf, 2, "%d", (int) pBoard->nID);*/
   sprintf(tmp_buf, "%d", (int) pBoard->nID);
   strncat(pBoard->pszCtrlDevName, tmp_buf, 1);
   /*snprintf(tmp_buf, 2, "0");*/
   sprintf(tmp_buf, "0");
   strncat(pBoard->pszCtrlDevName, tmp_buf, 1);

   /** System device name, like /dev/easy3201/[<board id], ... */
   /** BUT actually we only have one */
   strncpy(pBoard->pszSystemDevName, sSYS_DEV_NAME_DXT,
           strlen(sSYS_DEV_NAME_DXT));

   return IFX_SUCCESS;
}

/**
   Get capabilities, basically number of phone, data and pcm channels.

   \param pBoard - pointer to board
   \param pPath - path to FW files for download

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remark 
*/
IFX_return_t BOARD_Easy3111_GetCapabilities(BOARD_t* pBoard,
                                            IFX_char_t* pPath)
{
   IFX_return_t ret = IFX_ERROR;

   if ((pBoard == IFX_NULL) || (pPath == IFX_NULL))
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   ret = Common_GetCapabilities(pBoard);
   if (ret == IFX_ERROR)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Err, Capabilities not read. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
   }

   if (pBoard->nMaxAnalogCh > MAX_PHONES_EASY3111)
   {
      /* If MAX_PHONES_EASY3111 is too small than STARTUP_MAPPING_EASY3111[] table 
         has too small size and TAPIDEMO will from wrong place */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Err, MAX_PHONES_EASY3111 is too small. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }
   return ret;
}
/**
   Basic device initialization prior to DuSLIC-XT initialization.

   \param pBoard - pointer to board

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remark
      DEVICE_DUSLIC_XT_BasicInit for this board is done
         in BOARD_Easy3201_InitSystem()
*/
IFX_return_t BOARD_Easy3111_InitSystem(BOARD_t* pBoard)
{
   IFX_uint32_t nChipNum = 0;
   EASY3201_CPLD_GET_DEV_PARAM_t chip_param_dxt;
   TAPIDEMO_DEVICE_t *pDevice = IFX_NULL;
   TAPIDEMO_DEVICE_CPU_t   *pCpuDevice = IFX_NULL;

   if (pBoard == IFX_NULL)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   if (0 >= pBoard->nDevCtrl_FD)
   {
      pBoard->nDevCtrl_FD = Common_Open(pBoard->pszCtrlDevName);
      if (0 >= pBoard->nDevCtrl_FD)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
              ("Invalid device control file descriptor %s.\n"
               "(File: %s, line: %d)\n",
               pBoard->pszCtrlDevName, __FILE__, __LINE__));
         return IFX_ERROR;
      }
   }

   if (0 >= pBoard->nSystem_FD)
   {
      /* Get system fd */
      pBoard->nSystem_FD = Common_Open(pBoard->pszSystemDevName);

      if (0 >= pBoard->nSystem_FD)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
              ("Invalid system file descriptor. (File: %s, line: %d)\n",
               __FILE__, __LINE__));
         return IFX_ERROR;
      }
   }

   if ((pDevice = Common_GetDevice_CPU(pBoard->pDevice)) == IFX_NULL)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Err, CPU device not found. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }
   pCpuDevice = &pDevice->uDevice.stCpu;

   /* set chip number */
   nChipNum = 1;

   /* Store chip count */
   pBoard->nChipCnt = 1;

   /* activate chip reset */
   if (BOARD_Easy3211_Reset(pBoard, nChipNum, IFX_TRUE) != IFX_SUCCESS)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("ERROR: Basic Device Initialization fails on device %d"
            "(File: %s, line: %d)\n",
            nChipNum, __FILE__, __LINE__));
      return IFX_ERROR;
   }

   memset(&chip_param_dxt, 0, sizeof(EASY3201_CPLD_GET_DEV_PARAM_t));
   chip_param_dxt.nChipNum = nChipNum;

   /* read dxt device configuration parameters from board driver */
   if (TD_IOCTL(pBoard->nSystem_FD, FIO_EASY3201_GETBOARDPARAMS,
                (IFX_int32_t) &chip_param_dxt) != IFX_SUCCESS)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Err with ioctl cmd FIO_EASY3201_GETBOARDPARAMS."
             "(File: %s, line: %d)\n", __FILE__, __LINE__));            
      return IFX_ERROR;
   }

   /* set Irq Line number */
   pCpuDevice->nIrqNum = chip_param_dxt.nIrqNum;

   /* clear reset to allow vinetic access */
   if (BOARD_Easy3211_Reset(pBoard, nChipNum, IFX_FALSE) != IFX_SUCCESS)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("ERROR: Basic Device Initialization fails on device %d"
             "(File: %s, line: %d)\n",
             nChipNum, __FILE__, __LINE__));
      return IFX_ERROR;
   }


   return IFX_SUCCESS;
} /* BOARD_Easy3111_InitSystem() */

/* ============================= */
/* Global function definition    */
/* ============================= */

/**
   Release board structure (reserve memory, inicalize, ...).

   \param pBoard - pointer to board

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remark Must be called before application exit.
*/
IFX_void_t BOARD_Easy3111_RemoveBoard(BOARD_t* pBoard)
{
   if (pBoard->pDataChStat != IFX_NULL)
   {
      TD_OS_MemFree(pBoard->pDataChStat);
      pBoard->pDataChStat = IFX_NULL;
   }

   if (pBoard->rgoPhones != IFX_NULL)
   {
      TD_OS_MemFree(pBoard->rgoPhones);
      pBoard->rgoPhones = IFX_NULL;
   }

   if (pBoard->nCh_FD != IFX_NULL)
   {
      TD_OS_MemFree(pBoard->nCh_FD);
      pBoard->nCh_FD = IFX_NULL;
   }

   if (pBoard->rgnSockets != IFX_NULL)
   {
      TD_OS_MemFree(pBoard->rgnSockets);
      pBoard->rgnSockets = IFX_NULL;
   }

   if (pBoard->fPCM_ChFree != IFX_NULL)
   {
      TD_OS_MemFree(pBoard->fPCM_ChFree);
      pBoard->fPCM_ChFree = IFX_NULL;
   }

   Common_RemoveDeviceList(pBoard);
}

/**
   Read version after succesfull initialization and display it.

   \param pBoard - pointer to board
   \param pProgramArg - pointer to program arguments

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remark
      This must be done only after a successfull basic device initialization
*/
IFX_return_t BOARD_Easy3111_Init(BOARD_t* pBoard, PROGRAM_ARG_t* pProgramArg)
{
   /* check input arguments */
   if (pBoard == IFX_NULL)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   if (BOARD_Easy3111_PrepareBoard(pBoard, pProgramArg->sPathToDwnldFiles)
       == IFX_ERROR)
   {
       return IFX_ERROR;
   }

   /* Initialize chip */
   if (DEVICE_DUSLIC_XT_Init(pBoard, pProgramArg->sPathToDwnldFiles) 
       == IFX_ERROR)
   {
       return IFX_ERROR;
   }

   if (BOARD_Easy3111_CfgPCMInterface(pBoard, IFX_FALSE) == IFX_ERROR)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("PCM initialization failed. (File: %s, line: %d)\n",
             __FILE__, __LINE__));
      return IFX_ERROR;
   }

   return IFX_SUCCESS;
}

/**
   Register board.

   \param pBoard - pointer to board
 
   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remarks 
*/
IFX_return_t BOARD_Easy3111_Register(BOARD_t* pBoard)
{
   TD_PTR_CHECK(pBoard, IFX_ERROR);

   pBoard->pszBoardName = BOARD_NAME_EASY3111;
   pBoard->Init = BOARD_Easy3111_Init;
   pBoard->RemoveBoard = BOARD_Easy3111_RemoveBoard;

   return IFX_SUCCESS;
} /* BOARD_Easy3111_Register */

/*
   Detect node required by extension board EASY3111.

   \param  pCtrl  - pointer to status control structure

   \return board ID
 */
IFX_int32_t BOARD_Easy3111_DetectNode(CTRL_STATUS_t* pCtrl)
{
   TD_PTR_CHECK(pCtrl, IFX_ERROR);

   IFX_char_t pszDevNameWithPath[MAX_DEV_NAME_LEN_WITH_NUMBER];
   IFX_int32_t nLen;
   IFX_int32_t nDevNum; 
   IFX_int32_t fd = -1;

   /* Check if board should be detected */
   if (IFX_TRUE != nDetectDevice)
   {   
      /* Don't detect this board anymore */
      return NO_EXT_BOARD;
   }

   /* Max number of board is limited */
   if (nDeviceToDetect > TD_EASY3111_DEVICE_NUMBER)
   {
      nDetectDevice = IFX_FALSE;
      return NO_EXT_BOARD_DETECT;
   }
   
   /* clean up string */
   memset(pszDevNameWithPath, 0,
          sizeof(IFX_char_t) * MAX_DEV_NAME_LEN_WITH_NUMBER);

   /* set string */
   nLen = snprintf(pszDevNameWithPath, MAX_DEV_NAME_LEN_WITH_NUMBER,
                   "%s%d0", sCTRL_DEV_NAME_DXT, nDeviceToDetect);
   /* check return value */
   if (0 > nLen || MAX_DEV_NAME_LEN_WITH_NUMBER <= nLen)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Error: Unable to create string - it's too long (size: %d).\n"
             "(File: %s, line: %d)\n",
             nLen, __FILE__, __LINE__));
      return NO_EXT_BOARD_DETECT;         
   }
   /* Try to open node */ 
   fd = Common_Open(pszDevNameWithPath);
   if (fd > 0) 
   {
      nDevNum = nDeviceToDetect;
      /* Increase device number for next board */
      nDeviceToDetect++;
      TD_OS_DeviceClose(fd);
      return nDevNum;
   }

   /* Don' detect this board anymore */
   nDetectDevice = IFX_FALSE;

   return NO_EXT_BOARD_DETECT;
} /* BOARD_Easy3111_DetectNode */



