/******************************************************************************

                              Copyright (c) 2009
                            Lantiq Deutschland GmbH
                     Am Campeon 3; 85579 Neubiberg, Germany

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

  ****************************************************************************/

/**
   \file board_easy3201.c
   \date 2006-10-01
   \brief Interface implementation for easy3201 board.

   Application needs it to initialize board, prepare structures for this board
   and set mode in which board will work (using PCM, GPIO, ...).
*/

/* ============================= */
/* Includes                      */
/* ============================= */
#include "tapidemo.h"
#include "common.h"
#include "board_easy3201.h"
#include "pcm.h"


/* ============================= */
/* Defines                       */
/* ============================= */

/* ============================= */
/* Local variable definition     */
/* ============================= */


/** How channels are represented by device, socket.
    Device /dev/vin11 will get status, data from phone channel 0 and
    data channel 0. If at startup phone channel 0 is mapped to data channel 1
    then, phone will send actions to dev/vin11 and data will send actions to
    /dev/vin12.
    -1 means no channel at startup */

/** Max phones on EASY3201 (analog channels) 
    It is used only for STARTUP_MAPPING_EASY3201[] table. 
    Number of analog channels used in program is read from drivers 
    using IFX_TAPI_CAP_LIST */
enum { MAX_PHONES_EASY3201 = 2 };

/** Name of board */
static const char* BOARD_NAME_EASY3201 = "EASY3201";

/** Holds default phone channel to data, pcm channel mappings
    defined at startup. */
static const STARTUP_MAP_TABLE_t STARTUP_MAPPING_EASY3201[MAX_PHONES_EASY3201] =
{
   /* IMORTANT - for this board this is default mapping done by drivers,
      to change this mapping channels(phone and PCM) must be unmapped first */
   /** phone, data, pcm */
   {  0,  0,  0  },
   {  1,  1,  1  }
   /* others are free and can be used later for conferencing */
};


/* ============================= */
/* Global variable definition    */
/* ============================= */

/* ============================= */
/* Local function declaration    */
/* ============================= */
IFX_return_t BOARD_Easy3201_PrepareBoard(BOARD_t* pBoard, IFX_char_t* pPath);
IFX_return_t BOARD_Easy3201_PrepareNames(BOARD_t* pBoard);
IFX_return_t BOARD_Easy3201_CfgPCMInterface(BOARD_t* pBoard, 
                                            IFX_boolean_t hMaster);
IFX_return_t BOARD_Easy3201_GetCapabilities(BOARD_t* pBoard,
                                            IFX_char_t* pPath);
IFX_return_t BOARD_Easy3201_InitSystem(BOARD_t* pBoard);
IFX_return_t BOARD_Easy3201_Set_CPLD_Reg_For_PCM(IFX_boolean_t fMaster,
                                                 BOARD_t* pBoard,
                                                 IFX_boolean_t fLocalLoop);
IFX_void_t BOARD_Easy3201_RemoveBoard(BOARD_t* pBoard);
IFX_return_t BOARD_Easy3201_Init(BOARD_t* pBoard, PROGRAM_ARG_t* pProgramArg);

/* ============================= */
/* Local function definition     */
/* ============================= */

/**
   Prepare board structure (reserve memory, inicialize, ...).

   \param pBoard - pointer to Board
   \param pPath - path to FW files for download

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remark
*/
IFX_return_t BOARD_Easy3201_PrepareBoard(BOARD_t* pBoard, IFX_char_t* pPath)
{
   IFX_int32_t i = 0;
   TAPIDEMO_DEVICE_t* pDevice = IFX_NULL;
   TAPIDEMO_DEVICE_CPU_t* pCPUDevice = IFX_NULL;

   /* check input arguments */
   if (pBoard == IFX_NULL || pPath == IFX_NULL)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /* First 2 variables (id, type) set before this func is called */

   /* File descriptors not open */
   pBoard->nSystem_FD = -1;
   pBoard->nDevCtrl_FD = -1;
   /* Set flag to use system FD */
   pBoard->nUseSys_FD = IFX_TRUE;

   /* allocate memory for CPU device structure */
   pDevice = calloc(1, sizeof(TAPIDEMO_DEVICE_t));
   if (pDevice == IFX_NULL)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("ERR Allocate memory for CPU Device. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /* Init structure for CPU device and add it to the board */
   pDevice->nType = TAPIDEMO_DEV_CPU;
   pCPUDevice = &pDevice->uDevice.stCpu;
   /** Pointer to device dependent structure */
   pCPUDevice->pDevDependStuff = (IFX_void_t *) &DuslicXtDevInitStruct;
#ifdef USE_FILESYSTEM
   /** FW files */
   pCPUDevice->pszDRAM_FW_File = sDRAMFile_DXT;
   pCPUDevice->pszPRAM_FW_File[0] = sPRAMFile_DXT;
   pCPUDevice->pszPRAM_FW_File[1] = sPRAMFile_DXT_Old;
   pCPUDevice->pszBBD_CRAM_File[0] = sBBD_CRAM_File_DXT;
   pCPUDevice->pszBBD_CRAM_File[1] = sBBD_CRAM_File_DXT_Old;
#ifndef VXWORKS
   if (IFX_ERROR ==  Common_SetDownloadPath(pPath, pBoard, pCPUDevice))
   {
      /* Can't set download path */
      TD_OS_MemFree(pDevice);
      return IFX_ERROR;
   }
#endif /* VXWORKS */
#endif /* USE_FILESYSTEM */

   if (Common_AddDevice(pBoard, pDevice) == IFX_ERROR)
   {
      TD_OS_MemFree(pDevice);
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("ERR, CPU device not added to the board. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   if (DEVICE_DUSLIC_XT_Register(pCPUDevice) == IFX_ERROR)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("ERR, CPU device not registered. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   if (BOARD_Easy3201_PrepareNames(pBoard) == IFX_ERROR)
      return IFX_ERROR;

   /** Startup mapping table */
   pBoard->pChStartupMapping = STARTUP_MAPPING_EASY3201;

   /* Basic device initialization */
   if (BOARD_Easy3201_InitSystem(pBoard) == IFX_ERROR)
   {
       return IFX_ERROR;
   }

   if (IFX_ERROR == DEVICE_DUSLIC_XT_setupChannel(pBoard, pPath, IFX_FALSE))
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Err, Channel 0 is not initialized. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      TD_OS_DeviceClose(pBoard->nDevCtrl_FD);
      return IFX_ERROR;
   }

   if (IFX_ERROR == BOARD_Easy3201_GetCapabilities(pBoard, pPath))
   {
      return IFX_ERROR;
   }

   if (IFX_ERROR == Common_SetPhonesAndUsedChannlesNumber(pBoard))
   {
      return IFX_ERROR;
   }

   pBoard->nCh_FD = calloc(pBoard->nUsedChannels, sizeof(IFX_int32_t));
   if (pBoard->nMaxCoderCh > 0)
   {
      pBoard->pDataChStat = calloc(pBoard->nMaxCoderCh, sizeof(VOIP_DATA_CH_t));
      pBoard->rgnSockets = calloc(pBoard->nMaxCoderCh, sizeof(IFX_int32_t));
      if ((pBoard->pDataChStat == IFX_NULL) || (pBoard->rgnSockets == IFX_NULL))
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("ERR Allocate memory to prepare board. "
                "(File: %s, line: %d)\n",
               __FILE__, __LINE__));
         return IFX_ERROR;         
      }
   }

   pBoard->rgoPhones = calloc(pBoard->nMaxPhones, sizeof(PHONE_t));
   if ((pBoard->rgoPhones == IFX_NULL) || (pBoard->nCh_FD == IFX_NULL))
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("ERR Allocate memory to prepare board. "
             "(File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;         
   }

   for (i = 0; i < pBoard->nMaxCoderCh; i++)
   {
       pBoard->rgnSockets[i] = -1;
   }

   for (i = 0; i < pBoard->nUsedChannels; i++)
   {
       pBoard->nCh_FD[i] = -1;
   }

   /* Prepare array of free PCM channels  */
   pBoard->fPCM_ChFree = calloc(pBoard->nMaxPCM_Ch, sizeof(IFX_boolean_t));

   if (pBoard->fPCM_ChFree == IFX_NULL)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("ERR allocate memory for PCM channels. "
            "(File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;         
   }

   /* All PCM channels are free at start */
   for (i = 0; i < pBoard->nMaxPCM_Ch; i++)
   {
       pBoard->fPCM_ChFree[i] = IFX_TRUE;
   }

   if (IFX_SUCCESS != Common_Set_FDs(pBoard))
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Error setting fds. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   return IFX_SUCCESS;
}

/**
   Release board structure (reserve memory, inicalize, ...).

   \param pBoard - pointer to board

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remark Must be called before application exit.
*/
IFX_return_t BOARD_Easy3201_PrepareNames(BOARD_t* pBoard)
{
   IFX_char_t tmp_buf[10] = {0};

   /* check input arguments */
   if (pBoard == IFX_NULL)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /** Device name without numbers, like /dev/vmmc, /dev/vin, ... */
   strncpy(pBoard->pszChDevName, sCH_DEV_NAME_DXT, 
           strlen(sCH_DEV_NAME_DXT));

   /** Device control name numbers, like /dev/vmmc[board id]0, ... */
   strncpy(pBoard->pszCtrlDevName, sCTRL_DEV_NAME_DXT, 
           strlen(sCTRL_DEV_NAME_DXT));
   /*snprintf(tmp_buf, 2, "%d", (int) pBoard->nID);*/
   sprintf(tmp_buf, "%d", (int) pBoard->nID);
   strncat(pBoard->pszCtrlDevName, tmp_buf, 1);
   /*snprintf(tmp_buf, 2, "0");*/
   sprintf(tmp_buf, "0");
   strncat(pBoard->pszCtrlDevName, tmp_buf, 1);

   /** System device name, like /dev/easy3201/[<board id], ... */
   /** BUT actually we only have one */
   strncpy(pBoard->pszSystemDevName, sSYS_DEV_NAME_DXT,
           strlen(sSYS_DEV_NAME_DXT));

   return IFX_SUCCESS;
}

#ifndef STREAM_1_1
/**
   Configure PCM interface, can define clock, master/slave mode, ...

   \param pBoard  - pointer to board
   \param hMaster  - IFX_TRUE - master mode, IFX_FALSE - slave mode

   \return IFX_SUCCESS on OK, otherwise IFX_ERROR
   
   \remark for Easy3201 slave mode is used.
*/
IFX_return_t BOARD_Easy3201_CfgPCMInterface(BOARD_t* pBoard, 
                                            IFX_boolean_t hMaster)
{
   IFX_TAPI_PCM_IF_CFG_t pcm_if;
   IFX_return_t ret;

   if (IFX_NULL == pBoard)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH, 
           ("Invalid input argument(s). (File: %s, line: %d)\n", 
            __FILE__, __LINE__));
      return IFX_FALSE;
   }

   memset(&pcm_if, 0, sizeof(IFX_TAPI_PCM_IF_CFG_t));

   /* for Duslic-Xt only slave mode can be used */
   pcm_if.nOpMode = IFX_TAPI_PCM_IF_MODE_SLAVE_AUTOFREQ;

   pcm_if.nDCLFreq = IFX_TAPI_PCM_IF_DCLFREQ_2048;
   pcm_if.nDoubleClk = IFX_DISABLE;

   pcm_if.nSlopeTX = IFX_TAPI_PCM_IF_SLOPE_RISE;
   pcm_if.nSlopeRX = IFX_TAPI_PCM_IF_SLOPE_FALL;
   pcm_if.nOffsetTX = IFX_TAPI_PCM_IF_OFFSET_NONE;
   pcm_if.nOffsetRX = IFX_TAPI_PCM_IF_OFFSET_NONE;
   pcm_if.nDrive = IFX_TAPI_PCM_IF_DRIVE_ENTIRE;
   pcm_if.nShift = IFX_DISABLE;

   TRACE(TAPIDEMO, DBG_LEVEL_NORMAL, 
         ("%s: Configure PCM interface as slave, %s kHz\n",
          pBoard->pszBoardName, TAPIDEMO_PCM_FREQ_STR[pcm_if.nDCLFreq]));

   TD_ASSERT(IFX_SUCCESS == (ret = TD_IOCTL(pBoard->nDevCtrl_FD,
             IFX_TAPI_PCM_IF_CFG_SET, (IFX_int32_t) &pcm_if)),
             pBoard->nDevCtrl_FD, pBoard, 0, 0);
   if (ret != IFX_SUCCESS)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH, 
           ("Err configure PCM interface "
            "(File: %s, line: %d)\n", 
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /* set PCM frequency */
   oPCM_Data.nPCM_Rate = pcm_if.nDCLFreq;

   return IFX_SUCCESS;
} /* BOARD_Easy3201_CfgPCMInterface() */

#else /* !STREAM_1_1 */

IFX_return_t BOARD_Easy3201_CfgPCMInterface(BOARD_t* pBoard, 
                                            IFX_boolean_t hMaster)
{
   return IFX_SUCCESS;
} /* BOARD_Easy3201_CfgPCMInterface() */
#endif /* !STREAM_1_1 */

/**
   Get capabilities, basically number of phone, data and pcm channels.

   \param pBoard - pointer to board
   \param pPath - path to FW files for download

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remark 
*/
IFX_return_t BOARD_Easy3201_GetCapabilities(BOARD_t* pBoard,
                                            IFX_char_t* pPath)
{
   IFX_return_t ret = IFX_ERROR;

   if ((pBoard == IFX_NULL) || (pPath == IFX_NULL))
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   ret = Common_GetCapabilities(pBoard);
   if (ret == IFX_ERROR)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Err, Capabilities not read. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
   }

   if (pBoard->nMaxAnalogCh > MAX_PHONES_EASY3201)
   {
      /* If MAX_PHONES_EASY3201 is too small than STARTUP_MAPPING_EASY3201[] 
         table has too small size and TAPIDEMO will from wrong place */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Err, MAX_PHONES_EASY3201 is too small. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }
   return ret;
}

/**
   Basic device initialization prior to DuSLIC-XT initialization.

   \param pBoard - pointer to board

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remark
*/
IFX_return_t BOARD_Easy3201_InitSystem(BOARD_t* pBoard)
{
   IFX_uint32_t nChipNum = 0, nIrqNum;
   EASY3201_CPLD_CONFIG_t config_dxt;
   EASY3201_CPLD_GET_DEV_PARAM_t chip_param_dxt;
   TAPIDEMO_DEVICE_t *pDevice = IFX_NULL;
   TAPIDEMO_DEVICE_CPU_t   *pCpuDevice = IFX_NULL;
   IFX_char_t pszDevName[DEV_NAME_LEN];
   IFX_int32_t nDevFd;

   if (pBoard == IFX_NULL)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   if (0 >= pBoard->nDevCtrl_FD)
   {
      pBoard->nDevCtrl_FD = Common_Open(pBoard->pszCtrlDevName);
      if (0 >= pBoard->nDevCtrl_FD)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
              ("Invalid device control file descriptor %s.\n"
               "(File: %s, line: %d)\n",
               pBoard->pszCtrlDevName, __FILE__, __LINE__));
         return IFX_ERROR;
      }
   }

   if (0 >= pBoard->nSystem_FD)
   {
      /* Get system fd */
      pBoard->nSystem_FD = Common_Open(pBoard->pszSystemDevName);

      if (0 >= pBoard->nSystem_FD)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
              ("Invalid system file descriptor. (File: %s, line: %d)\n",
               __FILE__, __LINE__));
         return IFX_ERROR;
      }
   }

   if ((pDevice = Common_GetDevice_CPU(pBoard->pDevice)) == IFX_NULL)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Err, CPU device not found. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }
   pCpuDevice = &pDevice->uDevice.stCpu;


   memset(&config_dxt, 0, sizeof(config_dxt));
   config_dxt.nAccessMode = SYSTEM_AM_DEFAULT_DXT;
   config_dxt.nClkRate = TAPIDEMO_CLOCK_RATE;

   /* set system configuration, CPLD acces mode, ... */
   if (IFX_ERROR == TD_IOCTL(pBoard->nSystem_FD, FIO_EASY3201_CONFIG,
                             (IFX_int32_t) &config_dxt))
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Err with ioctl cmd FIO_EASY3201_CONFIG."
            "(File: %s, line: %d)\n", __FILE__, __LINE__));   
      return IFX_ERROR;
   }

   /* Go on with initialization, but only for chip 0 */
   if ((pBoard->nID - 1) == 0)
   {
      nChipNum = TD_IOCTL(pBoard->nSystem_FD, FIO_EASY3201_INIT, 0);
      if (nChipNum < 0)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
              ("Err with ioctl cmd FIO_EASY3201_INIT."
               "(File: %s, line: %d)\n", __FILE__, __LINE__));
         return IFX_ERROR;
      }
   }
   else
   {
      nChipNum = 1;
   }
   /* Store chip count */
   pBoard->nChipCnt = nChipNum;

   /* for all chips */
   for (nChipNum = 0; nChipNum < pBoard->nChipCnt; nChipNum++)
   {
      if (nChipNum > 0)
      {
         /* get device name to open */
         sprintf(pszDevName, "%s%d%d", sCTRL_DEV_NAME_DXT, (nChipNum +1), 0);
         nDevFd = Common_Open(pszDevName);
         if (0 >= nDevFd)
         {
            TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                 ("Invalid device control file descriptor %s.\n"
                  "(File: %s, line: %d)\n",
                  pszDevName, __FILE__, __LINE__));
            continue;
         }
      }
      /* initializing first chip */
      else
      {
         /* set file descriptor */
         nDevFd = pBoard->nDevCtrl_FD;
      }

      /* activate chip reset */
      if (BOARD_Easy3201_Reset(pBoard, nChipNum, IFX_TRUE) != IFX_SUCCESS)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
              ("ERROR: Basic Device Initialization fails on device %d"
               "(File: %s, line: %d)\n",
               nChipNum, __FILE__, __LINE__));
         return IFX_ERROR;
      }

      memset(&chip_param_dxt, 0, sizeof(EASY3201_CPLD_GET_DEV_PARAM_t));
      chip_param_dxt.nChipNum = nChipNum;

      /* read dxt device configuration parameters from board driver */
      if (TD_IOCTL(pBoard->nSystem_FD, FIO_EASY3201_GETBOARDPARAMS,
                   (IFX_int32_t) &chip_param_dxt) != IFX_SUCCESS)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("Err with ioctl cmd FIO_EASY3201_GETBOARDPARAMS."
                "(File: %s, line: %d)\n", __FILE__, __LINE__));
         return IFX_ERROR;
      }
      /* if initializing first chip */
      if (0 == nChipNum)
      {
         /* set Irq Line number */
         pCpuDevice->nIrqNum = chip_param_dxt.nIrqNum;
      }
      nIrqNum = chip_param_dxt.nIrqNum;

      if (DEVICE_DUSLIC_XT_BasicInit(nDevFd, nIrqNum) != IFX_SUCCESS)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("ERROR: Basic Device Initialization fails "
                "(File: %s, line: %d)\n",
                __FILE__, __LINE__));
         return IFX_ERROR;
      }

      /* clear reset to allow vinetic access */
      if (BOARD_Easy3201_Reset(pBoard, nChipNum, IFX_FALSE) != IFX_SUCCESS)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("ERROR: Basic Device Initialization fails on device %d"
                "(File: %s, line: %d)\n",
                nChipNum, __FILE__, __LINE__));
         return IFX_ERROR;
      }
   } /* for all chips */

   return IFX_SUCCESS;
} /* BOARD_Easy3201_InitSystem() */

/**
   Configure CPLD register for PCM initialization.

   \param fMaster - flag indicating if this board will be master (1) or
                    slave (0)
   \param pBoard - pointer to board
   \param fLocalLoop - 1 if PCM local loop is used, otherwise nothing (0).

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remark Must be called after system inicialization.
*/
IFX_return_t BOARD_Easy3201_Set_CPLD_Reg_For_PCM(IFX_boolean_t fMaster,
                                                 BOARD_t* pBoard,
                                                 IFX_boolean_t fLocalLoop)
{
   IFX_uint8_t reg2_value = 0;
   IFX_uint8_t reg1_value = 0;
   EASY3201_CPLD_REG_t reg_dxt = {0};
   IFX_return_t ret = IFX_SUCCESS;


   if (pBoard == IFX_NULL)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   if (!fLocalLoop)
   {
      memset(&reg_dxt, 0, sizeof(EASY3201_CPLD_REG_t));
   
      reg_dxt.nRegOffset = CMDREG1_OFF_DXT;
      /* Read register */
      ret = TD_IOCTL(pBoard->nSystem_FD, FIO_EASY3201_CPLDREG_READ,
                     (IFX_int32_t) &reg_dxt);
               
      if (ret == IFX_SUCCESS)
      {
         reg1_value = reg_dxt.nRegVal;
      }
      else
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
              ("Err reading CPLD register. (File: %s, line: %d)\n",
               __FILE__, __LINE__));
         return IFX_ERROR;
      }
     
      TRACE(TAPIDEMO, DBG_LEVEL_NORMAL,
           ("Read CMDREG%d 0x%x\n", 1, reg1_value));
   }

   memset(&reg_dxt, 0, sizeof(EASY3201_CPLD_REG_t));
   
   reg_dxt.nRegOffset = CMDREG2_OFF_DXT;
   /* Read register */
   ret = TD_IOCTL(pBoard->nSystem_FD, FIO_EASY3201_CPLDREG_READ,
                  (IFX_int32_t) &reg_dxt);
               
   if (ret == IFX_SUCCESS)
   {
      reg2_value = reg_dxt.nRegVal;
   }
   else
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Err reading CPLD register. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   TRACE(TAPIDEMO, DBG_LEVEL_NORMAL,
        ("Read CMDREG%d 0x%x\n", 2, reg2_value));

   if (fLocalLoop)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_LOW, ("Set PCM LOOP\n"));
      reg2_value |= PCM_LOOP;
   }
   else
   {
      if (fMaster)
      {
         reg1_value |= PCM_MASTER;
         reg2_value |= PCM_SEND_CLK;
         reg2_value |= PCMON_DXT;
      }
      else
      {
         reg1_value &= ~PCM_MASTER;
         reg2_value &= ~PCM_SEND_CLK;
         reg2_value |= PCMON_DXT;
      }
   }

   if (!fLocalLoop)
   {
      memset(&reg_dxt, 0, sizeof(EASY3201_CPLD_REG_t));
   
      reg_dxt.nRegOffset = CMDREG1_OFF_DXT;
      reg_dxt.nRegVal = reg1_value;
   
      /* Write register */
      ret = TD_IOCTL(pBoard->nSystem_FD, FIO_EASY3201_CPLDREG_WRITE,
                     (IFX_int32_t) &reg_dxt);
               
      if (ret != IFX_SUCCESS)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
              ("Err writing value 0x%0X to CPLD register. "
               "(File: %s, line: %d)\n", reg_dxt.nRegVal, __FILE__, __LINE__));
         return IFX_ERROR;
      }

      TRACE(TAPIDEMO, DBG_LEVEL_NORMAL,
           ("Write CMDREG%d 0x%x\n", 1, reg1_value));
   }

   memset(&reg_dxt, 0, sizeof(EASY3201_CPLD_REG_t));
   
   reg_dxt.nRegOffset = CMDREG2_OFF_DXT;
   reg_dxt.nRegVal = reg2_value;
   
   /* Write register */
   ret = TD_IOCTL(pBoard->nSystem_FD, FIO_EASY3201_CPLDREG_WRITE,
                  (IFX_int32_t) &reg_dxt);
               
   if (ret != IFX_SUCCESS)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Err writing value 0x%0X to CPLD register. "
            "(File: %s, line: %d)\n", reg_dxt.nRegVal, __FILE__, __LINE__));
      return IFX_ERROR;
   }

   TRACE(TAPIDEMO, DBG_LEVEL_NORMAL,
        ("Write CMDREG%d 0x%x\n", 2, reg2_value));

   return IFX_SUCCESS;
} /* BOARD_Easy3201_Set_CPLD_Reg_For_PCM() */

/* ============================= */
/* Global function definition    */
/* ============================= */
/**
   Release board structure (reserve memory, inicalize, ...).

   \param pBoard - pointer to board

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remark Must be called before application exit.
*/
IFX_void_t BOARD_Easy3201_RemoveBoard(BOARD_t* pBoard)
{
   if (pBoard->pDataChStat != IFX_NULL)
   {
      TD_OS_MemFree(pBoard->pDataChStat);
      pBoard->pDataChStat = IFX_NULL;
   }

   if (pBoard->rgoPhones != IFX_NULL)
   {
      TD_OS_MemFree(pBoard->rgoPhones);
      pBoard->rgoPhones = IFX_NULL;
   }

   if (pBoard->nCh_FD != IFX_NULL)
   {
      TD_OS_MemFree(pBoard->nCh_FD);
      pBoard->nCh_FD = IFX_NULL;
   }

   if (pBoard->rgnSockets != IFX_NULL)
   {
      TD_OS_MemFree(pBoard->rgnSockets);
      pBoard->rgnSockets = IFX_NULL;
   }

   if (pBoard->fPCM_ChFree != IFX_NULL)
   {
      TD_OS_MemFree(pBoard->fPCM_ChFree);
      pBoard->fPCM_ChFree = IFX_NULL;
   }

   Common_RemoveDeviceList(pBoard);
}

/**
   Read version after succesfull initialization and display it.

   \param pBoard - pointer to board
   \param pProgramArg - pointer to program arguments

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remark
      This must be done only after a successfull basic device initialization
*/
IFX_return_t BOARD_Easy3201_Init(BOARD_t* pBoard, PROGRAM_ARG_t* pProgramArg)
{
   IFX_boolean_t we_are_master = IFX_FALSE;
   IFX_boolean_t use_pcm_local_loop = IFX_FALSE;

    /* check input arguments */
   if (pBoard == IFX_NULL)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   if (BOARD_Easy3201_PrepareBoard(pBoard, pProgramArg->sPathToDwnldFiles)
       == IFX_ERROR)
   {
       return IFX_ERROR;
   }

   /* Initialize chip */
   if (DEVICE_DUSLIC_XT_Init(pBoard, pProgramArg->sPathToDwnldFiles) 
       == IFX_ERROR)
   {
       return IFX_ERROR;
   }

   /* Initialize PCM if used (just set values to some CPLD registers) */
   if ((pProgramArg->oArgFlags.nPCM_Master)
       || (pProgramArg->oArgFlags.nPCM_Slave))
   {
      if ((pBoard->nID - 1) == 0)
      {
         if (pProgramArg->oArgFlags.nPCM_Master)
         {
            we_are_master = IFX_TRUE;
         }

         if (pProgramArg->oArgFlags.nPCM_Loop)
         {
            use_pcm_local_loop = IFX_TRUE;
         }

         if (BOARD_Easy3201_Set_CPLD_Reg_For_PCM(we_are_master, pBoard,
                                         use_pcm_local_loop) != IFX_SUCCESS)
         {
            TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                  ("PCM initialization failed. (File: %s, line: %d)\n",
                  __FILE__, __LINE__));
            return IFX_ERROR;
         }
      }
   }

   if (BOARD_Easy3201_CfgPCMInterface(pBoard, we_are_master) == IFX_ERROR)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("PCM initialization failed. (File: %s, line: %d)\n",
             __FILE__, __LINE__));
      return IFX_ERROR;
   }

   return IFX_SUCCESS;
}

/**
   Set/Clear reset of device.

   \param pBoard - pointer to Board
   \param nArg - id of device to reset
   \param hSetReset - IFX_TRUE - set reset, IFX_FALSE - clear reset

   \return IFX_SUCCESS on ok, otherwise IFX_ERROR
*/
IFX_return_t BOARD_Easy3201_Reset(BOARD_t* pBoard,
                                  IFX_int32_t nArg,
                                  IFX_boolean_t hSetReset)
{
   EASY3201_CPLD_RESET_t reset_dxt;
   IFX_return_t ret = IFX_SUCCESS;
   IFX_int32_t nDevNum = nArg;

   /* check input arguments */
   if (pBoard == IFX_NULL)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   memset(&reset_dxt, 0, sizeof(EASY3201_CPLD_RESET_t));
      
   reset_dxt.nChipNum = nDevNum;
   if (hSetReset)
   {
      reset_dxt.nResetMode = EASY3201_CPLD_RESET_ACTIVE;
   }
   else
   {
      reset_dxt.nResetMode = EASY3201_CPLD_RESET_DEACTIVE;
   }
   ret = TD_IOCTL(pBoard->nSystem_FD, FIO_EASY3201_RESETCHIP,
                  (IFX_int32_t) &reset_dxt);
   if (ret != IFX_SUCCESS)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Err with iocl cmd FIO_EASY3201_RESETCHIP, dev no: %d, SetReset: %d"
            "(File: %s, line: %d)\n",
             nDevNum, hSetReset, __FILE__, __LINE__));
   }
   return ret;
}

/**
   Register board.

   \param pBoard - pointer to board
 
   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remarks 
*/
IFX_return_t BOARD_Easy3201_Register(BOARD_t* pBoard)
{
   TD_PTR_CHECK(pBoard, IFX_ERROR);

   pBoard->pszBoardName = BOARD_NAME_EASY3201;
   pBoard->Init = BOARD_Easy3201_Init;
   pBoard->RemoveBoard = BOARD_Easy3201_RemoveBoard;

   return IFX_SUCCESS;
}

