/******************************************************************************

                              Copyright (c) 2009
                            Lantiq Deutschland GmbH
                     Am Campeon 3; 85579 Neubiberg, Germany

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

  ****************************************************************************/

/**
   \file board_easy3332.c
   \date 2006-10-01
   \brief Interface implementation for DANUBE board.

   Application needs it to initialize board, prepare structures for this board
   and set mode in which board will work (using PCM, GPIO, ...).
*/

/* ============================= */
/* Includes                      */
/* ============================= */
#include "tapidemo.h"
#include "common.h"
#include "board_easy3332.h"


/* ============================= */
/* Defines                       */
/* ============================= */

/* ============================= */
/* Local variable definition     */
/* ============================= */


/** How channels are represented by device, socket.
    Device /dev/vin11 will get status, data from phone channel 0 and
    data channel 0. If at startup phone channel 0 is mapped to data channel 1
    then, phone will send actions to dev/vin11 and data will send actions to
    /dev/vin12.
    -1 means no channel at startup */

/** Max phones on CPE (analog channels)
    It is used only for STARTUP_MAPPING_CPE[] table. 
    Number of analog channels used in program is read from drivers 
    using IFX_TAPI_CAP_LIST */
enum { MAX_PHONES_CPE = 2 };

/** Name of board */
static const char* BOARD_NAME_EASY3332 = "EASY3332";

/** Holds default phone channel to data, pcm channel mappings
    defined at startup. */
static const STARTUP_MAP_TABLE_t STARTUP_MAPPING_CPE[MAX_PHONES_CPE] =
{
   /** phone, data, pcm */
   {  0,  0,  0  },
   {  1,  1,  1  }
   /* others are free and can be used later for conferencing */
};


/* ============================= */
/* Global variable definition    */
/* ============================= */

/* ============================= */
/* Local function declaration    */
/* ============================= */
IFX_return_t BOARD_Easy3332_PrepareBoard(BOARD_t* pBoard, IFX_char_t* pPath);
IFX_return_t BOARD_Easy3332_PrepareNames(BOARD_t* pBoard);
IFX_return_t BOARD_Easy3332_InitSystem(BOARD_t* pBoard);
IFX_return_t BOARD_Easy3332_Set_CPLD_Reg_For_PCM(IFX_boolean_t fMaster,
                                                 BOARD_t* pBoard,
                                                 IFX_boolean_t fLocalLoop);
IFX_return_t BOARD_Easy3332_GetCapabilities(BOARD_t* pBoard,
                                            IFX_char_t* pPath);
IFX_void_t BOARD_Easy3332_RemoveBoard(BOARD_t* pBoard);
IFX_return_t BOARD_Easy3332_Init(BOARD_t* pBoard, PROGRAM_ARG_t* pProgramArg);
IFX_return_t BOARD_Easy3332_Reset(BOARD_t* pBoard,
                                  IFX_int32_t nArg,
                                  IFX_boolean_t hSetReset);

/* ============================= */
/* Local function definition     */
/* ============================= */
/**
   Prepare board structure (reserve memory, inicialize, ...).

   \param pBoard - pointer to Board
   \param pPath - path to FW files for download

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remark 
*/
IFX_return_t BOARD_Easy3332_PrepareBoard(BOARD_t* pBoard, IFX_char_t* pPath)
{
   IFX_int32_t i = 0;
   TAPIDEMO_DEVICE_t* pDevice = IFX_NULL;
   TAPIDEMO_DEVICE_CPU_t* pCPUDevice = IFX_NULL;

   /* check input arguments */
   if (pBoard == IFX_NULL || pPath == IFX_NULL)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /* First 2 variables (id, type) set before this func is called */

   /* File descriptors not open */
   pBoard->nSystem_FD = -1;
   pBoard->nDevCtrl_FD = -1;
   /* Set flag to use system FD */
   pBoard->nUseSys_FD = IFX_TRUE;

   /* allocate memory for CPU device structure */
   pDevice = calloc(1, sizeof(TAPIDEMO_DEVICE_t));
   if (pDevice == IFX_NULL)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("ERR Allocate memory for CPU Device. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /* Init structure for CPU device and add it to the board */
   pDevice->nType = TAPIDEMO_DEV_CPU;
   pCPUDevice = &pDevice->uDevice.stCpu;
   /** Pointer to device dependent structure */
   pCPUDevice->pDevDependStuff = (IFX_void_t *) &VineticDevInitStruct;
#ifdef USE_FILESYSTEM
   /** FW files */
   pCPUDevice->pszDRAM_FW_File = sDRAMFile_CPE;
   pCPUDevice->pszPRAM_FW_File[0] = sPRAMFile_CPE;
   pCPUDevice->pszPRAM_FW_File[1] = sPRAMFile_CPE_Old;
   pCPUDevice->pszBBD_CRAM_File[0] = sBBD_CRAM_File_CPE;
   pCPUDevice->pszBBD_CRAM_File[1] = sBBD_CRAM_File_CPE_Old;
#ifndef VXWORKS
   if (IFX_ERROR ==  Common_SetDownloadPath(pPath, pBoard, pCPUDevice))
   {
      /* Can't set download path */
      TD_OS_MemFree(pDevice);
      return IFX_ERROR;
   }
#endif /* VXWORKS */
#endif /* USE_FILESYSTEM */

   if (Common_AddDevice(pBoard, pDevice) == IFX_ERROR)
   {
      TD_OS_MemFree(pDevice);
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("ERR, CPU device not added to the board. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   if (DEVICE_VINETIC_CPE_Register(pCPUDevice) == IFX_ERROR)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("ERR, CPU device not registered. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   if (BOARD_Easy3332_PrepareNames(pBoard) == IFX_ERROR)
      return IFX_ERROR;

   /** Startup mapping table */
   pBoard->pChStartupMapping = STARTUP_MAPPING_CPE;

   /* Basic device initialization */
   if (BOARD_Easy3332_InitSystem(pBoard) == IFX_ERROR)
   {
       return IFX_ERROR;
   }

   if (IFX_ERROR == DEVICE_VINETIC_CPE_setupChannel(pBoard, pPath, IFX_FALSE))
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Err, Channel 0 is not initialized. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      TD_OS_DeviceClose(pBoard->nDevCtrl_FD);
      return IFX_ERROR;
   }

   if (IFX_ERROR == BOARD_Easy3332_GetCapabilities(pBoard, pPath))
   {
      return IFX_ERROR;
   }

   if (IFX_ERROR == Common_SetPhonesAndUsedChannlesNumber(pBoard))
   {
      return IFX_ERROR;
   }

   if (pBoard->nMaxCoderCh > 0)
   {
      pBoard->pDataChStat = calloc(pBoard->nMaxCoderCh, sizeof(VOIP_DATA_CH_t));
      /* \todo maybe check for max ch number available and use this number */
      pBoard->nCh_FD = calloc(pBoard->nUsedChannels, sizeof(IFX_int32_t));
      pBoard->rgnSockets = calloc(pBoard->nMaxCoderCh, sizeof(IFX_int32_t));
   }

   pBoard->rgoPhones = calloc(pBoard->nMaxPhones, sizeof(PHONE_t));

   if ((pBoard->pDataChStat == IFX_NULL) || (pBoard->rgoPhones == IFX_NULL) ||
       (pBoard->nCh_FD == IFX_NULL) || (pBoard->rgnSockets == IFX_NULL))
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("ERR Allocate memory to prepare board. "
             "(File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;         
   }

   for (i = 0; i < pBoard->nMaxCoderCh; i++)
   {
       pBoard->nCh_FD[i] = -1;
       pBoard->rgnSockets[i] = -1;
   }

   /* Prepare array of free PCM channels  */
   pBoard->fPCM_ChFree = calloc(pBoard->nMaxPCM_Ch, sizeof(IFX_boolean_t));

   if (pBoard->fPCM_ChFree == IFX_NULL)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("ERR allocate memory for PCM channels. "
             "(File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;         
   }

   /* All PCM channels are free at start */
   for (i = 0; i < pBoard->nMaxPCM_Ch; i++)
   {
       pBoard->fPCM_ChFree[i] = IFX_TRUE;
   }

   if (IFX_SUCCESS != Common_Set_FDs(pBoard))
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Error setting fds. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   return IFX_SUCCESS;
}

/**
   Prepare device/channels names.

   \param pBoard - pointer to board

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remark 
*/
IFX_return_t BOARD_Easy3332_PrepareNames(BOARD_t* pBoard)
{
   IFX_char_t tmp_buf[10] = {0};

   /* check input arguments */
   if (pBoard == IFX_NULL)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /** Device name without numbers, like /dev/vmmc, /dev/vin, ... */
   strncpy(pBoard->pszChDevName, sCH_DEV_NAME_CPE, strlen(sCH_DEV_NAME_CPE));

   /** Device control name numbers, like /dev/vmmc[board id]0, ... */
   strncpy(pBoard->pszCtrlDevName, sCTRL_DEV_NAME_CPE, 
           strlen(sCTRL_DEV_NAME_CPE));
   /*snprintf(tmp_buf, 2, "%d", (int) pBoard->nID);*/
   sprintf(tmp_buf, "%d", (int) pBoard->nID);
   strncat(pBoard->pszCtrlDevName, tmp_buf, 1);
   /*snprintf(tmp_buf, 2, "0");*/
   sprintf(tmp_buf, "0");
   strncat(pBoard->pszCtrlDevName, tmp_buf, 1);

   /** System device name, like /dev/easy3332/[<board id], ... */
   strncpy(pBoard->pszSystemDevName, sSYS_DEV_NAME_CPE,
           strlen(sSYS_DEV_NAME_CPE));
   /*snprintf(tmp_buf, 2, "%d", (int) pBoard->nID - 1);*/
   sprintf(tmp_buf, "%d", (int) pBoard->nID - 1);
   strncat(pBoard->pszSystemDevName, tmp_buf, 1);

   return IFX_SUCCESS;
}

/**
   Get capabilities, basically number of phone, data and pcm channels.

   \param pBoard - pointer to board
   \param pPath - path to FW files for download

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remark 
*/
IFX_return_t BOARD_Easy3332_GetCapabilities(BOARD_t* pBoard,
                                            IFX_char_t* pPath)
{
   IFX_return_t ret = IFX_ERROR;

   if ((pBoard == IFX_NULL) || (pPath == IFX_NULL))
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   ret = Common_GetCapabilities(pBoard);
   if (ret == IFX_ERROR)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Err, Capabilities not read. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
   }

   if (pBoard->nMaxAnalogCh > MAX_PHONES_CPE)
   {
      /* If MAX_PHONES_CPE is too small than STARTUP_MAPPING_CPE[] table 
         has too small size and TAPIDEMO will from wrong place */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Err, MAX_PHONES_CPE is too small. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }
   return ret;
}

/**
   Basic device initialization prior to vinetic initialization.

   \param pBoard - pointer to board

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remark
      - In case of 8bit SPI, for the CPE system, base address  should be set to
        0x1F if not otherwise programmed by bootstrapping
      - Caller must make sure that the access mode is  supported
*/
IFX_return_t BOARD_Easy3332_InitSystem(BOARD_t* pBoard)
{
   IFX_return_t ret = IFX_SUCCESS;
   IFX_uint32_t chipnum = 0, i = 0;
   IFX_int32_t nAccessMode;
   IFX_int32_t vin_access_mode = 0;
   EASY3332_Config_t config_cpe;
   EASY3332_GetDevParam_t chip_param_cpe;
   TAPIDEMO_DEVICE_t *pDevice = IFX_NULL;
   TAPIDEMO_DEVICE_CPU_t   *pCpuDevice = IFX_NULL;


   if (pBoard == IFX_NULL)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   if (0 >= pBoard->nDevCtrl_FD)
   {
      pBoard->nDevCtrl_FD = Common_Open(pBoard->pszCtrlDevName);
      if (0 >= pBoard->nDevCtrl_FD)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
              ("Invalid device control file descriptor %s.\n"
               "(File: %s, line: %d)\n",
               pBoard->pszCtrlDevName, __FILE__, __LINE__));
         return IFX_ERROR;
      }
   }

   if (0 >= pBoard->nSystem_FD)
   {
      /* Get system fd */
      pBoard->nSystem_FD = Common_Open(pBoard->pszSystemDevName);

      if (0 >= pBoard->nSystem_FD)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
              ("Invalid system file descriptor. (File: %s, line: %d)\n",
               __FILE__, __LINE__));
         return IFX_ERROR;
      }
   }

   if ((pDevice = Common_GetDevice_CPU(pBoard->pDevice)) == IFX_NULL)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Err, CPU device not found. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }
   pCpuDevice = &pDevice->uDevice.stCpu;

   nAccessMode = SYSTEM_AM_DEFAULT_CPE;
   memset(&config_cpe, 0, sizeof(config_cpe));
   config_cpe.nAccessMode = nAccessMode;
   config_cpe.nClkRate = TAPIDEMO_CLOCK_RATE; 

   /* set system configuration, CPLD acces mode, ... */
   if (IFX_ERROR == TD_IOCTL(pBoard->nSystem_FD, FIO_EASY3332_CONFIG,
                             (IFX_int32_t) &config_cpe))
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Err with ioctl cmd FIO_EASY3332_CONFIG."
            "(File: %s, line: %d)\n", __FILE__, __LINE__));   
      return IFX_ERROR;
   }
   /* go on with initialization */
   chipnum = TD_IOCTL(pBoard->nSystem_FD, FIO_EASY3332_INIT, 0);
   if (chipnum < 0)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Err with ioctl cmd FIO_EASY3332_INIT."
            "(File: %s, line: %d)\n", __FILE__, __LINE__));         
      return IFX_ERROR;
   }
   
   /* Store chip count */
   pBoard->nChipCnt = chipnum;
   
   /* initialize each vinetic device */
   for (i = 0; i < chipnum; i++)
   {
      ret = IFX_ERROR;
      /* activate chip reset */
      if (BOARD_Easy3332_Reset(pBoard, i, IFX_TRUE) != IFX_SUCCESS)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
              ("ERROR: Basic Device Initialization fails on device %d"
               "(File: %s, line: %d)\n",
               i, __FILE__, __LINE__));
         break;
      }
   
      memset(&chip_param_cpe, 0, sizeof(EASY3332_GetDevParam_t));
      
      /* read vinetic device configuration parameters from board driver */
      if (TD_IOCTL(pBoard->nSystem_FD, FIO_EASY3332_GETBOARDPARAMS,
                   (IFX_int32_t) &chip_param_cpe) != IFX_SUCCESS)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
              ("Err with ioctl cmd FIO_EASY3332_GETBOARDPARAMS."
               "(File: %s, line: %d)\n", __FILE__, __LINE__));            
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
              ("ERROR: Basic Device Initialization fails on device %d"
               "(File: %s, line: %d)\n",
               i, __FILE__, __LINE__));
         break;
      }

      switch (nAccessMode)
      {
          case EASY3332_ACCESS_8BIT_MOTOROLA:
             vin_access_mode = VIN_ACCESS_PAR_8BIT;
          break;
          case EASY3332_ACCESS_8BIT_INTELMUX:
             vin_access_mode = VIN_ACCESS_PARINTEL_MUX8;
          break;
          case EASY3332_ACCESS_8BIT_INTELDEMUX:
             vin_access_mode = VIN_ACCESS_PARINTEL_DMUX8;
          break;
          case EASY3332_ACCESS_SPI:
             vin_access_mode = VIN_ACCESS_SPI;
          break;
          default:
             TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                   ("Unknown Physical Access Mode 0x%X."
                   "(File: %s, line: %d)\n", nAccessMode,
                   __FILE__, __LINE__));
          break;
      }

      pCpuDevice->AccessMode = vin_access_mode;
      pCpuDevice->nBaseAddress = chip_param_cpe.nBaseAddrPhy;
      pCpuDevice->nIrqNum = chip_param_cpe.nIrqNum;
   
      if (DEVICE_VINETIC_CPE_BasicInit(pBoard) != IFX_SUCCESS)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
              ("ERROR: Basic Device Initialization fails on device %d"
               "(File: %s, line: %d)\n",
               i, __FILE__, __LINE__));
         break;
      }
      
      /* clear reset to allow vinetic access */
      if (BOARD_Easy3332_Reset(pBoard, i, IFX_FALSE) != IFX_SUCCESS)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
              ("ERROR: Basic Device Initialization fails on device %d"
               "(File: %s, line: %d)\n",
               i, __FILE__, __LINE__));
         break;
      }
         
      /* this loop processed successfully */
      ret = IFX_SUCCESS;
   } /* for (i = 0; i < chipnum; i++)*/

   if (IFX_ERROR == ret)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("ERROR: Basic Device Initialization fails on device %d"
            "(File: %s, line: %d)\n",
            i, __FILE__, __LINE__));
   }

   return ret;
} /* BOARD_Easy3332_InitSystem() */

/**
   Configure CPLD register for PCM initialization.

   \param fMaster - flag indicating if this board will be master (1) or
                    slave (0)
   \param pBoard - pointer to board
   \param fLocalLoop - 1 if PCM local loop is used, otherwise nothing (0).

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remark Must be called after system inicialization.
*/
IFX_return_t BOARD_Easy3332_Set_CPLD_Reg_For_PCM(IFX_boolean_t fMaster,
                                                 BOARD_t* pBoard,
                                                 IFX_boolean_t fLocalLoop)
{
   IFX_uint8_t reg1_value = 0;
   EASY3332_Reg_t reg_cpe;
   IFX_return_t ret = IFX_SUCCESS;


   if (pBoard == IFX_NULL)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   memset(&reg_cpe, 0, sizeof(EASY3332_Reg_t));
   
   reg_cpe.nRegOffset = CMDREG1_OFF_CPE;
   /* Read register */
   ret = TD_IOCTL(pBoard->nSystem_FD, FIO_EASY3332_CPLDREG_READ,
                  (IFX_int32_t) &reg_cpe);
   if (ret == IFX_SUCCESS)
   {
      reg1_value = reg_cpe.nRegVal;
   }
   else
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Err reading CPLD register. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   TRACE(TAPIDEMO, DBG_LEVEL_NORMAL,
        ("Read CMDREG%d 0x%x\n", 1, reg1_value));

   if (IFX_TRUE == fMaster)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_LOW, 
            ("Board mode set to PCM MASTER mode\n"));
      reg1_value |= MASTER_CPE;
      reg1_value |= PCMEN_MASTER_CPE;
      reg1_value |= PCMON_CPE;
   }
   else
   {
      TRACE(TAPIDEMO, DBG_LEVEL_LOW, 
            ("Board mode set to PCM SLAVE mode\n"));
      reg1_value &= ~MASTER_CPE;
      reg1_value &= ~PCMEN_MASTER_CPE;
      reg1_value |= PCMON_CPE;
   }

   memset(&reg_cpe, 0, sizeof(EASY3332_Reg_t));
   
   reg_cpe.nRegOffset = CMDREG1_OFF_CPE;
   reg_cpe.nRegVal = reg1_value;
   /* Read register */
   ret = TD_IOCTL(pBoard->nSystem_FD, FIO_EASY3332_CPLDREG_WRITE,
                  (IFX_int32_t) &reg_cpe);
   if (ret != IFX_SUCCESS)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Err writing value 0x%0X to CPLD register. "
            "(File: %s, line: %d)\n", reg_cpe.nRegVal, __FILE__, __LINE__));
      return IFX_ERROR;
   }

   TRACE(TAPIDEMO, DBG_LEVEL_NORMAL,
        ("Write CMDREG%d 0x%x\n", 1, reg1_value));

   return IFX_SUCCESS;
} /* BVINETIC_Set_CPLD_Reg_For_PCM() */

/* ============================= */
/* Global function definition    */
/* ============================= */
/**
   Release board structure (reserve memory, inicalize, ...).

   \param pBoard - pointer to board

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remark Must be called before application exit.
*/
IFX_void_t BOARD_Easy3332_RemoveBoard(BOARD_t* pBoard)
{
   if (pBoard->pDataChStat != IFX_NULL)
   {
      TD_OS_MemFree(pBoard->pDataChStat);
      pBoard->pDataChStat = IFX_NULL;
   }

   if (pBoard->rgoPhones != IFX_NULL)
   {
      TD_OS_MemFree(pBoard->rgoPhones);
      pBoard->rgoPhones = IFX_NULL;
   }

   if (pBoard->nCh_FD != IFX_NULL)
   {
      TD_OS_MemFree(pBoard->nCh_FD);
      pBoard->nCh_FD = IFX_NULL;
   }

   if (pBoard->rgnSockets != IFX_NULL)
   {
      TD_OS_MemFree(pBoard->rgnSockets);
      pBoard->rgnSockets = IFX_NULL;
   }

   if (pBoard->fPCM_ChFree != IFX_NULL)
   {
      TD_OS_MemFree(pBoard->fPCM_ChFree);
      pBoard->fPCM_ChFree = IFX_NULL;
   }

   Common_RemoveDeviceList(pBoard);
}

/**
   Init board.

   \param pBoard - pointer to board
   \param pProgramArg - pointer to program arguments

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remark
      This must be done only after a successfull basic device initialization
*/
IFX_return_t BOARD_Easy3332_Init(BOARD_t* pBoard, PROGRAM_ARG_t* pProgramArg)
{
   IFX_boolean_t we_are_master = IFX_FALSE;
   IFX_boolean_t use_pcm_local_loop = IFX_FALSE;

    /* check input arguments */
   if (pBoard == IFX_NULL)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   if (BOARD_Easy3332_PrepareBoard(pBoard, pProgramArg->sPathToDwnldFiles)
       == IFX_ERROR)
   {
       return IFX_ERROR;
   }

   /* Initialize chip */
   if (DEVICE_VINETIC_CPE_Init(pBoard, pProgramArg->sPathToDwnldFiles) 
       == IFX_ERROR)
   {
       return IFX_ERROR;
   }

   /* Initialize PCM if used (just set values to some CPLD registers) */
   if ((pProgramArg->oArgFlags.nPCM_Master)
       || (pProgramArg->oArgFlags.nPCM_Slave))
   {
      if ((pBoard->nID - 1) == 0)
      {
         if (pProgramArg->oArgFlags.nPCM_Master)
         {
            we_are_master = IFX_TRUE;
         }

         if (pProgramArg->oArgFlags.nPCM_Loop)
         {
            use_pcm_local_loop = IFX_TRUE;
         }

         if (BOARD_Easy3332_Set_CPLD_Reg_For_PCM(we_are_master, pBoard,
                                         use_pcm_local_loop) != IFX_SUCCESS)
         {
            TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                  ("PCM initialization failed. (File: %s, line: %d)\n",
                  __FILE__, __LINE__));
            return IFX_ERROR;
         }
      }
   }
   return IFX_SUCCESS;
}

/**
   Set/Clear reset of device.

   \param pBoard - pointer to Board
   \param nArg - id of device to reset
   \param hSetReset - IFX_TRUE - set reset, IFX_FALSE - clear reset

   \return IFX_SUCCESS on ok, otherwise IFX_ERROR
*/
IFX_return_t BOARD_Easy3332_Reset(BOARD_t* pBoard,
                                  IFX_int32_t nArg,
                                  IFX_boolean_t hSetReset)
{
   EASY3332_Reset_t reset_cpe;
   IFX_return_t ret = IFX_SUCCESS;
   IFX_int32_t nDevNum = nArg;

   /* check input arguments */
   if (pBoard == IFX_NULL)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   memset(&reset_cpe, 0, sizeof(EASY3332_Reset_t));
      
   reset_cpe.nChipNum = nDevNum;
   if (hSetReset)
   {
      reset_cpe.nResetMode = EASY3332_RESET_ACTIVE;
   }
   else
   {
      reset_cpe.nResetMode = EASY3332_RESET_DEACTIVE;
   }
   ret = TD_IOCTL(pBoard->nSystem_FD, FIO_EASY3332_RESETCHIP,
                  (IFX_int32_t) &reset_cpe);
   if (ret != IFX_SUCCESS)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Err with iocl cmd FIO_EASY3332_RESETCHIP, dev no: %d, SetReset: %d"
            "(File: %s, line: %d)\n",
             nDevNum, hSetReset, __FILE__, __LINE__));
   }
   return ret;
}

/**
   Register board.

   \param pBoard - pointer to board
 
   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remarks 
*/
IFX_return_t BOARD_Easy3332_Register(BOARD_t* pBoard)
{
   TD_PTR_CHECK(pBoard, IFX_ERROR);

   pBoard->pszBoardName = BOARD_NAME_EASY3332;
   pBoard->Init = BOARD_Easy3332_Init;
   pBoard->RemoveBoard = BOARD_Easy3332_RemoveBoard;

   return IFX_SUCCESS;
}

