/******************************************************************************

                              Copyright (c) 2009
                            Lantiq Deutschland GmbH
                     Am Campeon 3; 85579 Neubiberg, Germany

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

  ****************************************************************************/

/**
   \file board_easy80910.c
   \date 2009-09-25
   \brief Interface implementation for EASY80910 boards.

   Application needs it to initialize board, prepare structures for this board
   and set mode in which board will work (using PCM, GPIO, ...).
*/

/* ============================= */
/* Includes                      */
/* ============================= */
#include "tapidemo.h"
#include "common.h"
#if (defined(TD_FAX_MODEM) && defined(TD_FAX_T38) && defined(HAVE_T38_IN_FW))
   #include "td_t38.h"
#endif /* (defined(TD_FAX_MODEM) && defined(TD_FAX_T38) && defined(HAVE_T38_IN_FW)) */
#include "board_easy80910.h"
#ifdef FXO
   #include "common_fxo.h"
#endif /* FXO */
#include "pcm.h"


#ifdef TD_DECT
#include "td_dect.h"
#endif /* TD_DECT */

/* ============================= */
/* Defines                       */
/* ============================= */

/* ============================= */
/* Local variable definition     */
/* ============================= */


/** How channels are represented by device, socket.
    Device /dev/vmmc11 will get status, data from phone channel 0 and
    data channel 0. If at startup phone channel 0 is mapped to data channel 1
    then, phone will send actions to dev/vmmc11 and data will send actions to
    /dev/vmmc12.
    -1 means no channel at startup */

/** Max phones on VMMC (analog channels)
    It is used only for STARTUP_MAPPING_VMMC[] table. 
    Number of analog channels used in program is read from drivers 
    using IFX_TAPI_CAP_LIST */
enum { MAX_PHONES_VMMC = 2 };

/** Number of PCM highways */
enum { PCM_HIGHWAYS = 1 };

/** Name of board */
static const char* BOARD_NAME_EASY80910 = "EASY80910";

/** Holds default phone channel to data, pcm channel mappings
    defined at startup. */
static const STARTUP_MAP_TABLE_t STARTUP_MAPPING_VMMC[MAX_PHONES_VMMC] =
{
   /** phone, data, pcm */
   {  0,  0,  0 },
   {  1,  1,  1 }
};

/* ============================= */
/* Global variable definition    */
/* ============================= */

/* ============================= */
/* Local function declaration    */
/* ============================= */
IFX_return_t BOARD_Easy80910_PrepareBoard(BOARD_t* pBoard, IFX_char_t* pPath);
IFX_return_t BOARD_Easy80910_PrepareNames(BOARD_t* pBoard);
IFX_return_t BOARD_Easy80910_CfgPCMInterface(BOARD_t* pBoard, 
                                             IFX_boolean_t hMaster);
IFX_return_t BOARD_Easy80910_GetCapabilities(BOARD_t* pBoard,
                                             IFX_char_t* pPath);
IFX_void_t BOARD_Easy80910_RemoveBoard(BOARD_t* pBoard);
IFX_return_t BOARD_Easy80910_Init(BOARD_t* pBoard, PROGRAM_ARG_t* pProgramArg);

/* ============================= */
/* Local function definition     */
/* ============================= */
/**
   Prepare board structure (reserve memory, inicialize, ...).

   \param pBoard - pointer to Board

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remark Must be called after system inicialization.
*/
IFX_return_t BOARD_Easy80910_PrepareBoard(BOARD_t* pBoard, IFX_char_t* pPath)
{
   IFX_int32_t i = 0;
   TAPIDEMO_DEVICE_t* pDevice = IFX_NULL;
   TAPIDEMO_DEVICE_CPU_t* pCPUDevice = IFX_NULL;

   /* check input arguments */
   TD_PTR_CHECK(pBoard, IFX_ERROR);
   TD_PTR_CHECK(pPath, IFX_ERROR);

   /* First three variables (id, type, sub type) set before this func is called */

   /* File descriptors not open */
   pBoard->nSystem_FD = -1;
   pBoard->nDevCtrl_FD = -1;
   pBoard->nUseSys_FD = IFX_FALSE;

   /* allocate memory for CPU device structure */
   pDevice = calloc(1, sizeof(TAPIDEMO_DEVICE_t));
   if (pDevice == IFX_NULL)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("ERR Allocate memory for CPU Device. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /* Init structure for CPU device and add it to the board */
   pDevice->nType = TAPIDEMO_DEV_CPU;
   pCPUDevice = &pDevice->uDevice.stCpu;
   /** Pointer to device dependent structure */
   pCPUDevice->pDevDependStuff = (IFX_void_t *) &VmmcDevInitStruct;
#ifdef USE_FILESYSTEM
   /** FW files */
   pCPUDevice->pszDRAM_FW_File = sDRAMFile_VMMC;
   pCPUDevice->pszPRAM_FW_File[0] = sPRAMFile_VMMC;
   pCPUDevice->pszPRAM_FW_File[1] = sPRAMFile_VMMC_Old;
   pCPUDevice->pszBBD_CRAM_File[0] = sBBD_CRAM_File_VMMC;
   pCPUDevice->pszBBD_CRAM_File[1] = sBBD_CRAM_File_VMMC_Old;
   pBoard->fBBD_Changed = IFX_FALSE;

#ifndef VXWORKS
   if (IFX_ERROR ==  Common_SetDownloadPath(pPath, pBoard, pCPUDevice))
   {
      /* Can't set download path */
      TD_OS_MemFree(pDevice);
      return IFX_ERROR;
   }
#endif /* VXWORKS */
#endif /* USE_FILESYSTEM */

   if (Common_AddDevice(pBoard, pDevice) == IFX_ERROR)
   {
      TD_OS_MemFree(pDevice);
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("ERR, CPU device not added to the board. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }


   if (DEVICE_Vmmc_Register(pCPUDevice) == IFX_ERROR)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("ERR, CPU device not registered. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   if (BOARD_Easy80910_PrepareNames(pBoard) == IFX_ERROR)
      return IFX_ERROR;

   /** Startup mapping table */
   pBoard->pChStartupMapping = STARTUP_MAPPING_VMMC;

   if (IFX_ERROR == BOARD_Easy80910_GetCapabilities(pBoard, pPath))
   {
      return IFX_ERROR;
   }

   if (IFX_ERROR == Common_SetPhonesAndUsedChannlesNumber(pBoard))
   {
      return IFX_ERROR;
   }

   if (pBoard->nMaxCoderCh > 0)
   {
      pBoard->pDataChStat = calloc(pBoard->nMaxCoderCh, sizeof(VOIP_DATA_CH_t));
      pBoard->nCh_FD = calloc(pBoard->nUsedChannels, sizeof(IFX_int32_t));
      pBoard->rgnSockets = calloc(pBoard->nMaxCoderCh, sizeof(IFX_int32_t));
   } /* if (pBoard->nMaxCoderCh > 0) */

   pBoard->rgoPhones = calloc(pBoard->nMaxPhones, sizeof(PHONE_t));

   if ((pBoard->pDataChStat == IFX_NULL) || (pBoard->nCh_FD == IFX_NULL)
       || (pBoard->rgnSockets == IFX_NULL) || (pBoard->rgoPhones == IFX_NULL))
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("ERR Allocate memory to prepare board. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   for (i = 0; i < pBoard->nMaxCoderCh; i++)
   {
      pBoard->nCh_FD[i] = -1;
      pBoard->rgnSockets[i] = -1;
   }

   /* Prepare array of free PCM channels */
   pBoard->fPCM_ChFree = calloc(pBoard->nMaxPCM_Ch, sizeof(IFX_boolean_t));

   if (pBoard->fPCM_ChFree == IFX_NULL)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("ERR Allocate memory for PCM channels. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /* All PCM channels are free at start */
   for (i = 0; i < pBoard->nMaxPCM_Ch; i++)
   {
       pBoard->fPCM_ChFree[i] = IFX_TRUE;
   }

   /* Store chip count */
   pBoard->nChipCnt = 1;

   /* Sets file descriptors */
   if (IFX_SUCCESS != Common_Set_FDs(pBoard))
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Error setting fds. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   pBoard->pChCustomMapping = IFX_NULL;

   COM_MOD_VERIFY_SYSTEM_INIT_PREPARE(pBoard->pCtrl);

   return IFX_SUCCESS;
}

/**
   Prepare device/channels names.

   \param pBoard - pointer to board

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remark 
*/
IFX_return_t BOARD_Easy80910_PrepareNames(BOARD_t* pBoard)
{
   IFX_char_t tmp_buf[10] = {0};

   /* check input arguments */
   TD_PTR_CHECK(pBoard, IFX_ERROR);

   /** Device name without numbers, like /dev/vmmc, /dev/vin, ... */
   strncpy(pBoard->pszChDevName, sCH_DEV_NAME_VMMC, (DEV_NAME_LEN - 1));

   /** Device control name numbers, like /dev/vmmc[board id]0, ... */
   strncpy(pBoard->pszCtrlDevName, sCTRL_DEV_NAME_VMMC, (DEV_NAME_LEN - 1));
   snprintf(tmp_buf, (10 -1), "%d0", (int) pBoard->nID);

   if (DEV_NAME_LEN > (strlen(tmp_buf) + strlen(pBoard->pszCtrlDevName)))
   {
      strncat(pBoard->pszCtrlDevName, tmp_buf, strlen(tmp_buf));
   }
   else
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Err, CtrlDev name combined of strings: %s and %s is too long. "
            "(File: %s, line: %d)\n",
             pBoard->pszCtrlDevName, tmp_buf, __FILE__, __LINE__));
   }

   /** System device name, like /dev/easy3332/[board id], ... */
   strncpy(pBoard->pszSystemDevName, sSYS_DEV_NAME_VMMC, (DEV_NAME_LEN - 1));
   memset(tmp_buf, 0, 10);
   snprintf(tmp_buf, (10 -1), "%d", (int) pBoard->nID - 1);
   if (DEV_NAME_LEN > (strlen(tmp_buf) + strlen(pBoard->pszSystemDevName)))
   {
      strncat(pBoard->pszSystemDevName, tmp_buf, strlen(tmp_buf));
   }
   else
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Err, SystemDev name combined of strings: %s and %s is too long. "
             "(File: %s, line: %d)\n",
             pBoard->pszSystemDevName, tmp_buf, __FILE__, __LINE__));
   }

   return IFX_SUCCESS;
}

/**
   Get capabilities, basically number of phone, data and pcm channels.

   \param pBoard - pointer to board
   \param pPath - path to FW files for download

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remark 
*/
IFX_return_t BOARD_Easy80910_GetCapabilities(BOARD_t* pBoard,
                                             IFX_char_t* pPath)
{
   IFX_return_t ret = IFX_ERROR;

   if ((pBoard == IFX_NULL) || (pPath == IFX_NULL))
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   pBoard->nDevCtrl_FD = Common_Open(pBoard->pszCtrlDevName);
   if (0 >= pBoard->nDevCtrl_FD)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid device control file descriptor %s.\n"
            "(File: %s, line: %d)\n",
            pBoard->pszCtrlDevName, __FILE__, __LINE__));
      return IFX_ERROR;
   }

   ret = DEVICE_Vmmc_setupChannel(pBoard, pPath);
   if (ret == IFX_ERROR)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Err, Channel 0 is not initialized. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      TD_OS_DeviceClose(pBoard->nDevCtrl_FD);
      return IFX_ERROR;
   }

   ret = Common_GetCapabilities(pBoard);
   if (ret == IFX_ERROR)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Err, Capabilities not read. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
   }

   TD_OS_DeviceClose(pBoard->nDevCtrl_FD);
   pBoard->nDevCtrl_FD = -1;
   if (pBoard->nMaxAnalogCh > MAX_PHONES_VMMC)
   {
      /* If MAX_PHONES_VMMC is too small than STARTUP_MAPPING_VMMC[] table 
         has too small size and TAPIDEMO will from wrong place */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Err, MAX_PHONES_VMMC is too small. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }
   return ret;
}

#ifndef STREAM_1_1
/**
   Configure PCM interface, can define clock, master/slave mode, ...

   \param pBoard  - pointer to board
   \param hMaster  - IFX_TRUE - master mode, IFX_FALSE - slave mode

   \return IFX_SUCCESS on OK, otherwise IFX_ERROR
   
   \remark Should be also used to enable clock if extension board will be used.
*/
IFX_return_t BOARD_Easy80910_CfgPCMInterface(BOARD_t* pBoard, 
                                             IFX_boolean_t hMaster)
{
   IFX_int32_t j;
   IFX_TAPI_PCM_IF_CFG_t pcm_if;
   IFX_return_t ret;

   if (IFX_NULL == pBoard)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH, 
           ("Invalid input argument(s). (File: %s, line: %d)\n", 
            __FILE__, __LINE__));
      return IFX_FALSE;
   }

   memset(&pcm_if, 0, sizeof(IFX_TAPI_PCM_IF_CFG_t));
   if (hMaster)
   {
      pcm_if.nOpMode = IFX_TAPI_PCM_IF_MODE_MASTER;
   }
   else
   {
      pcm_if.nOpMode = IFX_TAPI_PCM_IF_MODE_SLAVE_AUTOFREQ;
   }
   pcm_if.nDCLFreq = IFX_TAPI_PCM_IF_DCLFREQ_2048;
   pcm_if.nDoubleClk = IFX_DISABLE;

   pcm_if.nSlopeTX = IFX_TAPI_PCM_IF_SLOPE_RISE;
   pcm_if.nSlopeRX = IFX_TAPI_PCM_IF_SLOPE_FALL;
   pcm_if.nOffsetTX = IFX_TAPI_PCM_IF_OFFSET_NONE;
   pcm_if.nOffsetRX = IFX_TAPI_PCM_IF_OFFSET_NONE;
   pcm_if.nDrive = IFX_TAPI_PCM_IF_DRIVE_ENTIRE;
   pcm_if.nShift = IFX_DISABLE;

   TRACE(TAPIDEMO, DBG_LEVEL_NORMAL, 
        ("%s: Configure PCM interface as %s, %s kHz\n",
         pBoard->pszBoardName,
         (hMaster ? "master":"slave"), TAPIDEMO_PCM_FREQ_STR[pcm_if.nDCLFreq]));
   
   
   for (j = 0; j < PCM_HIGHWAYS; j++)
   {
      TD_ASSERT(IFX_SUCCESS == (ret = TD_IOCTL(pBoard->nDevCtrl_FD,
         IFX_TAPI_PCM_IF_CFG_SET, (IFX_int32_t) &pcm_if)), pBoard->nDevCtrl_FD,
         pBoard, 0, 0);
      if (ret != IFX_SUCCESS)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH, 
              ("Err configure PCM interface "
               "(File: %s, line: %d)\n", 
               __FILE__, __LINE__));
         return IFX_ERROR;
      }
   } /* for */

   /* set PCM frequency */
   oPCM_Data.nPCM_Rate = pcm_if.nDCLFreq;

   return IFX_SUCCESS;
} /* BOARD_Easy80910_CfgPCMInterface() */

#else /* !STREAM_1_1 */

IFX_return_t BOARD_Easy80910_CfgPCMInterface(BOARD_t* pBoard, 
                                             IFX_boolean_t hMaster)
{
   return IFX_SUCCESS;
} /* PCM_Init() */
#endif /* !STREAM_1_1 */


/**
   Release board structure (reserve memory, inicalize, ...).

   \param pBoard - pointer to board

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remark Must be called before application exit.
*/
IFX_void_t BOARD_Easy80910_RemoveBoard(BOARD_t* pBoard)
{
   if (pBoard->pDataChStat != IFX_NULL)
   {
      TD_OS_MemFree(pBoard->pDataChStat);
      pBoard->pDataChStat = IFX_NULL;
   }

   if (pBoard->rgoPhones != IFX_NULL)
   {
      TD_OS_MemFree(pBoard->rgoPhones);
      pBoard->rgoPhones = IFX_NULL;
   }

   if (pBoard->nCh_FD != IFX_NULL)
   {
      TD_OS_MemFree(pBoard->nCh_FD);
      pBoard->nCh_FD = IFX_NULL;
   }

   if (pBoard->rgnSockets != IFX_NULL)
   {
      TD_OS_MemFree(pBoard->rgnSockets);
      pBoard->rgnSockets = IFX_NULL;
   }

   if (pBoard->fPCM_ChFree != IFX_NULL)
   {
      TD_OS_MemFree(pBoard->fPCM_ChFree);
      pBoard->fPCM_ChFree = IFX_NULL;
   }

   Common_RemoveDeviceList(pBoard);
}

/**
   Init board.

   \param pBoard - pointer to board
   \param pProgramArg - pointer to program arguments

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remark
*/
IFX_return_t BOARD_Easy80910_Init(BOARD_t* pBoard, PROGRAM_ARG_t* pProgramArg)
{
    /* check input arguments */
   TD_PTR_CHECK(pBoard, IFX_ERROR);
   TD_PTR_CHECK(pProgramArg, IFX_ERROR);

   if (BOARD_Easy80910_PrepareBoard(pBoard, pProgramArg->sPathToDwnldFiles) == IFX_ERROR)
   {
       return IFX_ERROR;
   }

   /* Initialize chip */
   if (DEVICE_Vmmc_Init(pBoard, pProgramArg->sPathToDwnldFiles) == IFX_ERROR)
   {
       return IFX_ERROR;
   }

   /* Initialize the PCM Interface. Must do this, otherwise access to 
      extension board or FXO will not work. */
   if (BOARD_Easy80910_CfgPCMInterface(pBoard, 
                                       pProgramArg->oArgFlags.nPCM_Master) == IFX_ERROR)
   {
       return IFX_ERROR;
   }

#if 0
   /* is T.38 stack supported on VR9? */
#if (defined(TD_FAX_MODEM) && defined(TD_FAX_T38) && defined(HAVE_T38_IN_FW))
   if (pBoard->nT38_Support)
   {
      /** Get capabilities of the T.38 stack implementation. It is enough to read 
          capabilities only from one channel because they are the same for 
          all channels. */
      if (TD_T38_Init(pBoard) == IFX_ERROR)
         return IFX_ERROR;
      TRACE(TAPIDEMO, DBG_LEVEL_NORMAL, ("TAPI Fax T.38 Stack capabilities read successfully.\n"));
   }
#endif /* (defined(TD_FAX_MODEM) && defined(TD_FAX_T38)  && defined(HAVE_T38_IN_FW)) */
#endif

#ifdef TD_DECT
   /* check if DECT support is on */
   if (!pProgramArg->oArgFlags.nNoDect)
   {
      if (IFX_SUCCESS != TD_DECT_Init(pBoard->pCtrl, pBoard))
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("Err, TD_DECT_Init failed. (File: %s, line: %d)\n",
               __FILE__, __LINE__));
         return IFX_ERROR;
      }
      TD_DECT_CurrTimerExpirySelect(pBoard->pCtrl);
   }
#endif /* TD_DECT */
#ifdef FXO
   /* FXO */
   if (pProgramArg->oArgFlags.nFXO)
   {
      if (IFX_SUCCESS != FXO_Setup(pBoard))
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("Error FXO setup. (File: %s, line: %d)\n",
               __FILE__, __LINE__));
         return IFX_ERROR;
      }
   }
#endif /* FXO */

   return IFX_SUCCESS;
}

/* ============================= */
/* Global function definition    */
/* ============================= */

/**
   Register board.

   \param pBoard - pointer to board
 
   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remarks 
*/
IFX_return_t BOARD_Easy80910_Register(BOARD_t* pBoard)
{
   TD_PTR_CHECK(pBoard, IFX_ERROR);

   pBoard->pszBoardName = BOARD_NAME_EASY80910;
   pBoard->Init = BOARD_Easy80910_Init;
   pBoard->RemoveBoard = BOARD_Easy80910_RemoveBoard;

   return IFX_SUCCESS;
}


