/******************************************************************************

                              Copyright (c) 2009
                            Lantiq Deutschland GmbH
                     Am Campeon 3; 85579 Neubiberg, Germany

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

  ****************************************************************************/

/**
   \file conference.c
   \date 2005-12-01
   \brief Conferencing support for application.

   Implements methods to add new peer to existing connection by mapping
   new channels with used one. 
   Those channels must be unmapped when conference ends.
*/

/* ============================= */
/* Includes                      */
/* ============================= */

#include "ifx_types.h"
#include "tapidemo.h"
#include "conference.h"
#include "pcm.h"
#include "voip.h"
#include "analog.h"
#include "qos.h"
#include "state_trans.h"
#include "abstract.h"
#ifdef LINUX
#include "tapidemo_config.h"
#endif

#ifdef EASY336
#include "board_easy336.h"
#include "lib_svip.h"
#endif

/* ============================= */
/* Local structures              */
/* ============================= */


/* ============================= */
/* Global Structures             */
/* ============================= */


/* ============================= */
/* Global function declaration   */
/* ============================= */


/* ============================= */
/* Local variable definition     */
/* ============================= */


/* ============================= */
/* Local function declaration    */
/* ============================= */


/* ============================= */
/* Local function definition     */
/* ============================= */


/* ============================= */
/* Global function definition    */
/* ============================= */


/**
   Put phone on hold, because call to another phone will be made.

   \todo Implement it.

   \return IFX_SUCCESS if ok, otherwise IFX_ERROR
*/
IFX_return_t CONFERENCE_PutOnHold(IFX_void_t)
{
   return IFX_SUCCESS;
} /* CONFERENCE_PutOnHold() */

/**
   Resume phone, which is on hold.

   \todo Implement it.

   \return IFX_SUCCESS if ok, otherwise IFX_ERROR
*/
IFX_return_t CONFERENCE_ResumeOnHold(IFX_void_t)
{
   return IFX_SUCCESS;
} /* CONFERENCE_ResumeOnHold() */


/**
   Configure LEC and turn it on or off for analog phone or for PCM.

   \todo Implement it.

   \return status (IFX_SUCCESS or IFX_ERROR)
*/
IFX_return_t CONFERENCE_Set_LEC(IFX_void_t)
{
   return IFX_SUCCESS;
} /* CONFERENCE_Set_LEC() */


/**
   Search for first free conference and return index of it.

   \param pCtrl  - handle to connection control structure

   \return conference index if free found, otherwise NO_CONFERENCE

*/
IFX_int32_t CONFERENCE_GetIdx(CTRL_STATUS_t* pCtrl)
{
   IFX_int32_t i = 0;
   IFX_int32_t nConfIdx = NO_CONFERENCE;


   if (IFX_NULL == pCtrl)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH, 
           ("Invalid input argument(s). (File: %s, line: %d)\n", 
            __FILE__, __LINE__));
      return nConfIdx;
   }
   for (i = 0; i < MAX_CONFERENCES; i++)
   {
      if (IFX_FALSE == pCtrl->rgoConferences[i].fActive)
      {
         nConfIdx = i + 1;
         break;
      }
   }
   if (NO_CONFERENCE == nConfIdx)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH, 
           ("Reached maximum conferences that are possible. "
            "MAX conferences is %d. (File: %s, line: %d)\n", 
            MAX_CONFERENCES, __FILE__, __LINE__));
   }
   else
   {
      pCtrl->rgoConferences[nConfIdx - 1].fActive = IFX_TRUE;
      pCtrl->nConfCnt++;
   }

   return nConfIdx;
} /* CONFERENCE_GetIdx() */


/**
   Release chosen conference index and clear CONFERENCE_t structure content.

   \param pConfIdx - pointer to conference index of phone
   \param pCtrl  - handle to connection control structure

   \return IFX_ERROR if an failed, otherwise IFX_SUCCESS

*/
IFX_return_t CONFERENCE_ReleaseIdx(IFX_uint32_t* pConfIdx, CTRL_STATUS_t* pCtrl)
{
   if (IFX_NULL == pCtrl)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH, 
           ("Invalid input argument(s). (File: %s, line: %d)\n", 
            __FILE__, __LINE__));
      return IFX_ERROR;
   }
   /* check if conference index is out of range */
   if ((NO_CONFERENCE >= *pConfIdx) || (MAX_CONFERENCES < *pConfIdx))
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH, 
           ("Conference Idx out of range - %d. (File: %s, line: %d)\n", 
            *pConfIdx, __FILE__, __LINE__));
      return IFX_ERROR;
   }
   /* check if conference is in use */
   if (IFX_FALSE == pCtrl->rgoConferences[*pConfIdx - 1].fActive)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH, 
           ("Conference with Idx %d already released. (File: %s, line: %d)\n", 
            *pConfIdx, __FILE__, __LINE__));
      return IFX_ERROR;
   }
   else
   {
      /* release conference Idx it's, also done by memset */
      pCtrl->rgoConferences[*pConfIdx - 1].fActive = IFX_FALSE;      
      /* Clear conference content. */
      memset(&pCtrl->rgoConferences[*pConfIdx - 1], 0, sizeof(CONFERENCE_t));
      /* set conference index to NO_CONFERENCE */
      *pConfIdx = NO_CONFERENCE;
      /* one less conference is used */
      pCtrl->nConfCnt--;
   }

   return IFX_SUCCESS;
} /* CONFERENCE_ReleaseIdx() */


/**
   Start conference, put this PHONE into new state.

   \param pCtrl  - handle to connection control structure
   \param pPhone - pointer to PHONE

   \return IFX_ERROR if an failed, otherwise IFX_SUCCESS

   \todo when on hold will be implemented, new button, for example * and then 
         all can talk and listen.
*/
IFX_return_t CONFERENCE_Start(CTRL_STATUS_t* pCtrl, PHONE_t* pPhone)
{
   IFX_return_t ret = IFX_ERROR;
   CONFERENCE_t* pConf;

   if ((IFX_NULL == pCtrl) || (IFX_NULL == pPhone))
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH, 
           ("Invalid input argument(s). (File: %s, line: %d)\n", 
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /* Can start conference only if already in connection with one 
      party and we started call andwe are not in conference yet. */
   if ((1 == pPhone->nConnCnt) &&
       (IFX_TRUE == pPhone->rgoConn[0].fActive) &&
       (CALL_DIRECTION_TX == pPhone->nCallDirection))
   {
      if (NO_CONFERENCE == pPhone->nConfIdx)
      {
         /* We start conference, because more than two phones
            are in connection. */
         pPhone->nConfIdx = CONFERENCE_GetIdx(pCtrl);
         if (NO_CONFERENCE == pPhone->nConfIdx)
         {
            /* All conferences are occupied. */
            TRACE (TAPIDEMO, DBG_LEVEL_HIGH, ("All conferences occupied.\n"));
            return IFX_ERROR;
         }
         /* check if first conference isn't started already */
         if (IFX_TRUE != pPhone->nConfStarter)
         {
                     
            TRACE(TAPIDEMO, DBG_LEVEL_NORMAL, 
                  ("Phone No %d: Conference started\n", 
                   pPhone->nPhoneNumber));
            pPhone->nConfStarter = IFX_TRUE;
            pConf = &pCtrl->rgoConferences[pPhone->nConfStarter - 1];
            /* check type of first call and add first connection type
               to conference structure */
            if (EXTERN_VOIP_CALL == pPhone->rgoConn[0].nType)
            {
               if (NO_EXTERNAL_PEER == pConf->nExternalPeersCnt)
               {
                  pConf->nExternalPeersCnt++;
               }
            }
            else if (LOCAL_CALL == pPhone->rgoConn[0].nType)
            {
               if (NO_LOCAL_PEER == pConf->nLocalPeersCnt)
               {
                  pConf->nLocalPeersCnt++;
               }
            }
            else if (PCM_CALL == pPhone->rgoConn[0].nType)
            {
               if (NO_PCM_PEER == pConf->nPCM_PeersCnt)
               {
                  pConf->nPCM_PeersCnt++;
               }
            }
            /* conference started successfully */
            ret = IFX_SUCCESS;
         } /* if (IFX_TRUE != pPhone->nConfStarter) */
         else
         {
            /* Wrong input arguments */
            TRACE(TAPIDEMO, DBG_LEVEL_HIGH, 
                  ("Phone No %d: Conference allready started for this phone. "
                   "(File: %s, line: %d)\n", 
                   pPhone->nPhoneNumber, __FILE__, __LINE__));         
            CONFERENCE_ReleaseIdx(&pPhone->nConfIdx, pCtrl);
            ret = IFX_ERROR;
         } /* if (IFX_TRUE != pPhone->nConfStarter) */

      } /* if (NO_CONFERENCE == pPhone->nConfIdx) */
      else
      {
         /* Wrong input arguments */
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH, 
               ("Conference allready started. (File: %s, line: %d)\n", 
                __FILE__, __LINE__));
         ret = IFX_ERROR;
      } /* if (NO_CONFERENCE == pPhone->nConfIdx) */
   }
   else
   {
      TRACE (TAPIDEMO, DBG_LEVEL_HIGH,
            ("Phone No %d: UNABLE to start conference "
             "No connection to the phone exists OR we are not the initiator.\n",
             pPhone->nPhoneNumber));
      ret = IFX_ERROR;
   } /* only one active connection started where phone is caller/initiator */

   return ret;
} /* CONFERENCE_Start() */


/**
   Add local phone to conference.

   \param pCtrl  - pointer to status control structure
   \param pPhone - pointer to PHONE
   \param pDstPhone - pointer to called PHONE (local phone which
                   will be added to conference, connection )
   \param pConn - pointer to local connection

   \return IFX_SUCCESS if ok, otherwise IFX_ERROR

   \remark local calls can only be made on same board
*/
IFX_return_t CONFERENCE_AddLocalPeer(CTRL_STATUS_t* pCtrl,
                                     PHONE_t* pPhone,
                                     PHONE_t* pDstPhone,
                                     CONNECTION_t* pConn)
{
   CONFERENCE_t* pConf = IFX_NULL;
   IFX_return_t ret = IFX_SUCCESS;
   IFX_int32_t i = 0;
   /*PHONE_t* first_phone = IFX_NULL;*/
   CONNECTION_t* conn = IFX_NULL;

   if ((IFX_NULL == pCtrl) || (IFX_NULL == pPhone) || (IFX_NULL == pDstPhone)
       || (pConn == IFX_NULL))
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH, 
           ("Invalid input argument(s). (File: %s, line: %d)\n", 
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /* Get conference if exists */
   if ((NO_CONFERENCE != pPhone->nConfIdx)
       && (MAX_CONFERENCES >= pPhone->nConfIdx))
   {
      pConf = &pCtrl->rgoConferences[pPhone->nConfIdx - 1];
   } 

   if ((IFX_NULL != pConf) && (IFX_TRUE == pPhone->nConfStarter))
   {
      /* If we have conference and we are the initiator */
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_LOW, 
           ("Phone No %d: Adding local phone no %d to conference\n",
            pPhone->nPhoneNumber, pDstPhone->nPhoneNumber));

      /* do mapping for all connections */
      for (i = 0; i < pPhone->nConnCnt - 1; i++)
      {
         conn = &pPhone->rgoConn[i];
         switch (conn->nType)
         {
            case EXTERN_VOIP_CALL:
               /* Because of external call also map phone channel to 
                  all data channel(s) that are participating in conference on
                  this board. */
#ifndef EASY336
#ifndef TAPI_VERSION4
               ret = VOIP_MapPhoneToData(conn->nUsedCh, pDstPhone->nDev,
                                         pDstPhone->nPhoneCh,
                                         IFX_TRUE, pDstPhone->pBoard);
#endif /* TAPI_VERSION4 */
#endif /* EASY336 */
               break;
            case PCM_CALL:
#if (!defined(DXT) && !defined(EASY336))
               /* In connection also with PCM call, map this phone channel to
                  PCM channel */
               ret = PCM_MapToPhone(conn->nUsedCh, pDstPhone->nPhoneCh,
                                    IFX_TRUE, pDstPhone->pBoard);
#endif /* DXT */
               break;
            case LOCAL_CALL:
#ifndef EASY336
               /* Map previous called phone ch with new called phone ch. */
               ret = Common_MapLocalPhones(conn->oConnPeer.oLocal.pPhone,
                                           pDstPhone, TD_MAP_ADD);

#endif /* EASY336 */
               break;
            default:
               /* Unknown call, do nothing */
            break;
         } /* switch */
      } /* for */

      pConf->nLocalPeersCnt++;
      /* If first time calling local phone this flag is set */
      pPhone->fLocalPeerCalled = IFX_TRUE;
   } /* if */

   return ret;
} /* CONFERENCE_AddLocalPeer() */


/**
   Add external phone to conference.

   \param pCtrl - pointer to status control structure
   \param pPhone - pointer to PHONE
   \param pConn - pointer to connection

   \return IFX_SUCCESS if ok otherwise IFX_ERROR
*/
IFX_return_t CONFERENCE_AddExternalPeer(CTRL_STATUS_t* pCtrl,
                                        PHONE_t* pPhone,
                                        CONNECTION_t* pConn)
{
   CONFERENCE_t* p_conf = IFX_NULL;
   IFX_uint32_t new_data_ch = 0;
   IFX_return_t ret = IFX_SUCCESS;
#ifndef EASY336 
   IFX_uint32_t phone_ch = 0;
   IFX_uint32_t i = 0;
   CONNECTION_t* conn = IFX_NULL;
#endif
#ifndef TAPI_VERSION4
   IFX_int32_t nFd;
#endif
#ifdef EASY336
   RESOURCES_t resources;
   VOIP_DATA_CH_t *pCodec;
#endif

   if ((IFX_NULL == pCtrl) || (IFX_NULL == pPhone) || (IFX_NULL == pConn))
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH, 
           ("Invalid input argument(s). (File: %s, line: %d)\n", 
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /* Get conference */
   if ((NO_CONFERENCE != pPhone->nConfIdx)
       && (MAX_CONFERENCES >= pPhone->nConfIdx))
   {
      p_conf = &pCtrl->rgoConferences[pPhone->nConfIdx - 1];
   }

   if (IFX_FALSE == pPhone->fExtPeerCalled)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_LOW, 
            ("Phone No %d: AddExternalPeer() -> first VOIP EXTERN call\n",
             (int) pPhone->nPhoneNumber));

      /* First time making connection with other external phone */
      /* but we could be connected with local or PCM phone(s) */
      pPhone->fExtPeerCalled = IFX_TRUE;
      /* Save caller channel */
      pConn->nUsedCh = pPhone->nDataCh;
      pConn->nUsedCh_FD = pPhone->nDataCh_FD;
      pConn->nUsedSocket = pPhone->nSocket;

      new_data_ch = pPhone->nDataCh;

#ifdef EASY336
      if (pConn->voipID == SVIP_RM_UNUSED)
      {
         /* get codec */
         resources.nType = RES_CODEC | RES_VOIP_ID;
         resources.nConnID_Nr = SVIP_RM_UNUSED;
         if (Common_ResourcesGet(pPhone, &resources) != IFX_SUCCESS)
         {
            /* There is no more free data channels */
            TRACE(TAPIDEMO, DBG_LEVEL_HIGH, 
                 ("No free codecs available. \
                 (File: %s, line: %d)\n", 
                  __FILE__, __LINE__));
            /* Put this PHONE back to active for other phones in conference. */
            return IFX_ERROR;            
         }
         pConn->voipID = pPhone->pVoipIDs[pPhone->nVoipIDs - 1];
         pCodec = Common_GetCodec(pCtrl, pConn->voipID);
         ret = SVIP_libDevIP_Get(pCodec->nDev, 
            (IFX_int32_t*) &pConn->oUsedAddr.sin_addr.s_addr); 
         if (ret != SVIP_libStatusOk)
         {
            TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                 ("Err: SVIP_libDevIP_Get returned error %d (File: %s, "
                  "line: %d)\n",
                  ret, __FILE__, __LINE__));
               /* Put this PHONE back to active for other phones in 
                * conference. */
               return IFX_ERROR;
         }
         if (pCodec != IFX_NULL)
         {
            pConn->nUsedCh = pCodec->nCh;
            pConn->oUsedAddr.sin_port = htons(pCodec->nUDP_Port);
         }
         else
         {
            TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
              ("Err: Common_GetCodec failed (File: %s, "
               "line: %d)\n",
               __FILE__, __LINE__));
            /* Put this PHONE back to active for other phones in conference. */
            return IFX_ERROR;
         }
      }
#endif /* EASY336 */

   }
   else if ((IFX_TRUE == pPhone->nConfStarter)
          && (IFX_NULL != p_conf))
   {
      /* This channel is already in connection with other phone, so
         we must use different channel for new external phone */
      /* So primary data channel is used for dialing the number
         and when ext call, a new free data channel must be used
         and leave old one in state talking */

      TRACE(TAPIDEMO, DBG_LEVEL_LOW, 
            ("Phone No %d: AddExternalPeer() -> next VOIP EXTERN call\n",
             (int) pPhone->nPhoneNumber));

#ifndef EASY336
      /* Get free data channel */
      new_data_ch = VOIP_GetFreeDataCh(pPhone->pBoard);

      if (NO_FREE_DATA_CH == new_data_ch)
      {
         /* There is no more free data channels */
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH, 
              ("No more free data channels. (File: %s, line: %d)\n", 
               __FILE__, __LINE__));
         /* Put this PHONE back to active for other phones in conference. */
         return IFX_ERROR;
      }
#else /* EASY336 */
      if (pConn->voipID == SVIP_RM_UNUSED)
      {
         /* get codec */
         resources.nType = RES_CODEC | RES_VOIP_ID;
         resources.nConnID_Nr = SVIP_RM_UNUSED;
         if (Common_ResourcesGet(pPhone, &resources) != IFX_SUCCESS)
         {
            /* There is no more free data channels */
            TRACE(TAPIDEMO, DBG_LEVEL_HIGH, 
                 ("No free codecs available. \
                 (File: %s, line: %d)\n", 
                  __FILE__, __LINE__));
            /* Put this PHONE back to active for other phones in conference. */
            return IFX_ERROR;            
         }
         pConn->voipID = pPhone->pVoipIDs[pPhone->nVoipIDs - 1];
         pCodec = Common_GetCodec(pCtrl, pConn->voipID);
         ret = SVIP_libDevIP_Get(pCodec->nDev, 
            (IFX_int32_t*) &pConn->oUsedAddr.sin_addr.s_addr); 
         if (ret != SVIP_libStatusOk)
         {
            TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                 ("Err: SVIP_libDevIP_Get returned error %d (File: %s, "
                  "line: %d)\n",
                  ret, __FILE__, __LINE__));
               /* Put this PHONE back to active for other phones in 
                * conference. */
               return IFX_ERROR;
         }
         if (pCodec != IFX_NULL)
         {
            pConn->nUsedCh = pCodec->nCh;
            pConn->oUsedAddr.sin_port = htons(pCodec->nUDP_Port);
         }
         else
         {
            TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
              ("Err: Common_GetCodec failed (File: %s, "
               "line: %d)\n",
               __FILE__, __LINE__));
            /* Put this PHONE back to active for other phones in conference. */
            return IFX_ERROR;
         }
      }
#endif /* EASY336 */

      /* Save caller channel */
#ifndef EASY336
      pConn->nUsedCh = new_data_ch;
      pConn->nUsedCh_FD = VOIP_GetFD_OfCh(new_data_ch, pPhone->pBoard);
      pConn->nUsedSocket = VOIP_GetSocket(new_data_ch, pPhone->pBoard);
      /*pConn->nUsedSocket = VOIP_GetFreeSocket(pCtrl);*/

      TRACE(TAPIDEMO, DBG_LEVEL_LOW, 
           ("Phone No %d: Use data ch %d, socket %d "
            "for new external connection.\n",
            (int) pPhone->nPhoneNumber,
            (int) new_data_ch, (int) pConn->nUsedSocket));
#endif /* EASY336 */

      /* Map initiator to this new data channel - its important that initiator
         is mapped first, so its also mapped to signal module. */
#ifndef EASY336
#ifndef TAPI_VERSION4
      ret = VOIP_MapPhoneToData(new_data_ch, pPhone->nDev, pPhone->nPhoneCh,
                                IFX_TRUE, pPhone->pBoard);
#endif /* TAPI_VERSION4 */
      VOIP_ReserveDataCh(new_data_ch, pPhone->pBoard);
#endif /* EASY336 */

#ifndef TAPI_VERSION4
      nFd = VOIP_GetFD_OfCh(new_data_ch, pPhone->pBoard);
      /* stop playing tone on added data channel */
      if (0 < nFd)
      {
         TD_IOCTL(nFd, IFX_TAPI_TONE_STOP, NO_PARAM);
      }
#endif /* TAPI_VERSION4 */
   }

#ifndef EASY336
   /* for all connections */
   for (i = 0; i < pPhone->nConnCnt - 1; i++)
   {
      conn = &pPhone->rgoConn[i];

      switch (conn->nType)
      {
         case PCM_CALL:
            /* Map PCM to other data channels in conference */
            ret = PCM_MapToData(pPhone->nPCM_Ch, conn->nUsedCh,
                                IFX_TRUE, pPhone->pBoard);
         break;
         case LOCAL_CALL:
            /* Because of external call also map phone channel to 
               data channel */
            phone_ch = (conn->oConnPeer).oLocal.pPhone->nPhoneCh;
#ifndef TAPI_VERSION4
            ret = VOIP_MapPhoneToData(new_data_ch, pPhone->nDev, phone_ch,
                                      IFX_TRUE, pPhone->pBoard);
#endif /* TAPI_VERSION4 */
            /* Only map phones if first time calling external phone and 
               no conference before. */

         break;
         default:
            /* External call, do nothing */
         break;
      } 
   } /* for */
#endif /* EASY336 */

   if (IFX_NULL != p_conf)
   {
      p_conf->nExternalPeersCnt++;
   }

   return ret;
} /* CONFERENCE_AddExternalPeer() */


/**
   Add pcm phone to conference.

   \param pCtrl - pointer to status control structure
   \param pPhone - pointer to PHONE
   \param pConn - pointer to connection

   \return IFX_SUCCESS if ok, otherwise IFX_ERROR
*/
IFX_return_t CONFERENCE_AddPCM_Peer(CTRL_STATUS_t* pCtrl,
                                    PHONE_t* pPhone,
                                    CONNECTION_t* pConn)
{
   IFX_return_t ret = IFX_SUCCESS;
   CONFERENCE_t* p_conf = IFX_NULL;
   IFX_uint32_t new_pcm_ch = 0;
#if (!defined EASY336 && !defined XT16)
   IFX_uint32_t i = 0;
   IFX_uint32_t phone_ch = 0;
   CONNECTION_t* conn = IFX_NULL;
#endif
   IFX_boolean_t first_called = IFX_FALSE;


   if ((IFX_NULL == pCtrl) || (IFX_NULL == pPhone))
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH, 
           ("Invalid input argument(s). (File: %s, line: %d)\n", 
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /* Get conference */
   if ((NO_CONFERENCE != pPhone->nConfIdx)
       && (MAX_CONFERENCES >= pPhone->nConfIdx))
   {
      p_conf = &pCtrl->rgoConferences[pPhone->nConfIdx - 1];
   }

   if (IFX_FALSE == pPhone->fPCM_PeerCalled)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_LOW, ("AddPCM_Peer() -> first PCM call\n"));

      /* First time making connection with other pcm phone, but we could be
         connected with local phone(s) or external phone(s) */
      pPhone->fPCM_PeerCalled = IFX_TRUE;
      first_called = IFX_TRUE;
      /* Save caller channel */
      pConn->nUsedCh = pPhone->nPCM_Ch;
      pConn->nUsedCh_FD = PCM_GetFD_OfCh(pPhone->nPCM_Ch, pPhone->pBoard);

      /* Map phone channel to pcm channel */
      new_pcm_ch = pPhone->nPCM_Ch;

#if (!defined EASY336 && !defined XT16 && !defined DXT)
      ret = PCM_MapToPhone(pPhone->nPCM_Ch, pPhone->nPhoneCh,
                           IFX_TRUE, pPhone->pBoard);
#endif /* (!defined EASY336 && !defined XT16 && !defined DXT) */
   } /* if */
#if (!defined EASY336 && !defined XT16)
   else if ((IFX_TRUE == pPhone->nConfStarter) && (IFX_NULL != p_conf))
   {
      /* This pcm channel is already in connection with other pcm phone, so
         we must use different channel for new pcm phone */

      TRACE(TAPIDEMO, DBG_LEVEL_LOW, ("AddPCM_Peer() -> next PCM call\n"));

      /* Get free pcm channel */
      new_pcm_ch = PCM_GetFreeCh(pPhone->pBoard);
      if (NO_FREE_PCM_CH == new_pcm_ch)
      {
         /* There is no more free pcm channels */
         /* Wrong input arguments */
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH, 
              ("No more free pcm channels. \
              (File: %s, line: %d)\n", 
               __FILE__, __LINE__));
         /* Put this PHONE back to active for other phones in conference. */
         return IFX_ERROR;
      }

      PCM_ReserveCh(new_pcm_ch, pPhone->pBoard);

      /* Save caller channel */
      pConn->nUsedCh = new_pcm_ch;
      pConn->nUsedCh_FD = PCM_GetFD_OfCh(new_pcm_ch, pPhone->pBoard);

      TRACE(TAPIDEMO, DBG_LEVEL_LOW, 
           ("Use pcm ch %d, socket %d for new external connection.\n",
            (int) new_pcm_ch, (int) pConn->nUsedCh_FD));

#ifndef DXT
      /* Map initiator to new pcm channel */
      ret = PCM_MapToPhone(new_pcm_ch, pPhone->nPhoneCh,
                           IFX_TRUE, pPhone->pBoard);
#endif /* DXT */

   } /* else if */

   if (IFX_TRUE == pPhone->nConfStarter)
   {
      for (i = 0; i < pPhone->nConnCnt - 1; i++)
      {
         conn = &pPhone->rgoConn[i];
         switch (conn->nType)
         {
            case EXTERN_VOIP_CALL:
               /* Map PCM to data channel */
               ret = PCM_MapToData(new_pcm_ch, conn->nUsedCh,
                                   IFX_TRUE, pPhone->pBoard);
            break;
            case LOCAL_CALL:
               /* We have local connection, so map  phone ch to this pcm ch. */
               phone_ch = conn->oConnPeer.oLocal.pPhone->nPhoneCh;

#ifndef DXT
               ret = PCM_MapToPhone(new_pcm_ch, phone_ch,
                                    IFX_TRUE,
                                    conn->oConnPeer.oLocal.pPhone->pBoard);
#endif /* DXT */
            break;
            case PCM_CALL:
               if (IFX_FALSE == first_called)
               {
                  /* We have pcm connection, so map phone ch to this pcm ch. */
                  ret = PCM_MapToPCM(new_pcm_ch, conn->nUsedCh,
                                     IFX_TRUE, pPhone->pBoard);
               }
            break;
            default:
               /* Unknown call, do nothing */
            break;
         } 
      } /* for */

      if (IFX_NULL != p_conf)
      {
         p_conf->nPCM_PeersCnt++;
      }
   } /* if */
#endif /* !EASY336 && !XT16 */

   return ret;
} /* CONFERENCE_AddPCM_Peer() */


/**
   Remove peer from conference.

   \param pCtrl  - pointer to status control structure
   \param pPhone - pointer to PHONE
   \param pConn  - pointer to pointer of phone connections
   \param pConf  - pointer to conference structure

   \return IFX_SUCCESS if ok, otherwise IFX_ERROR

   \remark Each data channel can be connected only to one external, but 
           can be connected/mapped with many local or pcm peers.
           Because of deleting connection, pConn will point to first
           connection.
*/
IFX_return_t CONFERENCE_RemovePeer(CTRL_STATUS_t* pCtrl,
                                   PHONE_t* pPhone,
                                   CONNECTION_t** pConn,
                                   CONFERENCE_t* pConf)
{
   IFX_return_t ret = IFX_SUCCESS;
   IFX_boolean_t peer_removed = IFX_FALSE;
   CONNECTION_t* conn = IFX_NULL;
   CONNECTION_t* pTmpConn = IFX_NULL;
   IFX_int32_t i;
#ifdef EASY336   
   RESOURCES_t resources;
#endif

   if ((IFX_NULL == pCtrl) || (IFX_NULL == pPhone)
       || (IFX_NULL == pConf) || (IFX_NULL == pConn))
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH, 
           ("Invalid input argument(s). (File: %s, line: %d)\n", 
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   conn = *pConn;
   
   if (IFX_NULL == conn)
   {
      /* Connections are missing */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH, 
           ("Phone No %d: No phone connections to delete. (File: %s, line: %d)\n", 
            pPhone->nPhoneNumber, __FILE__, __LINE__));
      return IFX_ERROR;
   }
   /* removing local call */
   if (LOCAL_CALL == conn->nType)
   {
      if (IFX_TRUE == conn->fActive)
      {
         /* print call progress trace */
         TRACE(TAPIDEMO, DBG_LEVEL_NORMAL,
               ("Phone No %d: Disconnected %s with PhoneNo %d, PhoneCh %d\n",
                pPhone->nPhoneNumber, 
                Common_Enum2Name(conn->nType, TD_rgCallTypeName),
                conn->oConnPeer.nPhoneNum,
                conn->oConnPeer.oLocal.pPhone->nPhoneCh));
      }
      /* LOCAL */
      TRACE(TAPIDEMO, DBG_LEVEL_LOW,
           ("Phone No %d: Removing local party from conference, "
            "called phone ch %d\n",
            pPhone->nPhoneNumber,
            (IFX_int32_t) conn->oConnPeer.oLocal.pPhone->nPhoneCh));

#ifndef EASY336
      /* Unmap called phone ch with conference starter phone channel. */
      ret = Common_MapLocalPhones(pPhone, conn->oConnPeer.oLocal.pPhone,
                                  TD_MAP_REMOVE);
#ifndef TAPI_VERSION4
      /* Unmap called phone ch with conference starter data channel. */
      ret = VOIP_MapPhoneToData(pPhone->nDataCh,
                                conn->oConnPeer.oLocal.pPhone->nDev,
                                conn->oConnPeer.oLocal.pPhone->nPhoneCh,
                                IFX_FALSE, pPhone->pBoard);
#endif /* TAPI_VERSION4 */
      /* do unmapping for all existing connections */
      for (i=0; i<pPhone->nConnCnt; i++)
      {
         /* get connection */
         pTmpConn = &pPhone->rgoConn[i];
         /* for connection that is now removed take no action */
         if (conn == pTmpConn)
         {
            /* skip this one */
            continue;
         }
         /* check connection type */
         switch(pTmpConn->nType)
         {
            case LOCAL_CALL:
               /* Unmap previous called phone ch 
                  from phone channel that is leaving conference. */
               Common_MapLocalPhones(pTmpConn->oConnPeer.oLocal.pPhone,
                                     conn->oConnPeer.oLocal.pPhone,
                                     TD_MAP_REMOVE);
               break;
            case EXTERN_VOIP_CALL:
#ifndef TAPI_VERSION4
               /* Unmap previous called data ch 
                  from phone channel that is leaving conference. */
               ret = VOIP_MapPhoneToData(pTmpConn->nUsedCh,
                                         conn->oConnPeer.oLocal.pPhone->nDev,
                                         conn->oConnPeer.oLocal.pPhone->nPhoneCh,
                                         IFX_FALSE, pPhone->pBoard);
#endif /* TAPI_VERSION4 */
               break;
            case PCM_CALL:
#if (!defined(DXT) && !defined(EASY336))
               /* Unmap previous called data ch 
                  from phone channel that is leaving conference. */
               ret = PCM_MapToPhone(pTmpConn->nUsedCh,
                                    conn->oConnPeer.oLocal.pPhone->nPhoneCh,
                                    IFX_FALSE, pPhone->pBoard);
#endif /* (!defined(DXT) && !defined(EASY336)) */
               break;
            default:
               TRACE(TAPIDEMO, DBG_LEVEL_HIGH, 
                     ("Error restoring mapping for phone ch %d. "
                      "(File: %s, line: %d)\n", 
                      (int) conn->oConnPeer.oLocal.pPhone->nPhoneCh,
                      __FILE__, __LINE__));
               break;
         } /* switch(pTmpConn->nType) */
      } /* for all conections of phone */

#endif /* EASY336 */

      /* Flag indicating that peer was removed */
      peer_removed = IFX_TRUE;

      pConf->nLocalPeersCnt--;
      if (NO_LOCAL_PEER == pConf->nLocalPeersCnt)
      {
         pPhone->fLocalPeerCalled = IFX_FALSE;
      }
   } /* if */
   else if (EXTERN_VOIP_CALL == conn->nType)
   {
      if (IFX_TRUE == conn->fActive)
      {
         /* print call progress trace */
         TRACE(TAPIDEMO, DBG_LEVEL_NORMAL,
               ("Phone No %d: Disconnected %s with PhoneNo %d\n%son %s\n",
                pPhone->nPhoneNumber, 
                Common_Enum2Name(conn->nType, TD_rgCallTypeName),
                conn->oConnPeer.nPhoneNum,
                pPhone->pBoard->pIndentPhone,
                inet_ntoa(conn->oConnPeer.oRemote.oToAddr.sin_addr)));
      }
      /* check if external dialing in progress */
      if (IFX_TRUE == conn->bExtDialingIn)
      {
         conn->bExtDialingIn = IFX_FALSE;
         TD_TIMEOUT_STOP(pCtrl->nUseTimeoutSocket);
      }
      /* REMOTE */
      TRACE(TAPIDEMO, DBG_LEVEL_LOW,
           ("Phone No %d: Removing called external party %s:%d\n"
            "%sfrom conference, used data ch %d for calling.\n",
            pPhone->nPhoneNumber,
            inet_ntoa(conn->oConnPeer.oRemote.oToAddr.sin_addr),
            ntohs(conn->oConnPeer.oRemote.oToAddr.sin_port),
            pPhone->pBoard->pIndentPhone,
            (IFX_int32_t) conn->nUsedCh));

      /* if not default data channel */
      if (conn->nUsedCh != pPhone->nDataCh)
      {
#ifndef EASY336
#ifndef TAPI_VERSION4
         ret = VOIP_MapPhoneToData(conn->nUsedCh, pPhone->nDev, pPhone->nPhoneCh,
                                   IFX_FALSE, pPhone->pBoard);
#endif /* TAPI_VERSION4 */
         VOIP_FreeDataCh(conn->nUsedCh, pPhone->pBoard);
#endif /* EASY336 */
         
      }

      /* do unmapping for all existing connections */
      for (i=0; i<pPhone->nConnCnt; i++)
      {
         pTmpConn = &pPhone->rgoConn[i];
         /* for connection that is now removed take no action */
         if (conn == pTmpConn)
         {
            /* skip this connection */
            continue;
         }
         switch(pTmpConn->nType)
         {
            case LOCAL_CALL:            
#ifndef TAPI_VERSION4
               /* Unmap called phone channel 
                  from data channel used by ended connection */
               ret = VOIP_MapPhoneToData(conn->nUsedCh, pPhone->nDev,
                        pTmpConn->oConnPeer.oLocal.pPhone->nPhoneCh,
                        IFX_FALSE, pPhone->pBoard);
#endif /* TAPI_VERSION4 */
               break;
            case EXTERN_VOIP_CALL:
               break;
            case PCM_CALL:
               /* check if default data channel and phone channel */
               if (pPhone->nPCM_Ch == pTmpConn->nUsedCh &&
                   pPhone->nDataCh == conn->nUsedCh)
               {
                  /* unammping is not needed */
                  break;
               }
               /* Unmap called phone PCM channel 
                  from data channel used by ended connection */
               ret = PCM_MapToData(pTmpConn->nUsedCh, conn->nUsedCh,
                                   IFX_FALSE, pPhone->pBoard);
               break;
            default:
               TRACE(TAPIDEMO, DBG_LEVEL_HIGH, 
                     ("Error restoring mapping for phone ch %d. "
                      "(File: %s, line: %d)\n", 
                      (int) conn->oConnPeer.oLocal.pPhone->nPhoneCh,
                      __FILE__, __LINE__));
               break;
         } /* switch(pTmpConn->nType) */
      } /* for all conections of phone */

#ifndef TAPI_VERSION4
      /* Stop codec on this data channel connected to remote peer */
      VOIP_StopCodec(conn->nUsedCh, conn, pPhone->pBoard);
#else
#ifdef EASY336
      VOIP_StopCodec(conn->nUsedCh, conn,
                     ABSTRACT_GetBoard(pCtrl,
                                       IFX_TAPI_DEV_TYPE_VIN_SUPERVIP));
      /* return codec */
      resources.nType = RES_CODEC | RES_VOIP_ID;
      resources.nConnID_Nr = SVIP_RM_UNUSED;
      resources.nVoipID_Nr = conn->voipID;
      Common_ResourcesRelease(pPhone, &resources);
#endif /* EASY336 */
#endif /* TAPI_VERSION4 */

      /* Flag peer was removed */
      peer_removed = IFX_TRUE;

      pConf->nExternalPeersCnt--;
      if (NO_EXTERNAL_PEER == pConf->nExternalPeersCnt)
      {
         pPhone->fExtPeerCalled = IFX_FALSE;
      }
#ifdef QOS_SUPPORT
      if (pCtrl->pProgramArg->oArgFlags.nQos)
      {
/* no QoS for SVIP */
#ifndef EASY336
         /* QoS Support */
         QOS_StopSession(conn, pPhone);
#endif /* EASY336 */
      }
#endif /* QOS_SUPPORT */
   } /* else if */
   else if (PCM_CALL == conn->nType)
   {
      TAPIDEMO_PORT_DATA_t oPortData;

      if (IFX_TRUE == conn->fActive)
      {
         /* print call progress trace */
         TRACE(TAPIDEMO, DBG_LEVEL_NORMAL,
               ("Phone No %d: Disconnected %s with PhoneNo %d, on %s\n",
                pPhone->nPhoneNumber, 
                Common_Enum2Name(conn->nType, TD_rgCallTypeName),
                conn->oConnPeer.nPhoneNum,
                inet_ntoa(conn->oConnPeer.oPCM.oToAddr.sin_addr)));
      }
      /* PCM */
      TRACE(TAPIDEMO, DBG_LEVEL_LOW,
           ("Phone No %d: Removing called pcm party from conference, "
            "pcm ch %d\n",
            pPhone->nPhoneNumber, (int) conn->nUsedCh));

      /* Set this PCM channel as unused */
      if (pPhone->nPCM_Ch != conn->nUsedCh)
      {
         PCM_FreeCh(conn->nUsedCh, pPhone->pBoard);
      }
#ifndef DXT
      /* Unmap conference starter phone channel from
         PCM channel of ended connection*/
      ret = PCM_MapToPhone(conn->nUsedCh, pPhone->nPhoneCh,
                           IFX_FALSE, pPhone->pBoard);
#endif /* DXT */
      /* do unmapping for all existing connections */
      for (i=0; i<pPhone->nConnCnt; i++)
      {
         pTmpConn = &pPhone->rgoConn[i];
         /* for connection that is now removed take no action */
         if (conn == pTmpConn)
         {
            /* skip this one */
            continue;
         }
         switch(pTmpConn->nType)
         {
            case LOCAL_CALL:
#ifndef DXT
               /* Unmap phone channel from 
                  PCM channel of connection that ended */
               ret = PCM_MapToPhone(conn->nUsedCh,
                                    pTmpConn->oConnPeer.oLocal.pPhone->nPhoneCh,
                                    IFX_FALSE, pPhone->pBoard);
#endif /* DXT */
               break;
            case EXTERN_VOIP_CALL:
               /* Unmap phone channel from 
                  data channel of connection that ended */
               ret = PCM_MapToData(conn->nUsedCh, pTmpConn->nUsedCh,
                                   IFX_FALSE, pPhone->pBoard);
               break;
            case PCM_CALL:
               /* Unmap phone channel from 
                  PCM channel of connection that ended */
               ret = PCM_MapToPCM(conn->nUsedCh, pTmpConn->nUsedCh,
                                  IFX_FALSE, pPhone->pBoard);
               break;
            default:
               TRACE(TAPIDEMO, DBG_LEVEL_HIGH, 
                     ("Error restoring mapping for phone ch %d. "
                      "(File: %s, line: %d)\n", 
                      (int) conn->oConnPeer.oLocal.pPhone->nPhoneCh,
                      __FILE__, __LINE__));
               break;
         } /* switch(pTmpConn->nType) */
      } /* for all conections of phone */
      
      oPortData.nType = PORT_FXS;
      oPortData.uPort.pPhopne = pPhone;
      /* deactivate PCM and free timeslots */
      PCM_EndConnection(&oPortData, pCtrl->pProgramArg, pPhone->pBoard, conn);
      /* Flag peer was removed */
      peer_removed = IFX_TRUE;

      pConf->nPCM_PeersCnt--;
      if (NO_PCM_PEER == pConf->nPCM_PeersCnt)
      {
         pPhone->fPCM_PeerCalled = IFX_FALSE;
      }
   } /* if */

   /* if peer successfully removed */
   if (IFX_TRUE == peer_removed)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_LOW,
            ("Phone No %d: Peer removed, now remove phone connection.\n",
             pPhone->nPhoneNumber));

      /* conection isn't active */
      conn->fActive = IFX_FALSE;
      conn->nType = UNKNOWN_CALL_TYPE;
      /* Remove this connection from array. Move last one to this location 
         and clear last one, if we are not the last one. */
      if (0 < pPhone->nConnCnt)
      {
         if (conn != &pPhone->rgoConn[(pPhone->nConnCnt - 1)])
         {
            /* move data from existing connection
               to ended connection structure */
            memcpy(conn, &pPhone->rgoConn[pPhone->nConnCnt - 1],
                   sizeof(CONNECTION_t));
         }
         /* reset connection structure */
         memset(&pPhone->rgoConn[pPhone->nConnCnt - 1],
                0, sizeof(CONNECTION_t));
         /* Status we have one less peer */
         pPhone->nConnCnt--;
      } /* if (0 < pPhone->nConnCnt) */
   } /* if (IFX_TRUE == peer_removed) */

   /* pConn has changed, because one peer, connection was removed. */
   *pConn = &pPhone->rgoConn[0];
   return ret;
} /* CONFERENCE_RemovePeer() */


/**
   Clear everything after conference is finished.

   \param pCtrl  - pointer to status control structure
   \param pPhone - pointer to PHONE
   \param pConf  - pointer to conference structure

   \return IFX_SUCCESS if ok, otherwise IFX_ERROR
*/
IFX_return_t CONFERENCE_End(CTRL_STATUS_t* pCtrl,
                            PHONE_t* pPhone,
                            CONFERENCE_t* pConf)
{


   IFX_return_t ret = IFX_SUCCESS;
   CONNECTION_t* pConn = IFX_NULL;
#ifdef TAPI_VERSION4
#ifdef EASY336
   RESOURCES_t resources;
#endif
#endif /* TAPI_VERSION4 */

   if ((IFX_NULL == pCtrl) || (IFX_NULL == pPhone) || (IFX_NULL == pConf))
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH, 
           ("Invalid input argument(s). (File: %s, line: %d)\n", 
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /* end all connections */
   while (0 < pPhone->nConnCnt)
   {
      /* get pointer to last connection structure */
      pConn = &pPhone->rgoConn[pPhone->nConnCnt - 1];
      /* We started conference so end it */
      if (IFX_TRUE == pConn->fActive)
      {
         /* Send all phones which are in active connection BUSY */
         TAPIDEMO_SetAction(pCtrl, pPhone, pConn, IE_END_CONNECTION);
      }
      else
      {/* for all conections */
         /* those resources were reserved during dialing */
         if (IFX_TRUE == pConn->bExtDialingIn)
         {
#ifdef TAPI_VERSION4
#ifdef EASY336
            /* return codec */
            resources.nType = RES_CODEC | RES_VOIP_ID;
            resources.nConnID_Nr = SVIP_RM_UNUSED;
            resources.nVoipID_Nr = pConn->voipID;
            Common_ResourcesRelease(pPhone, &resources);
#endif
#endif /* TAPI_VERSION4 */
            pConn->bExtDialingIn = IFX_FALSE;
            TD_TIMEOUT_STOP(pCtrl->nUseTimeoutSocket);
         }
         /* Send all phones which are called (last phone ringing, ...) READY */
         /* Send called phone (is ringing) to stop ringing */
         TAPIDEMO_SetAction(pCtrl, pPhone, pConn, IE_READY);
      }
      /* end this connection */
      CONFERENCE_RemovePeer(pCtrl, pPhone, &pConn, pConf);
   }

   /* release this conference index */
   CONFERENCE_ReleaseIdx(&pPhone->nConfIdx, pCtrl);
   /* clear phone data */
   pPhone->nConnCnt = 0;
   /* set conference as ended */
   pPhone->nConfStarter = IFX_FALSE;

#ifdef TAPI_VERSION4
   /* stop DTMF detection */
   Common_DTMF_Disable(pCtrl, pPhone);
#ifdef EASY336

   resources.nType = RES_CONN_ID | RES_LEC;
   /* stop LEC */
   Common_LEC_Disable(pCtrl, pPhone);
   Common_ResourcesRelease(pPhone, &resources);
#endif /* EASY336 */
#endif /* TAPI_VERSION4 */
   TRACE(TAPIDEMO, DBG_LEVEL_NORMAL, 
         ("Phone No %d: Conference has ended.\n",
          pPhone->nPhoneNumber));

   return ret;
} /* CONFERENCE_End() */

