/******************************************************************************

                              Copyright (c) 2009
                            Lantiq Deutschland GmbH
                     Am Campeon 3; 85579 Neubiberg, Germany

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

  ****************************************************************************/

/**
   \file device_duslic_xt.c
   \date 2006-10-01
   \brief Interface implementation for DANUBE board.

   Application needs it to initialize board, prepare structures for this board
   and set mode in which board will work (using PCM, GPIO, ...).
*/

/* ============================= */
/* Includes                      */
/* ============================= */
#include "tapidemo.h"
#include "common.h"
#ifdef HAVE_DRV_DUSLICXT_HEADERS
    #include "drv_dxt_errno.h"
    #include "drv_dxt_strerrno.h"
#else
    #error "drv_dxt is missing"
#endif

#include "device_duslic_xt.h"

#ifndef USE_FILESYSTEM
/** Default FW file includes, to be provided */

#include "dxt_bbd_fxs.c"
#include "dxt_firmware.c"

#endif /* USE_FILESYSTEM */

/* ============================= */
/* Defines                       */
/* ============================= */

/* ============================= */
/* Local variable definition     */
/* ============================= */

/** Names of access modes */
static const IFX_char_t* DXT_ACCESS_MODES[] =
{
   "DXT_ACCESS_SPI",
   "DXT_ACCESS_HD_AUDIO",
   "DXT_ACCESS_HD_AUDIO_TEST",
   "DXT_ACCESS_TESTMODE",
   IFX_NULL
};

/* ============================= */
/* Global variable definition    */
/* ============================= */


#ifdef USE_FILESYSTEM
/** Prepare file names for DXT */
IFX_char_t* sPRAMFile_DXT = "voice_dxt_firmware.bin";
IFX_char_t* sPRAMFile_DXT_Old = "dxt_firmware.bin";
IFX_char_t* sDRAMFile_DXT = "";
/** File holding coefficients. */
IFX_char_t* sBBD_CRAM_File_DXT = "dxt_bbd.bin";
IFX_char_t* sBBD_CRAM_File_DXT_Old = "dxt_bbd_fxs.bin";
#endif /* USE_FILESYSTEM */

/** Device names */
IFX_char_t* sCH_DEV_NAME_DXT = "/dev/dxt";
IFX_char_t* sCTRL_DEV_NAME_DXT = "/dev/dxt";
IFX_char_t* sSYS_DEV_NAME_DXT = "/dev/easy3201/0";

/** Holding device dependent data */
DXT_IO_Init_t DuslicXtDevInitStruct;

/** Holding FW and CRAM, BBD configuration data */
#ifdef USE_FILESYSTEM
static IFX_uint8_t* pFW_Pram = IFX_NULL;
static IFX_uint8_t* pBBD_Cram = IFX_NULL;
static IFX_size_t nFW_Pram_Size = 0;
static IFX_size_t nBBD_Cram_Size = 0;
#endif /* USE_FILESYSTEM */


/* ============================= */
/* Local function declaration    */
/* ============================= */
IFX_return_t DEVICE_DUSLIC_XT_Fillin_FW_Ptr(IFX_void_t* pDrvInit,
                                            TAPIDEMO_DEVICE_CPU_t* pCpuDevice,
                                            IFX_char_t* pPath);
IFX_void_t DEVICE_DUSLIC_XT_Release_FW_Ptr(IFX_void_t* pDrvInit);
IFX_return_t DEVICE_DUSLIC_XT_GetVersions(BOARD_t* pBoard);
IFX_void_t DEVICE_DUSLIC_XT_DecodeErrnoForDrv(IFX_int32_t nCode);

/* ============================= */
/* Local function definition     */
/* ============================= */
/**
   Fills in firmware pointers.

   \param pDrvInit - handle to VINETIC_IO_INIT structure, not nil
   \param pCpuDevice - pointer to CPU device
   \param pPath - path to FW files for download

   \return IFX_SUCCESS on ok, otherwise IFX_ERROR

   \remarks
     In case no firmware binaries is found in the file system or no file system
     is used, default binaries will be used
*/
IFX_return_t DEVICE_DUSLIC_XT_Fillin_FW_Ptr(IFX_void_t* pDrvInit,
                                            TAPIDEMO_DEVICE_CPU_t* pCpuDevice,
                                            IFX_char_t* pPath)
{
   DXT_IO_Init_t* dxt_init = (DXT_IO_Init_t *) pDrvInit;
#ifdef USE_FILESYSTEM
   IFX_return_t ret;
   IFX_char_t pszFullFilename[MAX_PATH_LEN] = {0};
   IFX_boolean_t bFoundBinFile = IFX_FALSE;
#endif /* USE_FILESYSTEM */

   if ((dxt_init == IFX_NULL) || (pCpuDevice == IFX_NULL))
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /* Set all entries to nil */
   memset(dxt_init, 0, sizeof(DXT_IO_Init_t));

#ifdef USE_FILESYSTEM

   /* Did we already load FW files? This check is performed if more than
      one same type of DXT chip are in environment. */
   if ((pFW_Pram != IFX_NULL) && (pBBD_Cram != IFX_NULL))
   {
      dxt_init->pPRAMfw = pFW_Pram;
      dxt_init->pram_size = nFW_Pram_Size;
      dxt_init->pBBDbuf = pBBD_Cram;
      dxt_init->bbd_size = nBBD_Cram_Size;
      return IFX_SUCCESS;
   }

   if (pFW_Pram != IFX_NULL)
   {
      TD_OS_MemFree(pFW_Pram);
      pFW_Pram = IFX_NULL;
      nFW_Pram_Size = 0;
   }

   if (pBBD_Cram != IFX_NULL)
   {
      TD_OS_MemFree(pBBD_Cram);
      pBBD_Cram = IFX_NULL;
      nBBD_Cram_Size = 0;
   }

   /* Reading PRAM_FW file */
   /* Find PRAM_FW file with default names. */
   /* First check for new name, next for old one */
   bFoundBinFile = Common_FindPRAM_FW(pCpuDevice, pPath, pszFullFilename);
   if(IFX_TRUE == bFoundBinFile)
   {
      /* Fill in PRAM_FW buffers from binary */
      ret = TD_OS_FileLoad(pszFullFilename, &pFW_Pram, &nFW_Pram_Size);
      /* Release PRAM_FW memory if applicable */
      if ((IFX_ERROR == ret) || (IFX_NULL == pFW_Pram))
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
              ("Error reading firmware file %s.\n(File: %s, line: %d)\n",
               pszFullFilename, __FILE__, __LINE__));

         if (pFW_Pram != IFX_NULL)
         {
            TD_OS_MemFree(pFW_Pram);
            pFW_Pram = IFX_NULL;
         }
         return IFX_ERROR;
      }
      else
      {
         dxt_init->pPRAMfw = pFW_Pram;
         dxt_init->pram_size = nFW_Pram_Size;

         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
              ("Filesystem firmware binary used (%s).\n",
               pszFullFilename));
      } /* if ((IFX_ERROR == ret_fwram) ... */
   } /* if(IFX_TRUE == bFileFound) */
   else
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Error, firmware file %s is not found.\n(File: %s, line: %d)\n",
            pCpuDevice->pszPRAM_FW_File[0], __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /* Reading BBD_CRAM file */
   /* Find BBD_CRAM file with default names. */
   /* First check for new name, next for old one */
   bFoundBinFile = Common_FindBBD_CRAM(pCpuDevice, pPath, pszFullFilename);
   if(IFX_TRUE == bFoundBinFile)
      {
         /* Fill in BBD_CRAM buffers from binary */
         ret = TD_OS_FileLoad(pszFullFilename, &pBBD_Cram, &nBBD_Cram_Size);

         /* Release BBD_CRAM memory if applicable */
         if ((IFX_ERROR == ret) || (IFX_NULL == pBBD_Cram)) 
         {
            TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                 ("Error reading BBD file %s.\n(File: %s, line: %d)\n",
               pszFullFilename, __FILE__, __LINE__));

            if (pBBD_Cram != IFX_NULL)
            {
               TD_OS_MemFree(pBBD_Cram);
               pBBD_Cram = IFX_NULL;
            }
            return IFX_ERROR;
         }
         else
         {
            TRACE(TAPIDEMO, DBG_LEVEL_NORMAL, ("Filesystem BBD binary used (%s).\n",
                                                pszFullFilename));
            dxt_init->pBBDbuf = pBBD_Cram;
            dxt_init->bbd_size = nBBD_Cram_Size;
            bFoundBinFile = IFX_TRUE;
         }
      }
   else
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Error, BBD file %s is not found.\n(File: %s, line: %d)\n",
             pCpuDevice->pszBBD_CRAM_File[0], __FILE__, __LINE__));
      return IFX_ERROR;
   }

#else /* USE_FILESYSTEM */

   /* If file system support isn't used then use default PRAM bins */
   TRACE(TAPIDEMO, DBG_LEVEL_NORMAL,
         "Default firmware PRAM binaries used\n"));
   dxt_init->pPRAMfw = (IFX_uint8_t *) dxt_firmware;
   dxt_init->pram_size = sizeof(dxt_firmware);

   /* If file system support isn't used then use default BBD_CRAM bins */
   TRACE(TAPIDEMO, DBG_LEVEL_NORMAL, ("Default BBD binaries used\n"));
   dxt_init->pBBDbuf = (IFX_uint8_t *) dxt_bbd_buf;
   dxt_init->bbd_size = sizeof(dxt_bbd_buf);
#endif /* USE_FILESYSTEM */

   return IFX_SUCCESS;
} /* Fillin_FW_Ptr() */


/**
   Release in firmware pointers.

   \param pDrvInit - handle to VINETIC_IO_INIT structure, not nil
   \param pBoard - pointer to board

   \return none
*/
IFX_void_t DEVICE_DUSLIC_XT_Release_FW_Ptr(IFX_void_t* pDrvInit)
{
   DXT_IO_Init_t* dxt_init = (DXT_IO_Init_t *) pDrvInit;

   if (dxt_init == IFX_NULL)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return;
   }

#ifdef USE_FILESYSTEM
   /* Pram fw was read from file, as default isn't set: free memory */
   if (dxt_init->pPRAMfw != IFX_NULL)
   {
      TD_OS_MemFree(dxt_init->pPRAMfw);
      dxt_init->pPRAMfw = IFX_NULL;
      pFW_Pram = IFX_NULL;
      nFW_Pram_Size = 0;
   }

   if (dxt_init->pBBDbuf != IFX_NULL)
   {
      TD_OS_MemFree(dxt_init->pBBDbuf);
      dxt_init->pBBDbuf = IFX_NULL;
      pBBD_Cram = IFX_NULL;
      nBBD_Cram_Size = 0;
   }

#endif /* USE_FILESYSTEM */

} /* DEVICE_DUSLIC_XT_Release_FW_Ptr() */

/* ============================= */
/* Global function definition    */
/* ============================= */

/**
   Read version after succesfull initialization and display it.

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \param pBoard - pointer to board

   \remark
      This must be done only after a successfull basic device initialization
*/
IFX_return_t DEVICE_DUSLIC_XT_GetVersions(BOARD_t* pBoard)
{
   IFX_return_t ret;
   DXT_IO_Version_t vinVers;
   IFX_uint8_t* pVersionNumber;

   if (pBoard == IFX_NULL)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Invalid input argument(s). (File: %s, line: %d)\n",
             __FILE__, __LINE__));
      return IFX_ERROR;
   }

   memset(&vinVers, 0, sizeof(DXT_IO_Version_t));

   ret = TD_IOCTL(pBoard->nDevCtrl_FD, FIO_DXT_VERS, (IFX_int32_t) &vinVers);
   if (IFX_SUCCESS == ret)
   {
      TAPIDEMO_PRINTF(("DXT [HW revision 0x%02X, ASDSP revision 0x%02X, ID 0x%02X,"
                       " fxs ch %d, fxo ch %d] ready!\n"
                       "FW Version %d\n",
                       vinVers.nHwRev, vinVers.nAsdspRev, vinVers.nDevID,
                       vinVers.nFxsCh, vinVers.nFxoCh, vinVers.nFwRev));

      pVersionNumber = (IFX_uint8_t*) &vinVers.nDrvVers;
      TAPIDEMO_PRINTF(("Driver version %d.%d.%d.%d\n",
                       pVersionNumber[0],
                       pVersionNumber[1],
                       pVersionNumber[2],
                       pVersionNumber[3]));
   }
   else
   {
      TAPIDEMO_PRINTF(("DXT version could not be read.\n"));
   }

   return ret;
} /* DEVICE_DUSLIC_XT_GetVersions() */

/**
   Basic device initialization.

   \param nDevFD     - device file descriptor
   \param nIrqNum    - Irq Line number

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

*/
IFX_return_t DEVICE_DUSLIC_XT_BasicInit(IFX_int32_t nDevFD,
                                        IFX_uint32_t nIrqNum)
{
   DXT_BasicDeviceInit_t dxt_init;

   if ((0 > nDevFD) || (0>nIrqNum))
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /* Reset device data in the duslic driver */
   if (TD_IOCTL(nDevFD, FIO_DXT_DEV_RESET, 0) != IFX_SUCCESS)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Err with ioctl cmd FIO_DXT_DEV_RESET."
             "(File: %s, line: %d)\n", __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /* configure dxt device */
   memset(&dxt_init, 0, sizeof(DXT_BasicDeviceInit_t));
   dxt_init.nIrqNum = nIrqNum;

   TRACE(TAPIDEMO, DBG_LEVEL_NORMAL,
         ("Configure DXT device with IRQ 0x%0X and access mode %s\n",
          dxt_init.nIrqNum, DXT_ACCESS_MODES[0]));

   if (TD_IOCTL(nDevFD, FIO_DXT_BASICDEV_INIT,
                (IFX_int32_t) &dxt_init) != IFX_SUCCESS)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Err with ioctl cmd FIO_DXT_BASICDEV_INIT."
             "(File: %s, line: %d)\n", __FILE__, __LINE__));
      return IFX_ERROR;
   }
   return IFX_SUCCESS;
}

/**
   Device initialization.

   \param pBoard - pointer to board
   \param pPath - path to FW files for download

   \return IFX_SUCCES on ok, otherwise IFX_ERROR

   \remarks
     In case no firmware binaries is found in the file system or no file system
     is used, default binaries will be used
*/
IFX_return_t DEVICE_DUSLIC_XT_Init(BOARD_t* pBoard, IFX_char_t* pPath)
{
#ifdef TD_USE_CHANNEL_INIT
   IFX_TAPI_CH_INIT_t Init = {0};
#endif /* TD_USE_CHANNEL_INIT */
   IFX_int32_t i = 0;
   TAPIDEMO_DEVICE_t      *pDevice = IFX_NULL;
   TAPIDEMO_DEVICE_CPU_t  *pCpuDevice = IFX_NULL;

   if (pBoard == IFX_NULL)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   if ((pDevice = Common_GetDevice_CPU(pBoard->pDevice)) == IFX_NULL)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Err, CPU device not found. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }
   pCpuDevice = &pDevice->uDevice.stCpu;

#ifdef TD_USE_CHANNEL_INIT
   DEVICE_DUSLIC_XT_Fillin_FW_Ptr(pCpuDevice->pDevDependStuff, pCpuDevice, pPath);

   /* Set tapi init structure */
   memset(&Init, 0, sizeof(IFX_TAPI_CH_INIT_t));
   Init.nMode = IFX_TAPI_INIT_MODE_DEFAULT;
   Init.pProc = (IFX_void_t*) pCpuDevice->pDevDependStuff;
#endif /* TD_USE_CHANNEL_INIT */

#if 0
   /* Set report to low */
   TD_IOCTL(pBoard->nDevCtrl_FD, FIO_DXT_REPORT_SET, 0x1);

   /* Run Chip access test on DUSLIC-XT, only used for testing purposes. */
   if (pBoard->nType == TYPE_DUSLIC_XT)
   {
      if (TD_IOCTL(pBoard->nDevCtrl_FD, FIO_DXT_TCA, 0x1000) != IFX_SUCCESS)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("Test Chip Access error, (File: %s, line: %d)\n",
               __FILE__, __LINE__));
         return IFX_ERROR;
      }
   }
#endif /* STREAM_1_1 */

   /* Initialize all tapi channels */
   for (i = 0; i <= pBoard->nMaxAnalogCh; i++)
   {
      if (pBoard->nMaxAnalogCh == i)
      {
         /* Read version */
         DEVICE_DUSLIC_XT_GetVersions(pBoard);
         break;
      }
#ifdef TD_USE_CHANNEL_INIT
      /* Initialize all system channels */
      if (IFX_SUCCESS != TD_IOCTL(pBoard->nCh_FD[i], IFX_TAPI_CH_INIT,
                                  (IFX_int32_t) &Init))
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
              ("TAPI INIT error on ch %d, (File: %s, line: %d)\n",
               (int) i, __FILE__, __LINE__));
         break;
      }
#endif /* TD_USE_CHANNEL_INIT */

      /* Set appropriate feeding on all (analog) line channels */
      /* Set line in standby */
      if (IFX_SUCCESS != TD_IOCTL(pBoard->nCh_FD[i], IFX_TAPI_LINE_FEED_SET,
                                  IFX_TAPI_LINE_FEED_STANDBY))
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
              ("Err set line to standby for ch %d, (File: %s, line: %d)\n",
               (int) i, __FILE__, __LINE__));
         break;
      }
   } /* for */

   /*DEVICE_DUSLIC_XT_Release_FW_Ptr(pBoard->pDevDependStuff, pBoard);*/

   if (pBoard->nMaxAnalogCh != i)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Init failed for ch %d, (File: %s, line: %d)\n",
            (int) i, __FILE__, __LINE__));
      return IFX_ERROR;
   }

   return IFX_SUCCESS;
} /* DEVICE_DUSLIC_XT_Init() */

/**
   Handles the DRIVER error codes and stack.

   \param nCode - error code

   \remarks
*/
IFX_void_t DEVICE_DUSLIC_XT_DecodeErrnoForDrv(IFX_int32_t nCode)
{
   IFX_int32_t i = 0;

   for (i = 0; i < DXT_ERRNO_CNT; ++i)
   {
      if (DXT_drvErrnos[i] == (nCode & 0xFFFF))
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("LL Err: %s\n", DXT_drvErrStrings[i]));
         break;
      }
   }
}

/**
   Return last error number for DuSLIC-xT driver.

   \param nFd  -  channel file descriptor

   \return number of error if successful, otherwise IFX_ERROR.

   \remarks function not implemented - no FIO_DUSLIC_XT_LASTERR ioctl
*/
IFX_return_t DEVICE_DUSLIC_XT_GetErrnoNum(IFX_int32_t nFd)
{
   IFX_int32_t nLastErr = 0;
   /* IFX_int32_t ret;

   ret = TD_IOCTL(nFd, FIO_DUSLIC_XT_LASTERR, (IFX_int32_t) &nLastErr);

   if (IFX_ERROR == ret)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Failed to get error number for fd %d, (File: %s, line: %d)\n",
             nFd, __FILE__, __LINE__));
      return IFX_ERROR;
   } */
   return nLastErr;

}
#ifdef TD_USE_CHANNEL_INIT
/**
   Downloads fw and bbd files and calls IFX_TAPI_CH_INIT for first channel.

   \param pBoard - pointer to board
   \param pPath - path to FW files for download

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remarks It is necessary to call this function to have working the
    IFX_TAPI_CAP_LIST.
*/
IFX_return_t DEVICE_DUSLIC_XT_setupChannel(BOARD_t* pBoard, IFX_char_t* pPath,
                                           IFX_boolean_t hExtBoard)
{
   IFX_TAPI_CH_INIT_t Init;
   IFX_return_t ret = IFX_SUCCESS;
   IFX_char_t sdev_name[MAX_DEV_NAME_LEN_WITH_NUMBER];
   IFX_int32_t nCh_FD = -1;
   TAPIDEMO_DEVICE_t *pDevice = IFX_NULL;
   TAPIDEMO_DEVICE_CPU_t *pCpuDevice = IFX_NULL;

   /* check input arguments */
   if ((pBoard == IFX_NULL) || (pPath == IFX_NULL))
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Invalid input argument(s). (File: %s, line: %d)\n",
             __FILE__, __LINE__));
      return IFX_ERROR;
   }

   if ((pDevice = Common_GetDevice_CPU(pBoard->pDevice)) == IFX_NULL)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Err, CPU device not found. (File: %s, line: %d)\n",
             __FILE__, __LINE__));
      return IFX_ERROR;
   }
   pCpuDevice = &pDevice->uDevice.stCpu;

   ret = DEVICE_DUSLIC_XT_Fillin_FW_Ptr(pCpuDevice->pDevDependStuff,
                                        pCpuDevice, pPath);
   if (ret == IFX_ERROR)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Error, required binaries files are not downloaded.\n"
            "(File: %s, line: %d)\n",
            __FILE__, __LINE__));      
      return IFX_ERROR;
   }

   /* open channel 0, 10 max len of int32 as string */
   if(MAX_DEV_NAME_LEN_WITH_NUMBER > (strlen(pBoard->pszChDevName) + 2 * 10))
   {
      sprintf(sdev_name, "%s%d%d", (char *) pBoard->pszChDevName,
              (int) pBoard->nID, 1);
   }
   else
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Buffer overflow, array index of 'sdev_name' "
            "may be outside the bounds. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   nCh_FD = Common_Open(sdev_name);
   if (0 > nCh_FD)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid channel (ch 0) file descriptor."
            "(File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /* Set tapi init structure */
   memset(&Init, 0, sizeof(IFX_TAPI_CH_INIT_t));
   Init.nMode = IFX_TAPI_INIT_MODE_DEFAULT;
   Init.pProc = (IFX_void_t*) pCpuDevice->pDevDependStuff;
   ret = TD_IOCTL(nCh_FD, IFX_TAPI_CH_INIT, (IFX_int32_t) &Init);
   if (IFX_SUCCESS != ret)
   {
      /* Don't print error trace for extension board because it may not be error,
         e.g. when extension board node is created, but board isn't connected 
         to the main board */
      if (!hExtBoard)
      {       
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("ERR on ioctl cmd IFX_TAPI_CH_INIT, ch 0 "
                "(File: %s, line: %d)\n", __FILE__, __LINE__));
      }
      TD_OS_DeviceClose(nCh_FD);
      return IFX_ERROR;
   }

   TD_OS_DeviceClose(nCh_FD);
   return IFX_SUCCESS;
}
#else /* TD_USE_CHANNEL_INIT */
/**
   Downloads fw and bbd files to TAPI and starts TAPI.

   \param pBoard - pointer to board
   \param pPath - path to FW files for download

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.
*/
IFX_return_t DEVICE_DUSLIC_XT_setupChannel(BOARD_t* pBoard, IFX_char_t* pPath,
                                           IFX_boolean_t hExtBoard)
{
   IFX_return_t ret = IFX_SUCCESS;
   TAPIDEMO_DEVICE_t *pDevice = IFX_NULL;
   TAPIDEMO_DEVICE_CPU_t *pCpuDevice = IFX_NULL;
   IFX_int32_t nDevFd;
   IFX_TAPI_DEV_START_CFG_t oStartConfig = {0};

   /* check input arguments */
   TD_PTR_CHECK(pBoard, IFX_ERROR);
   TD_PTR_CHECK(pPath, IFX_ERROR);

   if ((pDevice = Common_GetDevice_CPU(pBoard->pDevice)) == IFX_NULL)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Err, CPU device not found. (File: %s, line: %d)\n",
             __FILE__, __LINE__));
      return IFX_ERROR;
   }
   pCpuDevice = &pDevice->uDevice.stCpu;

   ret = DEVICE_DUSLIC_XT_Fillin_FW_Ptr(pCpuDevice->pDevDependStuff,
                                        pCpuDevice, pPath);
   if (ret == IFX_ERROR)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Error, required binaries files are not downloaded.\n"
            "(File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }
   /* open device file descriptor */
   nDevFd = Common_Open(pBoard->pszCtrlDevName);
   if (0 > nDevFd)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid device file descriptor."
            "(File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }
   /* stop TAPI - all resources allocated by TAPI for this specific device
      are freed */
   if (IFX_SUCCESS != TD_IOCTL(nDevFd, IFX_TAPI_DEV_STOP, NO_PARAM))
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Err, Failed to stop. (File: %s, line: %d)\n",
             __FILE__, __LINE__));
      TD_OS_DeviceClose(nDevFd);
      return IFX_ERROR;
   }
   /* Download FW and BBD */
   if (IFX_SUCCESS != TD_IOCTL(nDevFd, FIO_DXT_INIT,
                               pCpuDevice->pDevDependStuff))
   {

      /* Don't print error trace for extension board because it may not be error,
         e.g. when extension board node is created, but board isn't connected 
         to the main board */
      if (!hExtBoard)
      {      
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("Err, Failed download FW and BBD. (File: %s, line: %d)\n",
                __FILE__, __LINE__));
      }
      TD_OS_DeviceClose(nDevFd);
      return IFX_ERROR;
   }
   /* start TAPI - reserve all resources needed by TAPI
      for this specific device. */
   oStartConfig.nMode = IFX_TAPI_INIT_MODE_DEFAULT;
   if (IFX_SUCCESS != TD_IOCTL(nDevFd, IFX_TAPI_DEV_START, &oStartConfig))
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Err, Failed to start device. (File: %s, line: %d)\n",
             __FILE__, __LINE__));
      TD_OS_DeviceClose(nDevFd);
      return IFX_ERROR;
   }
   TD_OS_DeviceClose(nDevFd);
   return IFX_SUCCESS;
}
#endif /* TD_USE_CHANNEL_INIT */
/**
   Register device to the board.

   \param pCpuDevice - pointer to device

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.

   \remarks
*/
IFX_return_t DEVICE_DUSLIC_XT_Register(TAPIDEMO_DEVICE_CPU_t* pCpuDevice)
{
   /* check input arguments */
   if (pCpuDevice == IFX_NULL)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }
   pCpuDevice->Version = DEVICE_DUSLIC_XT_GetVersions;
   pCpuDevice->DecodeErrno = DEVICE_DUSLIC_XT_DecodeErrnoForDrv;
   pCpuDevice->GetErrno = DEVICE_DUSLIC_XT_GetErrnoNum;

   pCpuDevice->FillinFwPtr = DEVICE_DUSLIC_XT_Fillin_FW_Ptr;
   pCpuDevice->ReleaseFwPtr = DEVICE_DUSLIC_XT_Release_FW_Ptr;

   return IFX_SUCCESS;
}


