/******************************************************************************

                              Copyright (c) 2009
                            Lantiq Deutschland GmbH
                     Am Campeon 3; 85579 Neubiberg, Germany

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

  ****************************************************************************/

/**
   \file com_client.c
   \date 2007-05-10
   \brief Implementation of communication module with control PC.

*/

/* ============================= */
/* Includes                      */
/* ============================= */

#include "com_client.h"

/* ============================= */
/* Global Structures             */
/* ============================= */

/** global pointer to ITM control structure */
TD_ITM_CONTROL_t* g_pITM_Ctrl;

/** global buffer for sending ITM messages */
IFX_char_t g_buf[TD_COM_BUF_SIZE];

/* ============================= */
/* Global function declaration   */
/* ============================= */

/* ============================= */
/* Local variable definition     */
/* ============================= */

/* ============================= */
/* Local function declaration    */
/* ============================= */

/* ============================= */
/* Local function definition     */
/* ============================= */

/* ============================= */
/* Global function definition    */
/* ============================= */

/**
   Send message through communication socket

   \param nSocket  - handler to communication socket
   \param msg      - pointer to the message string

   \return IFX_SUCCESS on OK, otherwise IFX_ERROR
*/
IFX_return_t COM_SendMessage(TD_OS_socket_t nSocket, IFX_char_t* msg)
{
   IFX_int32_t size = 0;
   IFX_int32_t ret = 0;

   /* check input parameters */
   TD_PTR_CHECK(msg, IFX_ERROR);

   if (NO_SOCKET == nSocket)
   {
      /* no connection to TCL script do not print trace */
      return IFX_ERROR;
   }
   TRACE(TAPIDEMO, DBG_LEVEL_LOW, ("COM_SendMessage()\n"));

   size = strlen(msg);
   TD_PARAMETER_CHECK((0 > size), size, IFX_ERROR);

   ret = TD_OS_SocketSend(nSocket, msg, size);
   if (ret != size)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Error! No data sent through communication socket,"
             "%s. (File: %s, line: %d)\n",
            strerror(errno), __FILE__, __LINE__));
      TD_OS_SocketClose(nSocket);
      g_pITM_Ctrl->nComOutSocket = NO_SOCKET;
      return IFX_ERROR;
   }

   return IFX_SUCCESS;
}/* COM_SendMessage() */

/**
   Initialize TCP socket used for communication with Control PC

   \param unIp   - IP address
   \param unPort - port number
   \param direction   - determines the direction of socket (in/out)

   \return socket number or NO_SOCKET if error
*/
TD_OS_socket_t COM_InitCommunicationSocket(IFX_uint32_t unIp,
                                        IFX_uint16_t unPort,
                                        IFX_boolean_t direction)
{
   TD_OS_socket_t socFd;
   TD_OS_sockAddr_t my_addr, serv_addr;

   if (COM_DIRECTION_OUT == direction)
   {
      /* Messages sent to the Control PC use TCP*/
      if (IFX_SUCCESS != TD_OS_SocketCreate(TD_OS_SOC_TYPE_STREAM, &socFd))
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("Can't create communication socket, %s."
                "(File: %s, line: %d)\n", strerror(errno), __FILE__, __LINE__));
         return NO_SOCKET;
      }
   }
   else
   {
      /* Messages received from the Control PC use UDP*/
      if (IFX_SUCCESS != TD_OS_SocketCreate(TD_OS_SOC_TYPE_DGRAM, &socFd))
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("Can't create communication socket, %s."
                "(File: %s, line: %d)\n", strerror(errno), __FILE__, __LINE__));
         return NO_SOCKET;
      }
   }

   bzero((IFX_char_t*) &my_addr, (IFX_int32_t) sizeof(my_addr));
   my_addr.sin_family = AF_INET;
   my_addr.sin_port = htons(unPort);

   if (COM_DIRECTION_IN == direction)
   {
      my_addr.sin_addr.s_addr = unIp ? unIp : htonl(INADDR_ANY);
      if (IFX_SUCCESS != TD_OS_SocketBind(socFd, &my_addr))
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
         ("Can't bind communication socket to port, %s. (File: %s, line: %d)\n",
               strerror(errno), __FILE__, __LINE__));
         return NO_SOCKET;
      }
   }

#ifdef LINUX
   /* make the socket non blocking */
   fcntl(socFd, F_SETFL, O_NONBLOCK);
#endif /* LINUX */

   if (COM_DIRECTION_OUT == direction)
   {
       bzero((IFX_char_t*) &serv_addr, (IFX_int32_t) sizeof(serv_addr));
       serv_addr.sin_family = AF_INET;
       serv_addr.sin_port = htons(unPort);
       serv_addr.sin_addr.s_addr = unIp;

       /* We can use TD_OS_SocketConnect, but there will be error 
         "Operation now in progress" from IFX-OS" */
       if (0 > connect(socFd, (struct sockaddr *) &serv_addr, sizeof(serv_addr)))
       {
           /* Socket is non blocking so ignore "Operation now in progress"*/
           if (EINPROGRESS != errno)
           {
               TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                    ("Can't connect with server, %s. (File: %s, line: %d)\n",
                     strerror(errno), __FILE__, __LINE__));
               return NO_SOCKET;
           }
       }
   }

   return socFd;
} /* COM_InitCommunicationSocket() */

/**
   Change communication socket and set rfds array.

   \param pctrl   - pointer to control structure
   \param pRFds   - pointer to fd set

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.  
*/
IFX_return_t COM_ResetCommunicationSockets(CTRL_STATUS_t* pCtrl, 
                                           TD_OS_socFd_set_t* pRFds)
{
   TD_OS_sockAddr_t oNew_IP_Addr;

   /* check input parameters */
   TD_PTR_CHECK(pCtrl, IFX_ERROR);
   TD_PTR_CHECK(pRFds, IFX_ERROR);

   /* if request to change IP address via TCL scripts*/
   if (IFX_TRUE == g_pITM_Ctrl->bChangeIPAddress)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_LOW,
            ("Address of board has been changed.\n"));

      TD_OS_SocFdClr(pCtrl->nAdminSocket, pRFds);
      TD_OS_SocketClose(pCtrl->nAdminSocket);
      pCtrl->nAdminSocket = NO_SOCKET;

      TD_OS_SocFdClr(g_pITM_Ctrl->nComInSocket, pRFds);
      g_pITM_Ctrl->fComIdentSent = IFX_FALSE;
      COM_CleanUpComSockets();
      g_pITM_Ctrl->nComInSocket = NO_SOCKET;

      g_pITM_Ctrl->bChangeIPAddress = IFX_FALSE;

      oNew_IP_Addr.sin_addr.s_addr =
         inet_addr(g_pITM_Ctrl->oVerifySytemInit.aParametrs);
      pCtrl->oTapidemo_IP_Addr = oNew_IP_Addr;
      pCtrl->nMy_IP_Addr =
         ntohl((int)oNew_IP_Addr.sin_addr.s_addr) & 0xffffff00;
      pCtrl->pProgramArg->oMy_IP_Addr.sin_addr.s_addr =
         oNew_IP_Addr.sin_addr.s_addr;


      pCtrl->nAdminSocket = TAPIDEMO_InitAdminSocket();
      g_pITM_Ctrl->nComInSocket =
         COM_InitCommunicationSocket(oNew_IP_Addr.sin_addr.s_addr,
                                     COM_COMMAND_PORT,
                                     COM_DIRECTION_IN);
      g_pITM_Ctrl->fComIdentSent = IFX_FALSE;

      /* Also add administration socket for handling calls */
      if (pCtrl->nAdminSocket == NO_SOCKET)
      {
         /* Without this socket we cannot make calls. */
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("Err, ITM: Could not Reset Communication Socket. "
                "(File: %s, line: %d)\n", __FILE__, __LINE__));
         return IFX_ERROR;
      }
      TD_FD_SET(pCtrl->nAdminSocket, pRFds, pCtrl->nMaxFdSocket, 
                TD_SELECT_SOCKET);

      if(NO_SOCKET != g_pITM_Ctrl->nComInSocket)
      {
         TD_FD_SET(g_pITM_Ctrl->nComInSocket, pRFds, pCtrl->nMaxFdSocket, 
                   TD_SELECT_SOCKET);
      }
   } /* if (IFX_TRUE == pCtrl->bChangeIPAddress) */
   return IFX_SUCCESS;
} /* COM_InitCommunicationSocket() */



/**
   Close communication sockets

   \param none

   \return IFX_void_t
*/
IFX_void_t COM_CleanUpComSockets(IFX_void_t)
{
   if (IFX_NULL == g_pITM_Ctrl)
   {
      return;
   }
   TRACE(TAPIDEMO, DBG_LEVEL_LOW,
         ("Close ITM communication sockets.\n"));

   if (NO_SOCKET != g_pITM_Ctrl->nComInSocket)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_LOW, ("Closing command socket\n"));
      TD_OS_SocketClose(g_pITM_Ctrl->nComInSocket);
      g_pITM_Ctrl->nComInSocket = NO_SOCKET;
   }
   if (NO_SOCKET != g_pITM_Ctrl->nComOutSocket)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_LOW, ("Closing connection with Control PC\n"));
      TD_OS_SocketClose(g_pITM_Ctrl->nComOutSocket);
      g_pITM_Ctrl->nComOutSocket = NO_SOCKET;
   }
   return;
}/*  */


