/******************************************************************************

                              Copyright (c) 2009
                            Lantiq Deutschland GmbH
                     Am Campeon 3; 85579 Neubiberg, Germany

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

  ****************************************************************************/

/**
   \file dev_help.c
   \date 2007-08-20
   \brief Implementation of developer help functions.

*/

/* ============================= */
/* Includes                      */
/* ============================= */

#include "common.h"
#include "tapidemo.h"

/* ============================= */
/* Local Structures              */
/* ============================= */

#ifdef TD_IOCTL_TRACE
/** holds numbr of FD its name and pointer to next structure */
typedef struct _TD_NAME_OF_FD_t
{
   /** FD number */
   IFX_int32_t nFd;
   /** name of FD */
   IFX_char_t acName[DEV_NAME_LEN];
   /** pointer to next structure */
   struct _TD_NAME_OF_FD_t *pNext;
} TD_NAME_OF_FD_t;
#endif /* TD_IOCTL_TRACE */

/* ============================= */
/* Global function declaration   */
/* ============================= */

/* ============================= */
/* Local variable definition     */
/* ============================= */

#ifdef TD_IOCTL_TRACE
/** pointer only available in this file to hold list of FDs with names */
static TD_NAME_OF_FD_t* g_FdNameList = IFX_NULL;
#endif /* TD_IOCTL_TRACE */

/* ============================= */
/* Local function declaration    */
/* ============================= */

/* ============================= */
/* Local function definition     */
/* ============================= */

/* ============================= */
/* Global function definition    */
/* ============================= */

/**
   Function to print Data Channels data.

   \param pBoard  - pointer to board structure

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.
*/
IFX_return_t TD_DEV_PrintMappedDataCh(BOARD_t* pBoard)
{
   IFX_int32_t nCounter;

   if (IFX_NULL == pBoard)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("MAPPING : BOARD : wrong input argument."
             "(File: %s, line: %d)\n", __FILE__, __LINE__));
      return IFX_ERROR;
   }

   for (nCounter = 0; nCounter < pBoard->nMaxCoderCh; nCounter++)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_NORMAL,
            ("MAPPING : BOARD : fMapped(%d), nDataCh(%d), "
             "nSocket(%d), nDataCh_Fd(%d) \n",
             pBoard->pDataChStat[nCounter].fMapped,
             nCounter,
             pBoard->rgnSockets[nCounter],
             pBoard->nCh_FD[nCounter]));
   }

   return IFX_SUCCESS;
} /* */

/**
   Function to print phones connections data.

   \param pBoard  - pointer to board structure

   \return IFX_SUCCESS if successful, otherwise IFX_ERROR.
*/
IFX_return_t TD_DEV_PrintConnections(BOARD_t* pBoard)
{
   IFX_int32_t i, j;

   if (IFX_NULL == pBoard)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("MAPPING : BOARD : wrong input argument."
             "(File: %s, line: %d)\n", __FILE__, __LINE__));
      return IFX_ERROR;
   }

   for (i = 0; i < pBoard->nMaxAnalogCh; i++)
   {
      if (0 < pBoard->rgoPhones[i].nConnCnt)
      {
         for (j = 0; j < pBoard->rgoPhones[i].nConnCnt; j++)
         {
            TRACE(TAPIDEMO, DBG_LEVEL_NORMAL,
                  ("PHONE(%d, %d) : CONNECTION : fActive(%d), nType(%d),"
                   " nUsedCh(%d), nUsedSocket(%d), nUsedCh_Fd(%d).\n",
                   i, j,
                   pBoard->rgoPhones[i].rgoConn[j].fActive,
                   pBoard->rgoPhones[i].rgoConn[j].nType,
                   pBoard->rgoPhones[i].rgoConn[j].nUsedCh,
                   pBoard->rgoPhones[i].rgoConn[j].nUsedSocket,
                   pBoard->rgoPhones[i].rgoConn[j].nUsedCh_FD));
         }
      }
      else
      {
         TRACE(TAPIDEMO, DBG_LEVEL_NORMAL,
               ("PHONE(%d) : Number of connections is 0.\n", i));
      }
   }

   return IFX_SUCCESS;
} /* */

#ifdef TD_IOCTL_TRACE
/* 
   Purpose of this functions is to be able to easily track usage of ioctls().
   Thanks to TD_IOCTL macro in all places where ioctl() was used TAPIDEMO will
   print trace with ioctl name, file name and line number, TD_IOCTL uses the
   same arguments as ioctl - this made the changes quicker, also for improving
   performance this macro can directly call ioctl() instead of calling debug
   function, furthermore it will be helpfull for tests (ITM) - info about
   ioctls() called during test will be send to test scripts so all tapi ioctls
   couold be listed after test end

   ioctl trace is listing all ioctls used, it can help
   to confirm if right set of ioctls was used.
   */

/**
   Gets name of FD if available.

   \param nFD        - number of FD

   \return pointer to string or IFX_NULL if FD is not registered
*/
IFX_char_t* TD_DEV_IoctlGetNameOfFd(IFX_int32_t nFd)
{
   /* get pointer to structure list */
   TD_NAME_OF_FD_t* pNext = g_FdNameList;

   /* check untill the last one (pNext->pNext is set to IFX_NULL) */
   while (IFX_NULL != pNext)
   {
      /* check FD number */
      if (pNext->nFd == nFd)
      {
         /* return pointer to name */
         return pNext->acName;
      }
      /* check next */
      pNext = pNext->pNext;
   } /* while (IFX_NULL != pNext) */
   /* no name was registered for this FD */
   return IFX_NULL;
}

/**
   Set name for given FD. Such name can be retrived with
   TD_DEV_IoctlGetNameOfFd().

   \param nFD        - number of FD
   \param pszFdName  - pointer to zero terminated string with name for FD

   \return IFX_SUCCESS on ok, otherwise IFX_ERROR
*/
IFX_return_t TD_DEV_IoctlSetNameOfFd(IFX_int32_t nFd, IFX_char_t* pszFdName)
{
   /* get pointer to structure list */
   TD_NAME_OF_FD_t* pNext = g_FdNameList;
   TD_NAME_OF_FD_t* pTmpNext;
   IFX_int32_t nStringLen = 0;

   /* check input parameters */
   TD_PTR_CHECK(pszFdName, IFX_ERROR);

   /* open function returned error - this error will not be handled here */
   if (0 > nFd)
   {
      return IFX_SUCCESS;
   }

   /* get string length */
   nStringLen = strlen(pszFdName);

   /* check string length */
   if (0 >= nStringLen ||
       DEV_NAME_LEN <= nStringLen)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Err: Invalid string length %d for fd %d. (File: %s, line: %d)\n",
             nStringLen, nFd, __FILE__, __LINE__));
      return IFX_ERROR;
   }

#if 0
/* debug trace - enable in case of problems after code modifications */
   /* print data of FD that will be added */
   TRACE(TAPIDEMO, DBG_LEVEL_LOW,
         ("FD %d: %s\n", nFd, pszFdName));
#endif

   /* get last pointer or structure with the same FD number */
   if (IFX_NULL != pNext)
   {
      if (pNext->nFd == nFd)
      {
         /* if the same FD number then set new name */
         strncpy(pNext->acName, pszFdName, DEV_NAME_LEN);
         return IFX_SUCCESS;
      }
      while (IFX_NULL != pNext->pNext)
      {
         pNext = pNext->pNext;
         if (pNext->nFd == nFd)
         {
            /* if the same FD number then set new name */
            strncpy(pNext->acName, pszFdName, DEV_NAME_LEN);
            return IFX_SUCCESS;
         }
      } /* while (IFX_NULL != pNext->pNext) */
   } /* if (IFX_NULL != pNext) */
   /* allocate memory */
   pTmpNext = calloc(1, sizeof(TD_NAME_OF_FD_t));

   /* check if memory is allocated */
   if (IFX_NULL == pTmpNext)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Err: Failed to allocate memory for %s. (File: %s, line: %d)\n",
             pszFdName, __FILE__, __LINE__));
      return IFX_ERROR;
   }
   /* check if first pointer is set */
   if (IFX_NULL == g_FdNameList)
   {
      g_FdNameList = pTmpNext;
   }
   else
   {
      pNext->pNext = pTmpNext;
   }
   /* set structure */
   pTmpNext->nFd = nFd;
   pTmpNext->pNext = IFX_NULL;
   strncpy(pTmpNext->acName, pszFdName, DEV_NAME_LEN);

   return IFX_SUCCESS;
}

/**
   Cleanup structures for ioctl() trace.

   \param none

   \return IFX_SUCCESS on ok, otherwise IFX_ERROR
*/
IFX_return_t TD_DEV_IoctlCleanUp(IFX_void_t)
{
   /* get pointer to structure list */
   TD_NAME_OF_FD_t* pNext = g_FdNameList;
   TD_NAME_OF_FD_t* pTemp = IFX_NULL;

   /* for all allocated structures */
   while (IFX_NULL != pNext)
   {
      /* get next pointer */
      pTemp = pNext->pNext;
      /* free memory */
      TD_OS_MemFree(pNext);
      /* set to next */
      pNext = pTemp;
   }
   return IFX_SUCCESS;
}

/**
   Can be used insted of ioctl() to print info about called ioctl() that should
   be called. In future also error handling can be moved in here.

   \param nFD        - number of FD for ioctl to be called on
   \param nIoctl     - ioctl number
   \param pArg       - ioctl argument
   \param pszIoctl   - ioctl name (string)
   \param pszFile    - file name where function was used
   \param nLine      - file line where function was used

   \return whatever ioctl returns
*/
IFX_int32_t TD_DEV_Ioctl(IFX_int32_t nFd, IFX_int32_t nIoctl, IFX_int32_t pArg,
                         IFX_char_t* pszIoctl,
                         IFX_char_t* pszFile, IFX_int32_t nLine)
{
   IFX_char_t* pFdName = IFX_NULL;

   /* print ioctl name */
   TRACE(TAPIDEMO, DBG_LEVEL_LOW,
         ("Calling ioctl %s on ", pszIoctl));

   /* get FD name */
   pFdName = TD_DEV_IoctlGetNameOfFd(nFd);

   /* check if name is available */
   if (IFX_NULL != pFdName)
   {
      /* print FD name */
      TRACE(TAPIDEMO, DBG_LEVEL_LOW,
            ("%s", pFdName));
   }
   else
   {
      /* print FD number */
      TRACE(TAPIDEMO, DBG_LEVEL_LOW,
            ("FD %d", nFd));
   }

   /* print file name and line number */
   TRACE(TAPIDEMO, DBG_LEVEL_LOW,
         ("\n   in file: %s on line: %d\n", pszFile, nLine));

   /* call ioctl() */
   return TD_OS_DeviceControl(nFd, nIoctl, pArg);
}
#endif /* TD_IOCTL_TRACE */



