
/******************************************************************************

                              Copyright (c) 2009
                            Lantiq Deutschland GmbH
                     Am Campeon 3; 85579 Neubiberg, Germany

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

  ****************************************************************************
   Module      : parse_cmd_arg.c
   Description : Parses command line arguments
*******************************************************************************/

/**
   \file parse_cmd_arg.c

   This file contains implementation of parsing command line arguments.
*/

/* ============================= */
/* Includes                      */
/* ============================= */

#include "ifx_types.h"
/*#include "sys_drv_debug.h"*/
#include "common.h"
#include "tapidemo.h"
#include "parse_cmd_arg.h"
#include "cid.h"

#ifdef FXO
#include "common_fxo.h"
#endif /* FXO */

/* ============================= */
/* Debug interface               */
/* ============================= */

/* ============================= */
/* Global Structures             */
/* ============================= */

/** Command line parameter delimiter */
const IFX_char_t* ARG_DELIMITER = " ";

/** Minimum valid port number. */
enum { MIN_PORT_NUM = 0 };

/** Maximum valid port number. */
enum { MAX_PORT_NUM = 65535 };

/** Default port for syslog. */
enum { DEFAULT_SYSLOG_PORT = 514 };

/** Max length of each argument */
enum { MAX_ARGUMENT_LEN = 256 };

/** Delimiter between ip addrres and port number. */
const IFX_char_t IP_ADDR_PORT_DELIM = ':';

/** Name of running application */
const IFX_char_t* PROGRAM_NAME = "tapidemo";

#ifndef LINUX
/** Structure holding description of each argument. */
struct option
{
   const char* name;
   int has_arg;
   int* flag;
   int val;
};
#endif /* LINUX */

enum
{
#ifdef TD_RM_DEBUG
   /** option value for additional RM debug (resource counting). */
   TAPIDEMO_OPTION_NUMBER_RM_DBG = 298,
#endif /* TD_RM_DEBUG */
#ifdef TD_DECT
   /** option value for DECT debug level set. */
   TAPIDEMO_OPTION_NUMBER_DECT_DBG = 299,
#endif /* TD_DECT */
   /** option value for VAD. */
   TAPIDEMO_OPTION_NUMBER_VAD = 300,
   /** option value for lec. */
   TAPIDEMO_OPTION_NUMBER_LEC = 301,
   /** option value for no lec. */
   TAPIDEMO_OPTION_NUMBER_NO_LEC = 302,
   /** option value for ground start. */
   TAPIDEMO_OPTION_NUMBER_GROUND_START = 303,
   /** print Tapidemo version */
   TAPIDEMO_OPTION_PRINT_VERSION = 304,
#ifdef LINUX
   /** print Tapidemo version */
   TAPIDEMO_OPTION_PRINT_CONFIGURE = 305
#endif
};

/* ============================= */
/* Global function declaration   */
/* ============================= */

/* ============================= */
/* Local variable definition     */
/* ============================= */

/** names of debug levels */
TD_ENUM_2_NAME_t TD_rgDebugLevelName[] =
{
   {DBG_LEVEL_LOW,      "LOW"},
   {DBG_LEVEL_NORMAL,   "NORMAL"},
   {DBG_LEVEL_HIGH,     "HIGH"},
   {DBG_LEVEL_OFF,      "OFF"},
   {TD_MAX_ENUM_ID,     "DEBUG_LEVEL_t"}
};

/** Command line arguments */
static struct option PROGRAM_OPTIONS[] =
{
   /*    name, has_arg, *flag, val */
   {"help"                , 0, 0, 'h'},
   {"version"             , 0, 0, TAPIDEMO_OPTION_PRINT_VERSION},
#ifdef LINUX
   {"configure"           , 0, 0, TAPIDEMO_OPTION_PRINT_CONFIGURE},
   {"daemon_logfile"      , 2, 0, 'r'},
   {"daemon_syslog"       , 2, 0, 's'},
#endif /* LINUX */
   {"debug"               , 1, 0, 'd'},
   {"ip-address"          , 1, 0, 'i'},
   {"wait"                , 0, 0, 'w'},
#ifdef VXWORKS
   {"qos"                 , 1, 0, 'q'},
#else
   {"qos"                 , 2, 0, 'q'},
#endif /* VXWORKS */
   {"cid"                 , 1, 0, 'c'},
   {"conferencing"        , 1, 0, 'k'},
   {"pcm"                 , 1, 0, 'p'},
   {"encoder_type"        , 1, 0, 'e'},
   {"packetization_time"  , 1, 0, 'f'},
   {"vad"                 , 1, 0, TAPIDEMO_OPTION_NUMBER_VAD},
   {"lec"                 , 1, 0, TAPIDEMO_OPTION_NUMBER_LEC},
   {"nolec"               , 0, 0, TAPIDEMO_OPTION_NUMBER_NO_LEC},
   {"different_network"   , 1, 0, 'n'},
   {"board_combination"   , 1, 0, 'b'},
#ifdef LINUX
   {"FXO"                 , 2, 0, 'x'},
   {"path_to_FW_files"    , 1, 0, 'l'},
#endif /* LINUX */
   /* karol - workaround for teridian extension board */
   {"Not_DuSLIC_FXO"      , 0, 0, 'g'},
   /* end workaround */
#if defined(XT16) || defined(WITH_VXT)
   /* ground start */
   {"gs"                  , 0, 0, TAPIDEMO_OPTION_NUMBER_GROUND_START},
#endif /* XT16 || WITH_VXT */
   /* dect debug flag */
   {"dect"                , 0, 0, 't'},
   /* from here additional arguments start - not listed in help */
#ifdef TD_DECT
   /* dect debug argument */
   {"dect_dbg"            , 1, 0, TAPIDEMO_OPTION_NUMBER_DECT_DBG},
#endif /* TD_DECT */
#ifdef TD_RM_DEBUG
   /* rm debug argument */
   {"rm_dbg"              , 0, 0, TAPIDEMO_OPTION_NUMBER_RM_DBG},
#endif /* TD_RM_DEBUG */
   /* start with ITM enabled (Internal Test Module) */
   {"itm"                 , 0, 0, 'm'},
   /* ITM modular test case - Verify System Initialization.
      When used program waits with initialization for control PC connection. */
   {"initialize_system"   , 0, 0, 'v'},
   {0                     , 0, 0, 0 }
};

/** string with short command line arguments */
const IFX_char_t OPTION_PARAMETERS[] =
#ifdef LINUX
   /* LINUX specific flags */
                                       "r::s::x::l:q::"
#else
   /* VxWorks specific flags
      VxWorks doesn't have filesystem so remove arguments working with file. */
                                       "q:"
#endif /* LINUX */
   /* common flags */
                                       "hd:i:wc:k:p:e:f:n:b:gtmv";

/** Strings describing command line parameters.
   {""} is delimiter between help message for one argument and its flag
   for end of table. */
static const IFX_char_t helpItems[][200] =
{
   {"Help screen"},
   {""},
   {"Print version"},
   {""},
#ifdef LINUX
   {"Print compilation arguments"},
   {""},
   {"Run as a daemon with trace redirection to file (default is OFF - 0)"},
      {"Set path to log file (by default is /opt/work/ifx/bin/tapidemo.log) <-r/path/name.log>"},
   {""},
   {"Run as a daemon with trace redirection to syslog (default is OFF - 0)"},
      {"Specify network IP address and port of remote syslog server "},
      {"(by default local syslog is used).<-sxxx.xxx.xxx.xxx:xxxx>"},
#endif /* LINUX */
   {""},
   {"Set debug level:"},
      {"4 - OFF"},
      {"3 - HIGH (default)"},
      {"2 - NORMAL"},
      {"1 - LOW"},
   {""},
   {"Specify network IP address of the board for connections, by default "},
      {"retrieve it automatically. Syntax is <xxx.xxx.xxx.xxx>"},
   {""},
   {"Wait after each state machine step (default is OFF - 0)"},
   {""},
   {"Qos support : packets redirected via udp redirector (default is OFF - 0)"},
   {""},
   {"CID support (default ON): showing CID on phone LCD:"},
      {" 0 - TELCORDIA, Belcore, USA"},
      {" 1 - ETSI FSK, Europe (default)"},
      {" 2 - ETSI DTMF, Europe"},
      {" 3 - SIN BT, Great Britain"},
      {" 4 - NTT, Japan"},
#ifndef TAPI_VERSION4
      {" 5 - KPN DTMF"},
      {" 6 - KPN DTMF+FSK"},
#endif /* TAPI_VERSION4 */
      {"10 - turn OFF CID support"},
   {""},
   {"Conferencing support (default is ON - 1): more than two phone connection."},
      {"HASH - '#' is used to start another call."},
   {""},
   {"PCM (default is OFF): using two boards connected over PCM."},
      {"Start/stop uses ethernet, but voice stream uses PCM:"},
      {" - One board is master <-p m>,"},
      {" - The other one is slave <-p s>"},
      {" - Using PCM on one board (will be master with PCM local loop) <-p l>"},
   {""},
   {"Encoder type (default is IFX_TAPI_ENC_TYPE_MLAW):"},
      {" 1 - IFX_TAPI_ENC_TYPE_G723_63 [G723, 6.3 kBit/s]"},
      {" 2 - IFX_TAPI_ENC_TYPE_G723_53 [G723, 5.3 kBit/s]"},

      {" 7 - IFX_TAPI_ENC_TYPE_G729_AB [G729 A and B (silence compression),"
             " 8 kBit/s]"},
      {" 8 - IFX_TAPI_ENC_TYPE_MLAW [G711 u-law, 64 kBit/s]"},
      {" 9 - IFX_TAPI_ENC_TYPE_ALAW [G711 A-law, 64 kBit/s]"},
      {"12 - IFX_TAPI_ENC_TYPE_G726_16 [G726, 16 kBit/s]"},
      {"13 - IFX_TAPI_ENC_TYPE_G726_24 [G726, 24 kBit/s]"},
      {"14 - IFX_TAPI_ENC_TYPE_G726_32 [G726, 32 kBit/s]"},
      {"15 - IFX_TAPI_ENC_TYPE_G726_40 [G726, 40 kBit/s]"},
      {"16 - IFX_TAPI_ENC_TYPE_G729_E [G729 E, 11.8 kBit/s]"},
      {"17 - IFX_TAPI_ENC_TYPE_ILBC_133 [iLBC, 13.3 kBit/s]"},
      {"18 - IFX_TAPI_ENC_TYPE_ILBC_152 [iLBC, 15.2 kBit/s]"},
      {"21 - IFX_TAPI_COD_TYPE_AMR_4_75 [AMR, 4.75 kBit/s]"},
      {"22 - IFX_TAPI_COD_TYPE_AMR_5_15 [AMR, 5.15 kBit/s]"},
      {"23 - IFX_TAPI_COD_TYPE_AMR_5_9 [AMR, 5.9 kBit/s]"},
      {"24 - IFX_TAPI_COD_TYPE_AMR_6_7 [AMR, 6.7 kBit/s]"},
      {"25 - IFX_TAPI_COD_TYPE_AMR_7_4 [AMR, 7.4 kBit/s]"},
      {"26 - IFX_TAPI_COD_TYPE_AMR_7_95 [AMR, 7.95 kBit/s]"},
      {"27 - IFX_TAPI_COD_TYPE_AMR_10_2 [AMR, 10.2 kBit/s]"},
      {"28 - IFX_TAPI_COD_TYPE_AMR_12_2 [AMR, 12.2 kBit/s]"},
   {""},
   {"Packetisation time (default is 10 ms): Length of frames in milliseconds"},
      {" 1 - 2.5 ms"},
      {" 2 - 5 ms"},
      {" 3 - 5.5 ms"},
      {" 4 - 10 ms"},
      {" 5 - 11 ms"},
      {" 6 - 20 ms"},
      {" 7 - 30 ms"},
      {" 8 - 40 ms"},
      {" 9 - 50 ms"},
      {"10 - 60 ms"},
   {""},
   {"Voice Activity Detection (VAD) configuration:"},
      {" 0 - IFX_TAPI_ENC_VAD_NOVAD - no voice activity detection (default)"},
      {" 1 - IFX_TAPI_ENC_VAD_ON - voice activity detection on"},
      {" 2 - IFX_TAPI_ENC_VAD_G711 - voice activity detection on without "
       "spectral information"},
      {" 4 - IFX_TAPI_ENC_VAD_SC_ONLY - voice activity detection on without "
       "comfort noise generation"},
  {""},
  {"LEC Type configuration:"},
         {" 0 - FX_TAPI_WLEC_TYPE_OFF - LEC and Echo Suppressor turned off."},
         {" 1 - IFX_TAPI_WLEC_TYPE_NE - LEC using fixed window. No Echo Suppressor "
          "(default)."},
         {" 2 - IFX_TAPI_WLEC_TYPE_NFE - LEC using fixed and moving window.  "
          "No Echo Suppressor."},
         {" 3 - IFX_TAPI_WLEC_TYPE_NE_ES - LEC using fixed window + Echo Suppressor."},
         {" 4 - IFX_TAPI_WLEC_TYPE_NFE_ES - LEC using fixed and moving window + "
          "Echo Suppressor."},
         {" 5 - IFX_TAPI_WLEC_TYPE_ES - Echo Suppressor."},
   {""},
   {"LEC is not enabled (by default LEC is enabled during call) - obsolete"},
   {""},
   {"Different network calls: (default is same network address),"},
      {"otherwise first 3 numbers of ip adrress are input <xxx.xxx.xxx>."},
   {""},
   {"Board combination to use (main board is the board for which TAPIDEMO"},
      {" was compiled, extension board is autodetected):"},
      {" 0 - no autodetection of the extension board (1 board)"},
      {" 1 - reserved - obsolete"},
      {" 2 - one EASY50712 (1 board) - obsolete"},
      {" 3 - one EASY50712 and one EASY50510 (2 boards) - obsolete"},
      {" 4 - one EASY3201 (1 board) - obsolete"},
      {" 5 - one EASY3201 with one EASY3111 (2 boards) - obsolete"},
      {" 6 - one EASY80800 (1 board) - obsolete"},
      {" 7 - one EASY508xx (1 board) - obsolete"},
      {" 8 - one EASY336 (1 board) - obsolete"},
      {" 9 - one EASY336 and one XT16 (2 boards) - obsolete"},
      {"10 - one XT16 (1 board) - obsolete"},
      {"11 - one EASY80910 (1 board) - obsolete"},
#ifdef LINUX
   {""},
   {"FXO support:"},
      {" no value - all FXO devices are automatically detected"},
      {" t - only Teridian FXO is used"},
      {" s - only FXO connected to SLIC121 is used"},
      {" All values can be used together e.g. -xst. Sequence not important."},
      {"  NOTE: this is the optional parameter and space between parameter"},
      {"        and value is not allowed. Correct example: -xs or -xts etc."},
   {""},
   {"Path to FW, BBD files."},
#endif /* LINUX */
/* karol -workaround for teridian extension board */
   {""},
   {"Not DuSLIC FXO."},
#if defined(XT16) || defined(WITH_VXT)
   {""},
   {"Ground Start support."},
#endif /* XT16 || WITH_VXT */
   {""},
   {"Enable DECT support."},
   {""},
   {""},
   /* data below two empty strings will not be printed */
   {"Modular test case - Verify System Initialization."},
   {""},
};

/** Max. number of possible command line arguments that follow application. */
#define TD_CMD_MAX_ARGS  32

/** List of all arguments. */
static IFX_char_t* pCmdArgs[TD_CMD_MAX_ARGS] = {0};


/* ============================= */
/* Local function declaration    */
/* ============================= */
IFX_return_t ReadOptions(IFX_int32_t nArgCnt,
                         IFX_char_t* pArgv[],
                         PROGRAM_ARG_t* pProgramArg);
IFX_int32_t GetArgCnt(IFX_char_t* pParam,
                      IFX_char_t* pCmdArgs[]);

#ifndef LINUX
static int getopt_long(int argc, char* const* argv, const char* shortopts,
                       const struct option* longopts, int* longindex);
#endif /* LINUX */

/* ============================= */
/* Local function definition     */
/* ============================= */

/** Used when reading command line arguments, to store argument value. */
static IFX_char_t* ifx_optarg = IFX_NULL;

#ifndef LINUX

/** Option index */
static IFX_int32_t optidx = 0;

/**
   Parse array of strings holding command line arguments.

   \param argc - number of arguments (basically number of words in string)
   \param argv - array of arguments (first one is the name of started program and
                        then parameters are following)
   \param shortopts - Mask which parameters are handled
   \param longopts - Command line arguments
   \param (int *) longindex)
*/
static int getopt_long(int argc, char* const* argv, const char* shortopts,
                       const struct option* longopts, int* longindex)
{
   IFX_int32_t j = 0;
   IFX_char_t* token = IFX_NULL;
   IFX_char_t short_arg = 0;
   IFX_char_t* long_arg = IFX_NULL;
   IFX_boolean_t valid_arg = IFX_FALSE;
   IFX_boolean_t need_val = IFX_FALSE;
   IFX_char_t* arg_location = IFX_NULL;

   /* check input parameters */
   TD_PTR_CHECK(argv, -1);
   TD_PTR_CHECK(shortopts, -1);
   TD_PTR_CHECK(longopts, -1);
   TD_PARAMETER_CHECK((argc <= 0), argc, -1);

   if ((optidx < 0) || (optidx >= argc))
   {
      optidx = 0;
      /* Index outside of argument list, but could also end correct, if
         we parse all the arguments and came to an end. */
      /*printf("Argument index outside argument list or at the end of argument "
             "list. (File: %s, line: %d)\n", __FILE__, __LINE__);*/
      return -1;
   }

   if (optidx == 0)
   {
      /* Skip program name */
      optidx++;
   }

   token = argv[optidx];
   if (token == IFX_NULL)
   {
      TAPIDEMO_PRINTF(("In argument list NULL value. (File: %s, line: %d)\n",
             __FILE__, __LINE__));
      return -1;
   }

   /* Valid argument must have '-' or '--' infront and optional next token
      has value. */
   if ((strlen(token) == 2) && (token[0] == '-') && (token[1] != '-'))
   {
      /* Have short argument, only one char {-<n>} */
      short_arg = token[1];
      /* Check if exists */
      arg_location = strchr(shortopts, short_arg);
      if (arg_location != IFX_NULL)
      {
         TAPIDEMO_PRINTF(("Valid short arg %c\n", short_arg));
         valid_arg = IFX_TRUE;
         /* set flag if value follows and we are not outside of string */
         if (((shortopts + strlen(shortopts)) > arg_location)
             && (*(arg_location + 1) == ':'))
         {
            need_val = IFX_TRUE;
         }
      }
   }
   else if ((strlen(token) > 2) && (token[0] == '-') && (token[1] == '-'))
   {
      /* Long argument, string {--<nnnnnn>}, using longopts */
      long_arg = &token[2];

      j = 0;
      while ((longopts[j].name != IFX_NULL) && (strlen(longopts[j].name) != 0))
      {
         if (!strncmp(long_arg, longopts[j].name, strlen(longopts[j].name)))
         {
            TAPIDEMO_PRINTF(("Valid long arg %s\n", long_arg));
            valid_arg = IFX_TRUE;
            short_arg = longopts[j].val;
            *longindex = (int) &longopts[j];
            if (longopts[j].has_arg == 1)
            {
               need_val = IFX_TRUE;
            }
            break;
         }
      }
   }
   else
   {
      /* Wrong syntax of argument */
      TAPIDEMO_PRINTF(("Wrong argument syntax. (File: %s, line: %d)\n",
             __FILE__, __LINE__));
      return -1;
   }

   /* Jump to next argument or value */
   optidx++;

   /* Check if value is needed for this argument and retrieve it. */
   if (need_val)
   {
      if (optidx > argc)
      {
         /* Error, value is missing */
         ifx_optarg = IFX_NULL;
         TAPIDEMO_PRINTF(("Value for arg is missing. (File: %s, line: %d)\n",
                __FILE__, __LINE__));
         return -1;
      }
      else
      {
         ifx_optarg = argv[optidx];
      }
      /* Move to next argument */
      optidx++;
   }

   return short_arg;
}
#endif /* LINUX */

/* ============================= */
/* Global function definition    */
/* ============================= */


/**
   Return array of arguments.

   \return 0 - ERROR or no words, 1 - only program name, >1 counted words
*/
IFX_char_t** GetCmdArgsArr(IFX_void_t)
{
   return &pCmdArgs[0];
}


/**
   Count all words separated with delimiter and returns number of them.

   \param pParam - string containing all words
   \param pCmdArgs - array of strings with first one as application name

   \return 0 - ERROR or no words, 1 - only program name, >1 counted words
*/
IFX_int32_t GetArgCnt(IFX_char_t* pParam, IFX_char_t* pCmdArgs[])
{
   IFX_int32_t cnt = 0;
   IFX_char_t* token = IFX_NULL;

   /* check input parameters */
   TD_PTR_CHECK(pCmdArgs, 0);
   TD_PTR_CHECK(pParam, 0);

   TAPIDEMO_PRINTF(("GetArgCnt(): Get word count in command "
         "line arguments list %s.\n", pParam));

   /* Add program name to first place */
   pCmdArgs[cnt] = (IFX_char_t *) &PROGRAM_NAME[0];
   cnt++;

   /* Convert ordinary string into array of strings */
   token = (char *) strtok(pParam, ARG_DELIMITER);
   while (token != IFX_NULL)
   {
      pCmdArgs[cnt] = token;
      cnt++;
      token = (char *) strtok(IFX_NULL, ARG_DELIMITER);
   }

   TAPIDEMO_PRINTF(("GetArgCnt(): Got %d words count in command "
          "line arguments list.\n", (int) cnt));

   return cnt;
}

/**
   Reads syslog argument.
 
   \param pProgramArg - pointer to program arguments structure

   \return IFX_SUCCESS argument is ok, otherwise IFX_ERROR
*/
IFX_return_t ReadOptions_Syslog(PROGRAM_ARG_t* pProgramArg)
{
   /* Formated command line options */
   IFX_char_t psCmdOption[MAX_ARGUMENT_LEN];
   IFX_char_t* port_num = IFX_NULL;
   IFX_int32_t arg_value = 0;
   IFX_uint32_t nIpAddrrLen;
   IFX_char_t psIpAddress[IP_ADDRESS_LEN];

   /* check input parameters */
   TD_PTR_CHECK(pProgramArg, IFX_ERROR);

   pProgramArg->oArgFlags.nDaemon= 1;
   if (ifx_optarg != IFX_NULL)
   {
      if(strnlen(ifx_optarg, MAX_ARGUMENT_LEN) == MAX_ARGUMENT_LEN)
      {
         TRACE (TAPIDEMO, DBG_LEVEL_HIGH,
                ("Program argument is incorrect. It is either too long "
                 "or it is non null terminated string\n (File: %s, line: %d)\n",
                 __FILE__, __LINE__));
         return IFX_ERROR;
      }
      /* Format command line option to string */
      snprintf(psCmdOption, MAX_ARGUMENT_LEN, "%s", ifx_optarg);

      if (strlen(psCmdOption) > IP_ADDRESS_LEN + PORT_NUM_LEN + 1)

      {
         TAPIDEMO_PRINTF(("Too long ip address with port number %s "
                          "right syntax is <xxx.xxx.xxx.xxx:nnnnn>>.\n",
                          psCmdOption));
         return IFX_ERROR;
      }

      /* Set port */
      port_num = strchr(psCmdOption, IP_ADDR_PORT_DELIM);
      if ((port_num == IFX_NULL) || (strlen(port_num) == 1))
      {
         /* Use default port number */
         arg_value = DEFAULT_SYSLOG_PORT;
         oTraceRedirection.oSyslog_IP_Addr.sin_port = htons(arg_value);
         nIpAddrrLen = strlen(psCmdOption);
      }
      else
      {
         arg_value = atoi((port_num + 1));
         if ((arg_value > MAX_PORT_NUM) || (arg_value < MIN_PORT_NUM))
         {
            TAPIDEMO_PRINTF(("Port number outside boundaries [%d..%d].\n",
                             MIN_PORT_NUM, MAX_PORT_NUM));
            return IFX_ERROR;
         }
         else
         {
            oTraceRedirection.oSyslog_IP_Addr.sin_port = htons(arg_value);
         }
         /* get address length */
         nIpAddrrLen = strlen(psCmdOption) - strlen(port_num);
      }

      /* Set ip address */
      if (IP_ADDRESS_LEN < nIpAddrrLen+1) 
      {
         TAPIDEMO_PRINTF(("Too long ip address."
                          "Right syntax is <xxx.xxx.xxx.xxx>.\n")); 
         return IFX_ERROR;
      }

      strncpy(psIpAddress, psCmdOption, nIpAddrrLen);
      psIpAddress[nIpAddrrLen]=0;

      /* In Linux returns 0 if address is invalid. */
      if (IFX_SUCCESS != TD_OS_SocketAton(psIpAddress,
                                &oTraceRedirection.oSyslog_IP_Addr))
      {
          TAPIDEMO_PRINTF(("Wrong format of IP address %s, it should be"
                           " <xxx.xxx.xxx.xxx>.\n", psIpAddress));
          return IFX_ERROR;
      }

      pProgramArg->nTraceRedirection = TRACE_REDIRECTION_SYSLOG_REMOTE;
   } /* if (ifx_optarg != IFX_NULL) */
   else
   {
      pProgramArg->nTraceRedirection = TRACE_REDIRECTION_SYSLOG_LOCAL;
   }

   return IFX_SUCCESS;
}

/**
   Specify network IP address of the board for connections.
 
   \param pProgramArg - pointer to program arguments structure

   \return IFX_SUCCESS argument is ok, otherwise IFX_ERROR
*/
IFX_return_t ReadOptions_IP_Addr(PROGRAM_ARG_t* pProgramArg)
{
   /* Formated command line options */
   IFX_char_t psCmdOption[MAX_ARGUMENT_LEN];

   /* check input parameters */
   TD_PTR_CHECK(pProgramArg, IFX_ERROR);

   
   if (ifx_optarg == IFX_NULL)
   {
      TRACE (TAPIDEMO, DBG_LEVEL_HIGH,
             ("Can't set different network.\n (File: %s, line: %d)\n",
               __FILE__, __LINE__));
      return IFX_ERROR;
   }

   if(strnlen(ifx_optarg, MAX_ARGUMENT_LEN) == MAX_ARGUMENT_LEN)
   {
      TRACE (TAPIDEMO, DBG_LEVEL_HIGH,
             ("Program argument is incorrect. It is either too long or "
              "it is non null terminated string\n (File: %s, line: %d)\n",
              __FILE__, __LINE__));
      return IFX_ERROR;
   }
   /* Format command line option to string */
   snprintf(psCmdOption, MAX_ARGUMENT_LEN, "%s", ifx_optarg);

   if (strlen(psCmdOption) > IP_ADDRESS_LEN)
   {
      TAPIDEMO_PRINTF(("Too long ip address %s right syntax is "
                       "<xxx.xxx.xxx.xxx>.\n", psCmdOption));
      return IFX_ERROR;
   }



   if (IFX_SUCCESS != TD_OS_SocketAton(psCmdOption, 
                                       &pProgramArg->oMy_IP_Addr))
   {
       TAPIDEMO_PRINTF(("Wrong format of IP address %s, it should be"
                        " <xxx.xxx.xxx.xxx>.\n", psCmdOption));
      return IFX_ERROR;
   }

   return IFX_SUCCESS;
}

/**
   Get different network calls option argument.
 
   \param pProgramArg - pointer to program arguments structure

   \return IFX_SUCCESS argument is ok, otherwise IFX_ERROR
*/
IFX_return_t ReadOptions_Network(PROGRAM_ARG_t* pProgramArg)
{
   /* Formated command line options */
   IFX_char_t psCmdOption[MAX_ARGUMENT_LEN];
   IFX_char_t tmp_ip_addr[DIFF_NET_IP_LEN + 2];

   /* check input parameters */
   TD_PTR_CHECK(pProgramArg, IFX_ERROR);

   /* Set ip address of different network */
   if (ifx_optarg == IFX_NULL)
   {
      TRACE (TAPIDEMO, DBG_LEVEL_HIGH,
             ("Can't set ip address .\n (File: %s, line: %d)\n",
               __FILE__, __LINE__));

      return IFX_ERROR;
   }

   pProgramArg->oArgFlags.nDiffNet = 0;
   /* Format command line option to string */
   snprintf(psCmdOption, MAX_ARGUMENT_LEN, "%s", ifx_optarg);
   if (DIFF_NET_IP_LEN <= strlen(psCmdOption))
   {
      TAPIDEMO_PRINTF(("IP address %s of different network is wrong, "
                       "right format is xxx.xxx.xxx\n", psCmdOption));
      return IFX_ERROR;
   }
   strncpy(tmp_ip_addr, psCmdOption, DIFF_NET_IP_LEN);
   /* Add last ip number, must not be 0 or 255 */
   if((DIFF_NET_IP_LEN + 2) > (strlen(tmp_ip_addr) + 2))
   {
      strcat(tmp_ip_addr, ".1");
   }
   else
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Buffer overflow, array index of 'tmp_ip_addr' "
            "may be outside the bounds. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /* We can't use TD_OS_SocketAton, because this function expects 
      IFXOS_sockAddr_t as the second arguments. Instead, 
      we use here struct in_addr. */
   /* Check if correct ip address */
   if ((0 == inet_aton(tmp_ip_addr, &pProgramArg->oDiffNetIP)))
   {
      TAPIDEMO_PRINTF(("IP address %s of different network is wrong, "
                       "right format is xxx.xxx.xxx\n", psCmdOption));
      return IFX_ERROR;
   }
   /* set different network flag */
   pProgramArg->oArgFlags.nDiffNet = 1;

   return IFX_SUCCESS;
}

/**
   Get FXO option argument.
 
   \param pProgramArg - pointer to program arguments structure

   \return IFX_SUCCESS argument is ok, otherwise IFX_ERROR
*/
IFX_return_t ReadOptions_Fxo(PROGRAM_ARG_t* pProgramArg)
{
   IFX_int32_t i;

   /* check input parameters */
   TD_PTR_CHECK(pProgramArg, IFX_ERROR);

   if (IFX_NULL != ifx_optarg)
   {
      /* Check every element of array */
      for (i=0; i < strlen(ifx_optarg); i++) 
      {
         if (0 == strncmp(&ifx_optarg[i], TD_CMD_ARG_FXO_TERIDIAN, 1))
         {
#ifdef TERIDIAN_FXO
            pProgramArg->oArgFlags.nFXO = 1;
            pProgramArg->nFxoMask |= TD_FXO_TER;
#else
            TAPIDEMO_PRINTF(("Err, Tapidemo compiled without Teridian support "
                             "(drv_ter1x66 missing).\n"
                             "Option '-x%s' can be used only when Tapidemo is "
                             "compiled with Teridian support. ",
                             TD_CMD_ARG_FXO_TERIDIAN));
            return IFX_ERROR;
#endif
         } else if (0 == strncmp(&ifx_optarg[i], TD_CMD_ARG_FXO_DUSLIC, 1))
         {
#ifdef DUSLIC_FXO
            pProgramArg->oArgFlags.nFXO = 1;
            pProgramArg->nFxoMask |= TD_FXO_DUS;
#else
            TAPIDEMO_PRINTF(("Err, Tapidemo compiled without Duslic support.\n"
                             "(driver missing).\n"
                             "Option '-x%s' can be used only when Tapidemo is "
                             "compiled with Duslic support. ",
                             TD_CMD_ARG_FXO_DUSLIC));
            return IFX_ERROR;
#endif
         } else if (0 == strncmp(&ifx_optarg[i], TD_CMD_ARG_FXO_SLIC121, 1))
         {
#ifdef SLIC121_FXO
            pProgramArg->oArgFlags.nFXO = 1;
            pProgramArg->nFxoMask |= TD_FXO_SLIC121;
#else
            TAPIDEMO_PRINTF(("Err, Tapidemo does not support the SLIC 121 FXO "
                             "on this board.\n "
                             "Option '-x%s' can't be used. \n",
                             TD_CMD_ARG_FXO_SLIC121));
            return IFX_ERROR;
#endif
         }
         else
         {
            TAPIDEMO_PRINTF(("Err, not supported argument '-x%s'. \n",
                             ifx_optarg));
            return IFX_ERROR;
         }
      } /* for */
   }
   else
   {
      /* '-x' is used without additional parameters. Set all supported
         FXO devices. */
      pProgramArg->oArgFlags.nFXO = 1;
#ifdef DUSLIC_FXO
      pProgramArg->nFxoMask |= TD_FXO_DUS;
#endif
#ifdef TERIDIAN_FXO
      pProgramArg->nFxoMask |= TD_FXO_TER;
#endif
#ifdef SLIC121_FXO
      pProgramArg->nFxoMask |= TD_FXO_SLIC121;
#endif
   }
   return IFX_SUCCESS;
}

/**
   Get QoS option argument.
 
   \param pProgramArg - pointer to program arguments structure

   \return IFX_SUCCESS argument is ok, otherwise IFX_ERROR
*/
IFX_return_t ReadOptions_QOS(PROGRAM_ARG_t* pProgramArg)
{
   const IFX_char_t* const UDP_FLAG_LOCAL = "l";
#ifdef VXWORKS
   const IFX_char_t* const UDP_FLAG_EXTERNAL = "e";
#endif /* VXWORKS */

   /* check input parameters */
   TD_PTR_CHECK(pProgramArg, IFX_ERROR);

   /* QOS */
   if (IFX_NULL != ifx_optarg)
   {
      if (0 == strcmp(UDP_FLAG_LOCAL, ifx_optarg))
      {
         pProgramArg->oArgFlags.nQOS_Local = 1;
         pProgramArg->oArgFlags.nQos = 1;
         pProgramArg->oArgFlags.nUseCodersForLocal = 1;
      } /* if (0 == strcmp(UDP_FLAG_LOCAL, ifx_optarg)) */
#ifndef VXWORKS
      else
      {
         TAPIDEMO_PRINTF(("Wrong parameter for setting UDP (QOS), "
                          "should be %s - local or nothing - external.\n",
                          UDP_FLAG_LOCAL));
         return IFX_ERROR;
      }
#else
      else if (0 == strcmp(UDP_FLAG_EXTERNAL, ifx_optarg))
      {
         pProgramArg->oArgFlags.nQOS_Local = 0;
         pProgramArg->oArgFlags.nQos = 1;
      } /* else if (0 == strcmp(UDP_FLAG_EXTERNAL, ifx_optarg)) */
      else
      {
         TAPIDEMO_PRINTF(("Wrong parameter for setting UDP (QOS), should be %s - "
                "local or %s - external.\n",
                UDP_FLAG_LOCAL, UDP_FLAG_EXTERNAL));
         return IFX_ERROR;
      }
#endif /* VXWORKS */
      return IFX_SUCCESS;
   } /* if (ifx_optarg != IFX_NULL) */

   /* Set default settings */
   pProgramArg->oArgFlags.nQOS_Local = 0;
   pProgramArg->oArgFlags.nQos = 1;

   return IFX_SUCCESS;
}

/**
   Get LEC option argument.
 
   \param pProgramArg - pointer to program arguments structure

   \return IFX_SUCCESS argument is ok, otherwise IFX_ERROR
*/
IFX_return_t ReadOptions_LEC(PROGRAM_ARG_t* pProgramArg)
{
   IFX_int32_t arg_value = 0;

   /* check input parameters */
   TD_PTR_CHECK(pProgramArg, IFX_ERROR);

   /* LEC configuration */
   if (IFX_NULL != ifx_optarg)
   {
      if (Common_IsNumber(ifx_optarg))
      {  
         arg_value = atoi(ifx_optarg);
         if ((IFX_TAPI_WLEC_TYPE_OFF <= arg_value) &&
             (IFX_TAPI_WLEC_TYPE_ES >= arg_value))
         {
            switch (arg_value)
            {
               case IFX_TAPI_WLEC_TYPE_OFF:
               case IFX_TAPI_WLEC_TYPE_NE:
               case IFX_TAPI_WLEC_TYPE_NFE:
               case IFX_TAPI_WLEC_TYPE_NE_ES:
               case IFX_TAPI_WLEC_TYPE_NFE_ES:
               case IFX_TAPI_WLEC_TYPE_ES:
                  pProgramArg->nLecCfg = arg_value;
                  break;
               default:
                  return IFX_ERROR;
            } /* switch (arg_value) */
            return IFX_SUCCESS;
         } /* if */
      } /* if */
   } /* if */
   return IFX_ERROR;
 } /* ReadOptions_LEC */


/**
   Reads program arguments.

   \param nArgCnt     - number of arguments (basically number of words in string)
   \param pArgv       - array of arguments (first one is the name of started program and
                        then parameters are following)
   \param pProgramArg - pointer to program arguments structure

   \return IFX_SUCCESS arguments ok, otherwise IFX_ERROR
*/
IFX_return_t ReadOptions(IFX_int32_t nArgCnt,
                         IFX_char_t* pArgv[],
                         PROGRAM_ARG_t* pProgramArg)
{
   const IFX_char_t* const PCM_FLAG_MASTER = "m";
   const IFX_char_t* const PCM_FLAG_SLAVE = "s";
   const IFX_char_t* const PCM_FLAG_LOOP = "l";

   IFX_int32_t option_index = 0, option = 0;
   IFX_int32_t arg_value = 0;

   /* check input parameters */
   TD_PTR_CHECK(pProgramArg, IFX_ERROR);
   TD_PTR_CHECK(pArgv, IFX_ERROR);

   TRACE(TAPIDEMO, DBG_LEVEL_LOW, ("Read program arguments.\n"));

   if (1 >= nArgCnt)
   {
      /* None arguments passed to program */
      ProgramHelp();
      return IFX_ERROR;
   }

   while ((option = getopt_long(nArgCnt, pArgv,
                                OPTION_PARAMETERS, PROGRAM_OPTIONS,
                                (int *) option_index)) != -1)
   {
#ifdef LINUX
      ifx_optarg = optarg;
#endif
      switch (option)
      {
         case 'h':
            pProgramArg->oArgFlags.nHelp = 1;
            break;
         case 'g':
            pProgramArg->hUseDuslicFxo = IFX_FALSE;
            break;
#ifdef LINUX
         case 'r':
            pProgramArg->oArgFlags.nDaemon= 1;
            pProgramArg->nTraceRedirection = TRACE_REDIRECTION_FILE;
            if (IFX_NULL != ifx_optarg)
            {
               strncpy(oTraceRedirection.sPathToLogFiles, ifx_optarg, MAX_PATH_LEN - 1);
            }
            else
            {
               strncpy(oTraceRedirection.sPathToLogFiles,
                       "/opt/lantiq/bin/tapidemo.log", MAX_PATH_LEN - 1);
            }
            break;
         case 's':
            if (IFX_SUCCESS != ReadOptions_Syslog(pProgramArg))
            {
               return IFX_ERROR;
            }
            break;
#endif /* LINUX */
         case 'd':
            if (IFX_NULL != ifx_optarg)
            {     
               if (Common_IsNumber(ifx_optarg)) 
               {
                  arg_value = atoi(ifx_optarg);
                  if ((arg_value > DBG_LEVEL_OFF) || (arg_value < DBG_LEVEL_LOW))
                  {
                     arg_value = DBG_LEVEL_OFF;
                     TRACE (TAPIDEMO, DBG_LEVEL_HIGH,
                            ("Can't set debug level. Use default level.\n"
                             "(File: %s, line: %d)\n", __FILE__, __LINE__));
                  }
                  pProgramArg->nDbgLevel = arg_value;
                  g_pITM_Ctrl->nComDbgLevel = arg_value;
                  break;
               }
            }
            TRACE (TAPIDEMO, DBG_LEVEL_HIGH,
                   ("Can't set debug level.\n (File: %s, line: %d)\n",
                     __FILE__, __LINE__));
            return IFX_ERROR;
         case 'i':
            if (IFX_SUCCESS != ReadOptions_IP_Addr(pProgramArg))
            {
               return IFX_ERROR;
            }
            break;
         case 'w':
            pProgramArg->oArgFlags.nWait = 1;
            break;
         case 'q':
            if (IFX_SUCCESS != ReadOptions_QOS(pProgramArg))
            {
               return IFX_ERROR;
            }
            break;
         case 'c':
            /* CID */
            if (IFX_NULL != ifx_optarg)
            {
               if (Common_IsNumber(ifx_optarg))
               {          
                  /* get CID standard */
                  pProgramArg->nCIDStandard = atoi(ifx_optarg);
                  /* if arguemt is equal TD_TURN_OFF_CID then CID should not be used */
                  if (TD_TURN_OFF_CID == pProgramArg->nCIDStandard)
                  {
                     /* CID will not be used */
                     pProgramArg->oArgFlags.nCID = 0;
                  }
                  else
                  {
                     /* CID will not be used */
                     pProgramArg->oArgFlags.nCID = 1;
                  }
                  break;
               }
            }
            TRACE (TAPIDEMO, DBG_LEVEL_HIGH,
                   ("Can't set CID.\n (File: %s, line: %d)\n",
                     __FILE__, __LINE__));
            return IFX_ERROR;
         case 'k':
            /* Conferencing */
            if (ifx_optarg != IFX_NULL)
            {
               if (Common_IsNumber(ifx_optarg))
               {
                  arg_value = atoi(ifx_optarg);
                  if (0 == arg_value)
                  {
                     pProgramArg->oArgFlags.nConference = 0;
                     pProgramArg->oArgFlags.nUseCodersForLocal = 1;
                  }
                  else
                  {
                     pProgramArg->oArgFlags.nConference = 1;
                  }
               }
               else
               {
                  TRACE (TAPIDEMO, DBG_LEVEL_HIGH,
                         ("Can't set conference.\n (File: %s, line: %d)\n",
                           __FILE__, __LINE__));
                  return IFX_ERROR;
               }
            } /* if (ifx_optarg != IFX_NULL) */
            else
            {
               pProgramArg->oArgFlags.nConference = 1;
            } /* if (ifx_optarg != IFX_NULL) */
            break;
         case 'p':
            /* Using PCM */
            if (IFX_NULL != ifx_optarg)
            {
               if (0 == strcmp(PCM_FLAG_MASTER, ifx_optarg))
               {
                  pProgramArg->oArgFlags.nPCM_Master = 1;
                  pProgramArg->oArgFlags.nPCM_Slave = 0;
                  pProgramArg->oArgFlags.nPCM_Loop = 0;
               }
               else if (0 == strcmp(PCM_FLAG_SLAVE, ifx_optarg))
               {
                  pProgramArg->oArgFlags.nPCM_Master = 0;
                  pProgramArg->oArgFlags.nPCM_Slave = 1;
                  pProgramArg->oArgFlags.nPCM_Loop = 0;
               }
               else if (0 == strcmp(PCM_FLAG_LOOP, ifx_optarg))
               {
                  pProgramArg->oArgFlags.nPCM_Master = 1;
                  pProgramArg->oArgFlags.nPCM_Slave = 0;
                  pProgramArg->oArgFlags.nPCM_Loop = 1;
               }
               else
               {
                  TAPIDEMO_PRINTF(("Wrong parameter for setting PCM, should be %s - "
                         "master or %s - slave.\n",
                         PCM_FLAG_MASTER, PCM_FLAG_SLAVE));
                  return IFX_ERROR;
               }
               break;
            } /* if (ifx_optarg != IFX_NULL) */
            TRACE (TAPIDEMO, DBG_LEVEL_HIGH,
                   ("Can't set PCM.\n (File: %s, line: %d)\n",
                     __FILE__, __LINE__));
            return IFX_ERROR;
         case 'e':
            /* Set encoder type */
            if (IFX_NULL != ifx_optarg)
            {
               if (Common_IsNumber(ifx_optarg))
               {
                  pProgramArg->oArgFlags.nEncTypeDef = 1;
                  pProgramArg->nEnCoderType = atoi(ifx_optarg);
                  break;
               }
            } /* if (ifx_optarg != IFX_NULL) */
            TRACE (TAPIDEMO, DBG_LEVEL_HIGH,
                   ("Can't set encoder type.\n (File: %s, line: %d)\n",
                    __FILE__, __LINE__));
            return IFX_ERROR;
         case 'f':
            /* Set packetisation time */
            if (IFX_NULL != ifx_optarg)
            {
               if (Common_IsNumber(ifx_optarg))
               {
                  pProgramArg->oArgFlags.nFrameLen = 1;
                  pProgramArg->nPacketisationTime = atoi(ifx_optarg);
                  break;
               }
            }
            TRACE (TAPIDEMO, DBG_LEVEL_HIGH,
                   ("Can't set packetisation time.\n (File: %s, line: %d)\n",
                     __FILE__, __LINE__));
            return IFX_ERROR;
         case TAPIDEMO_OPTION_NUMBER_VAD:
            /* voice activity detection (VAD) configuration */
            if (IFX_NULL != ifx_optarg)
            {
               if (Common_IsNumber(ifx_optarg))
               {  
                  arg_value = atoi(ifx_optarg);
                  if ((IFX_TAPI_ENC_VAD_NOVAD <= arg_value) &&
                      (IFX_TAPI_ENC_VAD_SC_ONLY >= arg_value))
                  {
                     pProgramArg->nVadCfg = arg_value;
                     switch (arg_value)
                     {
                        case IFX_TAPI_ENC_VAD_NOVAD:
                           /* TAPIDEMO_PRINTF(
                              ("Set VAD: %d - IFX_TAPI_ENC_VAD_NOVAD - "
                               "no voice activity detection.\n", arg_value)); */
                           break;
                        case IFX_TAPI_ENC_VAD_ON:
                           /* TAPIDEMO_PRINTF(
                              ("Set VAD: %d - IFX_TAPI_ENC_VAD_ON - "
                               "voice activity detection on.\n", arg_value)); */
                           break;
                        case IFX_TAPI_ENC_VAD_G711:
                           /* TAPIDEMO_PRINTF(
                              ("Set VAD: %d - IFX_TAPI_ENC_VAD_G711 - "
                               "voice activity detection on without "
                               "spectral information.\n", arg_value)); */
                           break;
                        case IFX_TAPI_ENC_VAD_SC_ONLY:
                           /* TAPIDEMO_PRINTF(
                              ("Set VAD: %d - IFX_TAPI_ENC_VAD_SC_ONLY - "
                               "voice activity detection on without "
                               "comfort noise generation.\n", arg_value)); */
                           break;
                        default:
                           TAPIDEMO_PRINTF(
                              ("Invalid vad argument %d, using default.\n",
                               arg_value));
                           pProgramArg->nVadCfg = DEFAULT_VAD_CFG;
                           break;
                     } /* switch (arg_value) */
                  }
                  else
                  {
                     TAPIDEMO_PRINTF(("Invalid vad argument %d, using default.\n",
                                      arg_value));
                  }
                  break;
               }
            }
            TRACE (TAPIDEMO, DBG_LEVEL_HIGH,
                   ("Can't set voice activity detection.\n (File: %s, line: %d)\n",
                     __FILE__, __LINE__));
            return IFX_ERROR;
         case TAPIDEMO_OPTION_NUMBER_LEC:
            if (IFX_SUCCESS != ReadOptions_LEC(pProgramArg))
            {
               TRACE (TAPIDEMO, DBG_LEVEL_HIGH,
                      ("Can't set LEC. Use default LEC setting.\n"));
               pProgramArg->nLecCfg = DEFAULT_LEC_CFG;
            }
            break;
         case TAPIDEMO_OPTION_NUMBER_NO_LEC:
            /* do not enable LEC - obsolete */
            pProgramArg->oArgFlags.nNoLEC = 1;
            break;
         case 'n':
            if (IFX_SUCCESS != ReadOptions_Network(pProgramArg))
            {
               return IFX_ERROR;
            }
            break;
         case 'b':
            /* Set board combination */
            if (IFX_NULL != ifx_optarg)
            {
               if (Common_IsNumber(ifx_optarg))
               {
                  arg_value = atoi(ifx_optarg);
                  if ((0 > arg_value) || (UNSUPPORTED_COMB <= arg_value))
                  {
                     /* invalid board combination, don't change it */
                     TAPIDEMO_PRINTF(("Unknown board combination selected %d, "
                                      "ignoring.\n", (int) arg_value));
                  } 
                  else 
                  {
                     /* set board combination */
                     pProgramArg->nBoardComb = arg_value;
                     pProgramArg->oArgFlags.nUseCombinationBoard = 1;
                  }
                  break;
               }
            }
            TRACE (TAPIDEMO, DBG_LEVEL_HIGH,
                   ("Can't set board combination.\n (File: %s, line: %d)\n",
                     __FILE__, __LINE__));
            return IFX_ERROR;
         case 'v':
            /* enable initialization test */
            g_pITM_Ctrl->oVerifySytemInit.fEnabled = IFX_TRUE;
            break;
         case 'm':
            /* enable ITM */
            g_pITM_Ctrl->fITM_Enabled = IFX_TRUE;
            break;
#ifdef LINUX
         case 'x':
            /* FXO */
            if (IFX_SUCCESS != ReadOptions_Fxo(pProgramArg))
            {
               return IFX_ERROR;
            }
            break;
         case 'l':
            /* Path to download files */
            if (IFX_NULL == ifx_optarg)
            {
               TRACE (TAPIDEMO, DBG_LEVEL_HIGH,
                      ("Can't set path to download files.\n "
                       "(File: %s, line: %d)\n",
                        __FILE__, __LINE__));
               return IFX_ERROR;
            }

            strncpy(pProgramArg->sPathToDwnldFiles, ifx_optarg, MAX_PATH_LEN - 1);
            pProgramArg->oArgFlags.nUseCustomDownloadsPath = IFX_TRUE;
            break;
#endif
#if defined(XT16) || defined(WITH_VXT)
         case TAPIDEMO_OPTION_NUMBER_GROUND_START:
            /* enable Ground Start support */
            pProgramArg->oArgFlags.nGroundStart = 1;
            break;
#endif /* XT16 || WITH_VXT */
         case 't':
#ifndef TD_DECT
            /* DECT argument used without DECT support */
            TAPIDEMO_PRINTF(("Invalid command line argument - "
                             "compiled without DECT support.\n"
                             "(File: %s, line: %d)\n",
                             __FILE__, __LINE__));
#else
            /* enable DECT */
            pProgramArg->oArgFlags.nNoDect = 0;
#endif
            break;
#ifdef TD_RM_DEBUG
         case TAPIDEMO_OPTION_NUMBER_RM_DBG:
            /* enable RM debug (resource counting) */
            pProgramArg->oArgFlags.nRM_Count = 1;
            break;
#endif /* TD_RM_DEBUG */
#ifdef TD_DECT
         case TAPIDEMO_OPTION_NUMBER_DECT_DBG:
            if (IFX_NULL != ifx_optarg)
            {
               if (Common_IsNumber(ifx_optarg))
               {
                  arg_value = atoi(ifx_optarg);
                  if ((arg_value > DBG_LEVEL_OFF) || (arg_value < DBG_LEVEL_LOW))
                  {
                     arg_value = DBG_LEVEL_OFF;
                  }
                  pProgramArg->nDectDbgLevel = arg_value;
                  break;
               }
            }
            TRACE (TAPIDEMO, DBG_LEVEL_HIGH,
                   ("Can't set dect debug level.\n (File: %s, line: %d)\n",
                     __FILE__, __LINE__));
            return IFX_ERROR;
#endif /* TD_DECT */
        case TAPIDEMO_OPTION_PRINT_VERSION:
            pProgramArg->oArgFlags.nVersion = 1;
            break;
#ifdef LINUX
         case TAPIDEMO_OPTION_PRINT_CONFIGURE:
            pProgramArg->oArgFlags.nConfigure = 1;
            break;
#endif /* LINUX */
         default:
            /* Unknown argument */
            /* Go out if unknown arg */
#ifdef LINUX
            TAPIDEMO_PRINTF(("Unknown or incomplete arguments: %c. "
                             "(File: %s, line: %d)\n",
                              optopt, __FILE__, __LINE__));
#else
            TAPIDEMO_PRINTF(("Unknown or incomplete arguments."
                             "(File: %s, line: %d)\n",
                              __FILE__, __LINE__));
#endif /* LINUX */
            return IFX_ERROR;
      } /* switch */
   } /* while */

#ifdef LINUX
   /* Check if all arguments were parsed */
   /* Print unknown/incorrect one */
   if (optind < nArgCnt)
   {
       TAPIDEMO_PRINTF(("Unknown or incorrect arguments: "));
       while (optind < nArgCnt)
          TAPIDEMO_PRINTF(("%s ", pArgv[optind++]));
       TAPIDEMO_PRINTF(("\n"));

       return IFX_ERROR;
   }
#endif /* LINUX */
   /* verify system initialization can only be used when test mode is on */
   if (g_pITM_Ctrl->oVerifySytemInit.fEnabled && !g_pITM_Ctrl->fITM_Enabled)
   {
      TAPIDEMO_PRINTF(("Warning, verify system initialization cannot be used "
                       "when test mode isn't turned on\n"));
      g_pITM_Ctrl->oVerifySytemInit.fEnabled = IFX_FALSE;
   }
   return IFX_SUCCESS;
} /* ReadOptions() */

/**
   Displays program usage with arguments.

   \return status (IFX_SUCCESS or IFX_ERROR)
*/
IFX_return_t ProgramHelp(IFX_void_t)
{
   IFX_int32_t i = 0;
   IFX_int32_t j = 0;

   TAPIDEMO_PRINTF(("Program usage.\n"));

   TAPIDEMO_PRINTF(("Usage: <%s> [options]\r\n", (IFX_char_t *) PROGRAM_NAME));
   TAPIDEMO_PRINTF(("Following options defined:\r\n\n"));
   i = 0;
   j = 0;
   while ((PROGRAM_OPTIONS[i].name) && (0 != strlen(helpItems[j])))
   {
      /* print option number and long name */
      TAPIDEMO_PRINTF(("(%d) '--%s'", (int)i + 1, PROGRAM_OPTIONS[i].name));
      /* print option short name if avaible */
      if ((0 < PROGRAM_OPTIONS[i].val) && ((255 > PROGRAM_OPTIONS[i].val)))
      {
         TAPIDEMO_PRINTF((" OR '-%c'", PROGRAM_OPTIONS[i].val));
      }
      /* print option help first line */
      TAPIDEMO_PRINTF((" : %s\n", helpItems[j]));
      j++;
      while (0 != strlen(helpItems[j]))
      {
         TAPIDEMO_PRINTF(("\t%s\n", helpItems[j]));
         j++;
      }
      /* Jump over delimiter for help of next argument. */
      j++;

      i++;
   }

   return IFX_SUCCESS;
} /* ProgramHelp() */

/**
   Print used program arguments.

   \param pProgramArg - pointer to program arguments structure

   \return IFX_SUCCESS arguments ok, otherwise IFX_ERROR
*/
IFX_return_t PrintUsedOptions(PROGRAM_ARG_t* pProgramArg)
{
   /* check input parameters */
   TD_PTR_CHECK(pProgramArg, IFX_ERROR);

#ifdef TD_DECT
   if (!pProgramArg->oArgFlags.nNoDect) 
   {
      TAPIDEMO_PRINTF(("- DECT support.\n"));
   }
#endif /* TD_DECT */

   if (pProgramArg->oArgFlags.nFXO)
   {
#ifdef FXO
      TAPIDEMO_PRINTF(("- FXO support on.\n"));
#else
      TAPIDEMO_PRINTF(("- FXO support unavailable.\n"
                      "  Please rebuild with all required header files.\n"));
#endif /* FXO */
   } /* FXO */

   /* karol -workaround for teridian extension board */
   if (!pProgramArg->hUseDuslicFxo)
   {
      TAPIDEMO_PRINTF(("- DuSLIC FXO support disabled.\n"));
   }

   if (!pProgramArg->oArgFlags.nConference)
   {
      TAPIDEMO_PRINTF(("- Conference support disabled.\n"));
   } /* Conference */
   /* CID */
   if (pProgramArg->oArgFlags.nCID)
   {
      TAPIDEMO_PRINTF(("- CID support: %s (standard %d).\n",
                       Common_Enum2Name(CID_GetStandard(),
                                        TD_rgCidStandardName),
                       CID_GetStandard()));
   } /* CID */
   else
   {
      TAPIDEMO_PRINTF(("- CID support: disabled.\n"));
   }

   switch (pProgramArg->nVadCfg)
   {
      case IFX_TAPI_ENC_VAD_NOVAD:
         TAPIDEMO_PRINTF(("- VAD support: IFX_TAPI_ENC_VAD_NOVAD - "
                          "no voice activity detection.\n"));
         break;
      case IFX_TAPI_ENC_VAD_ON:
         TAPIDEMO_PRINTF(("- VAD support: IFX_TAPI_ENC_VAD_ON - "
                          "voice activity detection on.\n"));
         break;
      case IFX_TAPI_ENC_VAD_G711:
         TAPIDEMO_PRINTF(("- VAD support: IFX_TAPI_ENC_VAD_G711 - "
                          "voice activity detection on without "
                          "spectral information.\n"));
         break;
      case IFX_TAPI_ENC_VAD_SC_ONLY:
         TAPIDEMO_PRINTF(("- VAD support: IFX_TAPI_ENC_VAD_SC_ONLY - "
                          "voice activity detection on without "
                          "comfort noise generation.\n"));
         break;
      default:
         break;
   } /* VAD */

   switch (pProgramArg->nLecCfg)
   {
      case IFX_TAPI_WLEC_TYPE_OFF:
         TAPIDEMO_PRINTF(
            ("- LEC support: IFX_TAPI_WLEC_TYPE_OFF - \n"
             "               LEC and Echo Suppressor turned off.\n")); 
         break;
      case IFX_TAPI_WLEC_TYPE_NE:
          TAPIDEMO_PRINTF(
            ("- LEC support: IFX_TAPI_WLEC_TYPE_NE - \n"
             "               LEC using fixed window. No Echo Suppressor.\n")); 
         break;
      case IFX_TAPI_WLEC_TYPE_NFE:
          TAPIDEMO_PRINTF(
            ("- LEC support: IFX_TAPI_WLEC_TYPE_NFE - \n"
             "               LEC using fixed and moving window. No Echo Suppressor.\n")); 
         break;
      case IFX_TAPI_WLEC_TYPE_NE_ES:
          TAPIDEMO_PRINTF(
            ("- LEC support: IFX_TAPI_WLEC_TYPE_NE_ES - \n"
             "               LEC using fixed window + Echo Suppressor.\n")); 
         break;
      case IFX_TAPI_WLEC_TYPE_NFE_ES:
          TAPIDEMO_PRINTF(
            ("- LEC support: IFX_TAPI_WLEC_TYPE_NFE_ES - \n"
             "               LEC using fixed and moving window + Echo Suppressor.\n")); 
         break;
      case IFX_TAPI_WLEC_TYPE_ES:
          TAPIDEMO_PRINTF(
            ("- LEC support: IFX_TAPI_WLEC_TYPE_ES - Echo Suppressor.\n")); 
         break;
   }


   if (pProgramArg->oArgFlags.nNoLEC)
   {
      TAPIDEMO_PRINTF(("- LEC support disabled.\n"));
   } /* LEC */

   if (pProgramArg->oArgFlags.nPCM_Slave)
   {
      TAPIDEMO_PRINTF(("- PCM slave mode.\n"));
   }

   if (pProgramArg->oArgFlags.nPCM_Master)
   {
      if (pProgramArg->oArgFlags.nPCM_Loop)
      {
         TAPIDEMO_PRINTF(("- PCM master mode with local loop.\n"));
      }
      else
      {
         TAPIDEMO_PRINTF(("- PCM master mode.\n"));
      }
   } /* PCM */

   /* if using coders for local calls */
   if (pProgramArg->oArgFlags.nUseCodersForLocal)
   {
      /* if QoS flags are set */
      if (pProgramArg->oArgFlags.nQOS_Local && pProgramArg->oArgFlags.nQos)
      {
         TAPIDEMO_PRINTF(("- Local UDP redirection.\n"));
         TAPIDEMO_PRINTF(("- Coders for local connections.\n"));
      }
      /* if conference is disabled */
      else if (!pProgramArg->oArgFlags.nConference)
      {
         TAPIDEMO_PRINTF(("- Coders for local connections.\n"));
      }
   }

   if (!(pProgramArg->oArgFlags.nQOS_Local)
       && pProgramArg->oArgFlags.nQos)
   {
      TAPIDEMO_PRINTF(("- External UDP redirection.\n"));
   } /* External UDP */

   if (pProgramArg->oArgFlags.nDiffNet)
   {
      IFX_char_t sNetworkAddr[MAX_ARGUMENT_LEN] = {0};
      IFX_int32_t i, j;

      /* copy string representation of address */
      strncpy(sNetworkAddr, inet_ntoa(pProgramArg->oDiffNetIP),
              MAX_ARGUMENT_LEN -1);
      /* remove last part of IP address */
      for (i=0, j=0; i<MAX_ARGUMENT_LEN; i++)
      {
         if ('.' == sNetworkAddr[i])
         {
            /* increase counter */
            j++;
            /* only 3 first bytes are network address */
            if (3 == j)
            {
               /* set end of string */
               sNetworkAddr[i] = '\0';
               break;
            } /* if (3 == j) */
         } /* if ('.' == sNetworkAddr[i]) */
      } /* for (i=0, j=0; i<MAX_ARGUMENT_LEN; i++) */
      /* check for null termination of string */
      if (sNetworkAddr[i] == '\0')
      {
         TAPIDEMO_PRINTF(("- Calling network address: %s.\n",
                          sNetworkAddr));
      }
      else
      {
         TAPIDEMO_PRINTF(("Err - Invalid calling network address: %s.\n",
                          inet_ntoa(pProgramArg->oDiffNetIP)));

      }
   } /* Calling network address */

   if (pProgramArg->oArgFlags.nWait)
   {
      TAPIDEMO_PRINTF(("- Waiting for user input in state machine.\n"));
   } /* Wait */

   if (pProgramArg->oArgFlags.nDaemon)
   {
      TAPIDEMO_PRINTF(("- Daemon mode with "));
   } /* Daemon */
   if (pProgramArg->nTraceRedirection == TRACE_REDIRECTION_SYSLOG_LOCAL)
   {
      TAPIDEMO_PRINTF(("redirecting logs to local syslog.\n"));
   } /* Log redirection */
   else if (pProgramArg->nTraceRedirection == TRACE_REDIRECTION_SYSLOG_REMOTE)
   {
      TAPIDEMO_PRINTF(("redirecting logs to remote syslog.\n"));
   } /* Log redirection */
   else if (pProgramArg->nTraceRedirection == TRACE_REDIRECTION_FILE)
   {
      TAPIDEMO_PRINTF(("redirecting logs to file.\n"));
   } /* Log redirection */

   TAPIDEMO_PRINTF(("- Debug level: %s.\n",
                    Common_Enum2Name(pProgramArg->nDbgLevel,
                                     TD_rgDebugLevelName)));

#ifdef TD_FAX_T38
   TAPIDEMO_PRINTF(("- Transparent fax/modem and T.38 transmission support.\n"));
#else
   #ifdef TD_FAX_MODEM
      TAPIDEMO_PRINTF(("- Transparent fax/modem transmission support.\n"));
   #endif /* TD_FAX_MODEM */
#endif /* TD_FAX_T38 */

#if defined(XT16) || defined(WITH_VXT)
   if (pProgramArg->oArgFlags.nGroundStart)
   {
      TAPIDEMO_PRINTF(("- Ground Start support enabled.\n"));
   }
   else
   {
      TAPIDEMO_PRINTF(("- Ground Start support disabled.\n"));
   }
#endif /* XT16 || WITH_VXT */
#ifdef TD_RM_DEBUG
   /* enable RM debug (resource counting) */
   if (pProgramArg->oArgFlags.nRM_Count)
   {
      TAPIDEMO_PRINTF(("- RM debug (resource counting) on.\n"));
   }
#endif /* TD_RM_DEBUG */
   /* enable ITM Internal Test Mode */
   if (g_pITM_Ctrl->fITM_Enabled)
   {
      TAPIDEMO_PRINTF(("- Internal Test Mode (ITM) enabled.\n"));
   }

   return IFX_SUCCESS;
} /* PrintUsedOptions */
