

/******************************************************************************

                              Copyright (c) 2009
                            Lantiq Deutschland GmbH
                     Am Campeon 3; 85579 Neubiberg, Germany

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

  ****************************************************************************/

/**
   \file pcm.c
   \date 2006-03-29
   \brief Implementation of PCM module for application.

   PCM is initialized, handling with channel resources, defining timeslot for
   connection, mapping pcm channels, ...
*/

/* ============================= */
/* Includes                      */
/* ============================= */

#include "tapidemo.h"
#include "pcm.h"
#include "cid.h"
#ifdef EASY336
#include "lib_svip_rm.h"
#endif /* EASY336 */

/* ============================= */
/* Global Structures             */
/* ============================= */

/** Array of timeslots with their status */
typedef struct _TIMESLOTS_t
{
   /** IFX_TRUE - On PCM master board this one is used for TX, else is RX
       On slave mode is opposite. For now variable is only set never checked. */
   IFX_boolean_t fUsedForTX;
   /** timeslot is reserved */
   IFX_boolean_t fReserved;
   /** how many more timeslots are reserved for one direction. */
   IFX_uint32_t nNumOfTimslots;
} TIMESLOTS_t;

/* ============================= */
/* Global variable definition    */
/* ============================= */

/** PCM data */
TAPIDEMO_PCM_DATA_t oPCM_Data;

/* ============================= */
/* Global function declaration   */
/* ============================= */

/* ============================= */
/* Local variable definition     */
/* ============================= */

/* In narrowband mode A-Law and u-Law uses 1 timeslot */
static IFX_int32_t NB_TIMESLOT_MULTIPLIER = 1;

/* In narrowband mode Linear 16 needs 2 timeslots */
static IFX_int32_t NB_TIMESLOT_MULTIPLIER_LIN = 2;

/* In wideband mode A-Law and u-Law uses 2 timeslots */
static IFX_int32_t WB_TIMESLOT_MULTIPLIER = 2;

/* In wideband mode Linear 16 needs 4 timeslots */
static IFX_int32_t WB_TIMESLOT_MULTIPLIER_LIN = 4;

/** Table of all PCM timeslots */
static TIMESLOTS_t* pTimeslots = IFX_NULL;

/** Number of PCM highways on board */
#ifndef EASY336
static IFX_int32_t nPCM_Highway = 0;
#else
static IFX_int32_t nPCM_Highway = RM_SVIP_NUM_HIGHWAYS;
#endif

enum { RESOULTION_NAMES_LEN = 3};

/** PCM protocol names */
static IFX_char_t* RESOLUTION_NAMES[RESOULTION_NAMES_LEN] =
{
   "A-Law 8 bit",
   "u-Law 8 bit",
   "Linear 16 bit"
};

const IFX_int32_t TAPIDEMO_PCM_FREQ_MAX_TIMESLOT_NUMBER[] =
{
   /* IFX_TAPI_PCM_IF_DCLFREQ_512 */
   8,
   /* IFX_TAPI_PCM_IF_DCLFREQ_1024 */
   16,
   /* IFX_TAPI_PCM_IF_DCLFREQ_1536 */
   24,
   /* IFX_TAPI_PCM_IF_DCLFREQ_2048 */
   32,
   /* IFX_TAPI_PCM_IF_DCLFREQ_4096 */
   64,
   /* IFX_TAPI_PCM_IF_DCLFREQ_8192 */
   128,
   /* IFX_TAPI_PCM_IF_DCLFREQ_16384 */
   256
};

/* ALaw, uLaw, Linear and 8 or 16 bit. */
#ifndef DXT
/** default narrow band resolution */
static IFX_TAPI_PCM_RES_t ePCM_NB_Resolution = IFX_TAPI_PCM_RES_NB_ALAW_8BIT;
/** default wide band resolution */
static IFX_TAPI_PCM_RES_t ePCM_WB_Resolution = IFX_TAPI_PCM_RES_WB_ALAW_8BIT;
#else /* DXT */
/** default narrow band resolution */
static IFX_TAPI_PCM_RES_t ePCM_NB_Resolution = IFX_TAPI_PCM_RES_NB_LINEAR_16BIT;
/** default wide band resolution */
static IFX_TAPI_PCM_RES_t ePCM_WB_Resolution = IFX_TAPI_PCM_RES_WB_LINEAR_16BIT;
#endif /* DXT */

/** max length of string with "Phone No X" or "FXO No X" */
#define MAX_STRING_LEN  30
/* ============================= */
/* Local function declaration    */
/* ============================= */

/* ============================= */
/* Local function definition     */
/* ============================= */


/**
   Prepares table with timeslots. This table is global, not done for each board.

   \return IFX_SUCCESS if table was created, otherwise IFX_ERROR
*/
IFX_return_t PCM_PrepareTimeslotTable()
{
   IFX_return_t ret = IFX_SUCCESS;

   if (pTimeslots != IFX_NULL)
   {
      /* Table with timeslots is already prepared. */
      TRACE(TAPIDEMO, DBG_LEVEL_NORMAL,
           ("Table with timeslots already initialized. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_SUCCESS;
   }
   /* allocate memory */
   pTimeslots = calloc(oPCM_Data.nMaxTimeslotNum, sizeof(TIMESLOTS_t));

   if (pTimeslots == IFX_NULL)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("ERR allocate memory for timeslot pairs. (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   return ret;
}


/**
   Release table with timeslots from memory.
*/
IFX_void_t PCM_ReleaseTimeslotTable()
{
   if (pTimeslots == IFX_NULL)
   {
      /* Table already released from memory */
      return;
   }

   if (pTimeslots != IFX_NULL)
   {
      TD_OS_MemFree(pTimeslots);
      pTimeslots = IFX_NULL;
   }

   TRACE(TAPIDEMO, DBG_LEVEL_NORMAL, ("Timeslot table released from memory\n"));
}

/**
   Returns number of timeslot needed for PCM call in one direction.

   \param fWideBand  - if IFX_TRUE get number of timeslots for WideBand

   \return number of timeslots needed
*/
IFX_int32_t PCM_GetNumberOfTimeslots(IFX_boolean_t fWideBand)
{
   /* get number of timeslots according to WB/NB and PCM resolution */
   if (IFX_TAPI_PCM_RES_NB_LINEAR_16BIT == ePCM_NB_Resolution)
   {
      /* for linear codec */
      if (IFX_TRUE == fWideBand)
      {
         return WB_TIMESLOT_MULTIPLIER_LIN;
      }
      else
      {
         return NB_TIMESLOT_MULTIPLIER_LIN;
      }
   }
   else
   {
      if (IFX_TRUE == fWideBand)
      {
         return WB_TIMESLOT_MULTIPLIER;
      }
      else
      {
         return NB_TIMESLOT_MULTIPLIER;
      }
   } /* if (IFX_TAPI_PCM_RES_NB_LINEAR_16BIT == ePCM_NB_Resolution) */
} /* PCM_GetNumberOfTimeslots() */

/* ============================= */
/* Global function definition    */
/* ============================= */

/**
   PCM Initialization.

   \param pProgramArg  - pointer to program arguments

   \return IFX_SUCCESS on OK, otherwise IFX_ERROR
*/
IFX_return_t PCM_Init(PROGRAM_ARG_t* pProgramArg)
{
   if (pProgramArg == IFX_NULL)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   /* get number of timeslots */
   oPCM_Data.nMaxTimeslotNum =
      TAPIDEMO_PCM_FREQ_MAX_TIMESLOT_NUMBER[oPCM_Data.nPCM_Rate];
   if (0 >= oPCM_Data.nMaxTimeslotNum)
   {
      /* Could not prepare timeslot table */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("PCM: Invalid number of timeslots(%d). (File: %s, line: %d)\n",
             oPCM_Data.nMaxTimeslotNum, __FILE__, __LINE__));

      return IFX_ERROR;
   }

   if (PCM_PrepareTimeslotTable() != IFX_SUCCESS)
   {
      /* Could not prepare timeslot table */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("PCM_PrepareTimeslotTable() failed. (File: %s, line: %d)\n",
            __FILE__, __LINE__));

      return IFX_ERROR;
   }

   return IFX_SUCCESS;
} /* PCM_Init() */


/**
   PCM release.

   \param pProgramArg  - pointer to program arguments
   \param pBoard       - board handle

   \return IFX_SUCCESS on OK, otherwise IFX_ERROR
*/
IFX_return_t PCM_Release(PROGRAM_ARG_t* pProgramArg, BOARD_t* pBoard)
{
   PCM_ReleaseTimeslotTable();

   return IFX_SUCCESS;
} /* PCM_Release() */

/**
   Returns timeslot number and reserves it.

   \param nTS_direction - direction for witch timeslots are needed
   \param fWideBand     - if IFX_TRUE WideBand is used

   \return timeslot or TIMESLOT_NOT_SET if none was found
*/
IFX_int32_t PCM_GetTimeslots(TS_DIRECTION_t nTS_direction,
                             IFX_boolean_t fWideBand)
{
   IFX_int32_t nTimeslot, j;
   IFX_int32_t nNumOfTimslots;
   IFX_boolean_t fTimeslotFound = IFX_FALSE;

   if (TS_RX != nTS_direction && TS_TX != nTS_direction)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_FALSE;
   }

   /* get number of timeslots */
   nNumOfTimslots = PCM_GetNumberOfTimeslots(fWideBand);

   /* for all timeslots */
   for(nTimeslot = 0; nTimeslot < oPCM_Data.nMaxTimeslotNum; nTimeslot++)
   {
      /* check if timeslot avaible */
      if (pTimeslots[nTimeslot].fReserved == IFX_FALSE)
      {
         /* first free timeslot found */
         fTimeslotFound = IFX_TRUE;
         /* check if nNumberOfTimslots is free */
         for (j = 1; j<nNumOfTimslots; j++)
         {
            if ( (oPCM_Data.nMaxTimeslotNum <= (nTimeslot + j)) ||
                 (IFX_TRUE == pTimeslots[nTimeslot + j].fReserved) )
            {
               /* timeslot is not valid */
               fTimeslotFound = IFX_FALSE;
               break;
            }
         }
         if (IFX_TRUE == fTimeslotFound)
         {
            /* reserve all needed timeslots */
            for (j = 0; j<nNumOfTimslots; j++)
            {
               pTimeslots[nTimeslot + j].fReserved = IFX_TRUE;
               /* set direction */
               if (TS_RX == nTS_direction)
               {
                  pTimeslots[nTimeslot + j].fUsedForTX = IFX_FALSE;
               }
               else
               {
                  pTimeslots[nTimeslot + j].fUsedForTX = IFX_TRUE;
               }
               /* set number of reserved timeslots after this one */
               pTimeslots[nTimeslot + j].nNumOfTimslots = nNumOfTimslots - j - 1;
            }
            TRACE(TAPIDEMO, DBG_LEVEL_LOW,
                  ("PCM: Timeslot %d reserved (number of timeslots %d).\n",
                   nTimeslot, nNumOfTimslots));
            return nTimeslot;
         } /* if timeslot is valid */
      } /* if timeslot noot reserved */
   } /* for all timeslots */

   TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
         ("PCM: Unable to get timeslots (number of timeslots %d). "
          "(File: %s, line: %d)\n",
          nNumOfTimslots,__FILE__, __LINE__));

   return TIMESLOT_NOT_SET;
} /* PCM_GetTimeslots() */

/**
   Free timeslots from timeslots table.

   \return nTimeslot - number of timeslot to free

   \return timeslot or -1 if none was found
*/
IFX_return_t PCM_FreeTimeslots(IFX_int32_t nTimeslot)
{
   IFX_int32_t i = 0;
   IFX_int32_t nNumOfTimslots;

   if (0 > nTimeslot || oPCM_Data.nMaxTimeslotNum <= nTimeslot)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Invalid timeslot number %d(%d). (File: %s, line: %d)\n",
             nTimeslot, nTimeslot + i,__FILE__, __LINE__));
      return IFX_ERROR;
   }

   /* get number of timeslots to free */
   nNumOfTimslots = pTimeslots[nTimeslot].nNumOfTimslots;

   do
   {
      /* check if timeslot is reserved */
      if (IFX_FALSE == pTimeslots[nTimeslot + i].fReserved)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("PCM: Timeslot %d(%d) is not reserved. (File: %s, line: %d)\n",
                nTimeslot, nTimeslot + i, __FILE__, __LINE__));
         return IFX_ERROR;
      }
      /* reset timslot */
      pTimeslots[nTimeslot + i].fReserved = IFX_FALSE;
      pTimeslots[nTimeslot + i].fUsedForTX = IFX_FALSE;

      /* check if last reserved timeslot is reached */
      if (0 != pTimeslots[nTimeslot + i].nNumOfTimslots)
      {
         /* reset number of timeslots */
         pTimeslots[nTimeslot + i].nNumOfTimslots = 0;
         /* go to next timeslot */
         i++;
         /* check if valid timeslot number */
         if ((0 > (nTimeslot + i)) ||
             (oPCM_Data.nMaxTimeslotNum < (nTimeslot + i)))
         {
            TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                  ("PCM: Invalid timeslot number %d(%d). "
                   "(File: %s, line: %d)\n",
                   nTimeslot, nTimeslot + i +1, __FILE__, __LINE__));
            return IFX_ERROR;
         }
         /* check if valid number of timeslots */
         if (pTimeslots[nTimeslot + i].nNumOfTimslots != (nNumOfTimslots - i))
         {
            TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
                  ("PCM: Invalid number of timeslots %d(%d). "
                   "(File: %s, line: %d)\n",
                   nNumOfTimslots, nTimeslot + i, __FILE__, __LINE__));
            return IFX_ERROR;
         }
      }
      else
      {
         /* reset number of timeslots */
         pTimeslots[nTimeslot + i].nNumOfTimslots = 0;
         TRACE(TAPIDEMO, DBG_LEVEL_LOW,
               ("PCM: Timeslot %d released (number of timeslots %d).\n",
                nTimeslot, nNumOfTimslots + 1));
         /* timeslots are free to use */
         return IFX_SUCCESS;
      } /* if (0 != pTimeslots[nTimeslot].nNumOfTimslots) */
   }
   while(i <= nNumOfTimslots);

   return IFX_ERROR;
} /* PCM_FreeTimeslots() */

/**
   Activate PCM

   \param pPhone - phone for which PCM activation is done
   \param nPCM_Ch_FD - PCM channel file descriptor
   \param fActivate - IFX_TRUE - activate pcm, IFX_FALSE - deactivate pcm
   \param nTimeslot_RX  - number of receiving timeslot
   \param nTimeslot_TX  - number of transmiting timeslot
   \param fWideBand  - IFX_TRUE use WideBand, otherwise use NarrowBand

   \return IFX_SUCCESS all ok, otherwise IFX_ERROR
   \remarks pPhone should be only used with TAPI_VERSION4, when making FXO calls
      pPhone is IFX_NULL so using it could cause program crash
*/
IFX_return_t PCM_SetActivation(TAPIDEMO_PORT_DATA_t* pPortData,
                               IFX_int32_t nPCM_Ch_FD,
                               IFX_boolean_t fActivate,
                               IFX_int32_t nTimeslot_RX, 
                               IFX_int32_t nTimeslot_TX,
                               IFX_boolean_t fWideBand)
{
   IFX_TAPI_PCM_CFG_t pcmConf;
   IFX_int32_t nNumOfTimeslots;
   IFX_return_t ret = IFX_SUCCESS ;
#ifdef TAPI_VERSION4
   IFX_TAPI_PCM_ACTIVATION_t PCM_Activation;
#endif/* TAPI_VERSION4 */
   IFX_char_t aName[MAX_STRING_LEN];
   IFX_char_t* pTraceIndention;

   if ((0 > nPCM_Ch_FD) || (IFX_NULL == pPortData) ||
       (IFX_TRUE == fActivate &&
        ((0 > nTimeslot_RX) || (oPCM_Data.nMaxTimeslotNum <= nTimeslot_RX) ||
         (0 > nTimeslot_TX) || (oPCM_Data.nMaxTimeslotNum <= nTimeslot_TX))))
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   if (PORT_FXO == pPortData->nType)
   {
      /* no FXO support for FXO in tapidemo for TAPI_VERSION4 */
#ifndef TAPI_VERSION4
      if (IFX_NULL == pPortData->uPort.pFXO)
#endif /* TAPI_VERSION4 */
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("Invalid input argument. (File: %s, line: %d)\n", __FILE__, __LINE__));
         return IFX_ERROR;
      }
      snprintf(aName, (int) MAX_STRING_LEN, "FXO No %d:", 
               pPortData->uPort.pFXO->nFXO_Number);
   }
   else if (PORT_FXS == pPortData->nType)
   {
      if (IFX_NULL == pPortData->uPort.pPhopne)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("Invalid input argument. (File: %s, line: %d)\n", __FILE__, __LINE__));
         return IFX_ERROR;
      }
      snprintf(aName, (int) MAX_STRING_LEN, "Phone No %d:", 
               pPortData->uPort.pPhopne->nPhoneNumber);
   }
   else
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Invalid input argument. (File: %s, line: %d)\n", __FILE__, __LINE__));
      return IFX_ERROR;
   }

   memset(&pcmConf, 0, sizeof (IFX_TAPI_PCM_CFG_t));

   pcmConf.nHighway = nPCM_Highway;
   pcmConf.nResolution = ePCM_NB_Resolution;

   nNumOfTimeslots = PCM_GetNumberOfTimeslots(fWideBand);

#ifdef STREAM_1_1
   pcmConf.nRate = oPCM_Data.nPCM_Rate;
#endif /* STREAM_1_1 */
   /* set timeslots numbers */
   pcmConf.nTimeslotRX = nTimeslot_RX;
   pcmConf.nTimeslotTX = nTimeslot_TX;

   TRACE(TAPIDEMO, DBG_LEVEL_LOW,
         ("%s %s and %s, uses %d timeslots (RX %d, TX %d)\n",
          aName, fWideBand ? "WB" : "NB",
          pcmConf.nResolution == IFX_TAPI_PCM_RES_NB_LINEAR_16BIT ?
          "16-bit linear" : "a-law/u-law",  2 * nNumOfTimeslots,
          (IFX_int32_t) pcmConf.nTimeslotRX,
          (IFX_int32_t) pcmConf.nTimeslotTX));

   /* Correct PCM resolution */
   if (fWideBand)
   {
      pcmConf.nResolution = ePCM_WB_Resolution;
   }

   if (((pcmConf.nTimeslotRX + nNumOfTimeslots) > oPCM_Data.nMaxTimeslotNum) ||
       ((pcmConf.nTimeslotTX + nNumOfTimeslots) > oPCM_Data.nMaxTimeslotNum))
   {
      /* Wrong timeslot idx */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("%s Invalid timeslots: RX %d, TX %d, max. %d. "
             "(File: %s, line: %d)\n",
             aName, (int) pcmConf.nTimeslotRX, (int) pcmConf.nTimeslotTX,
             oPCM_Data.nMaxTimeslotNum, __FILE__, __LINE__));
      return IFX_ERROR;
   }

   if (IFX_TRUE == fActivate)
   {
#ifdef TAPI_VERSION4
      if (pPortData->uPort.pPhopne->pBoard->fSingleFD)
      {
         pcmConf.dev = pPortData->uPort.pPhopne->nDev;
         pcmConf.ch = pPortData->uPort.pPhopne->nPCM_Ch;
         TD_ASSERT ((ret = TD_IOCTL(nPCM_Ch_FD,
            IFX_TAPI_PCM_CFG_SET, (IFX_int32_t) &pcmConf)) == IFX_SUCCESS,
            nPCM_Ch_FD, pPortData->uPort.pPhopne->pBoard, pcmConf.dev, pcmConf.ch);
      } /* if (pPhone->pBoard->fSingleFD) */
      else
#endif /* TAPI_VERSION4 */
      {
            ret = TD_IOCTL(nPCM_Ch_FD, IFX_TAPI_PCM_CFG_SET,
                           (IFX_int32_t) &pcmConf);
      } /* if (pPhone->pBoard->fSingleFD) */
      if (IFX_ERROR == ret)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("%s ERR on ioctl cmd IFX_TAPI_PCM_CFG_SET. "
                "(File: %s, line: %d)\n",
                aName, __FILE__, __LINE__));
         return IFX_ERROR;
      }
   }
   /* set ret to IFX_ERROR so if ioctl is somehow skipped error is printed */
   ret = IFX_ERROR;
   /* PCM channel has been configured, however the communication
      is not active yet */
   if (IFX_TRUE == fActivate)
   {
#ifdef TAPI_VERSION4
      if (pPortData->uPort.pPhopne->pBoard->fSingleFD)
      {
         PCM_Activation.dev = pPortData->uPort.pPhopne->nDev;
         PCM_Activation.ch = pPortData->uPort.pPhopne->nPCM_Ch;
         PCM_Activation.mode = IFX_ENABLE;
         TD_ASSERT ((ret = TD_IOCTL(nPCM_Ch_FD,
            IFX_TAPI_PCM_ACTIVATION_SET, (IFX_int32_t) &PCM_Activation)) ==
            IFX_SUCCESS, nPCM_Ch_FD, pPortData->uPort.pPhopne->pBoard,
            PCM_Activation.dev, PCM_Activation.ch);
      } /* if (pPhone->pBoard->fSingleFD) */
      else
#endif /* TAPI_VERSION4 */
      {
         ret = TD_IOCTL(nPCM_Ch_FD, IFX_TAPI_PCM_ACTIVATION_SET, IFX_ENABLE);
      } /* if (pPhone->pBoard->fSingleFD) */
      if (IFX_ERROR == ret)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("%s ERR on ioctl cmd IFX_TAPI_PCM_ACTIVATION_SET. "
                "(File: %s, line: %d)\n", aName, __FILE__, __LINE__));
         return IFX_ERROR;
      }
   }
   else
   {
#ifdef TAPI_VERSION4
      if (pPortData->uPort.pPhopne->pBoard->fSingleFD)
      {
         PCM_Activation.dev = pPortData->uPort.pPhopne->nDev;
         PCM_Activation.ch = pPortData->uPort.pPhopne->nPCM_Ch;
         PCM_Activation.mode = IFX_DISABLE;
         TD_ASSERT ((ret = TD_IOCTL(nPCM_Ch_FD,
            IFX_TAPI_PCM_ACTIVATION_SET, (IFX_int32_t) &PCM_Activation)) ==
            IFX_SUCCESS, nPCM_Ch_FD, pPortData->uPort.pPhopne->pBoard,
            PCM_Activation.dev, PCM_Activation.ch);
      } /* if (pPhone->pBoard->fSingleFD) */
      else
#endif /* TAPI_VERSION4 */
      {
         ret = TD_IOCTL(nPCM_Ch_FD, IFX_TAPI_PCM_ACTIVATION_SET, IFX_DISABLE);
      }  /* if (pPhone->pBoard->fSingleFD) */

      if (IFX_ERROR == ret)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("%s ERR on ioctl cmd IFX_TAPI_PCM_ACTIVATION_SET. "
                "(File: %s, line: %d)\n", aName, __FILE__, __LINE__));
         return IFX_ERROR;
      }
   }

   /* Every time, we need to count indention size because it can be different, 
      It depends on type of connection */
   pTraceIndention = Common_GetTraceIndention(strlen(aName)+1);
   if (pTraceIndention == IFX_NULL) 
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Cannot prepare trace indention. (File: %s, line: %d)\n",
              __FILE__, __LINE__));
      return IFX_ERROR;
   }

   if(RESOULTION_NAMES_LEN > ePCM_NB_Resolution)
   {
      TRACE(TAPIDEMO, DBG_LEVEL_LOW,
            ("%s %s  PCM on highway %d, resolution %s,\n"
             "%s%s kHz with timeslot pair (RX %d, TX %d)\n",
             aName, (IFX_TRUE == fActivate) ? "Activate" : "Deactivate",
             (int) nPCM_Highway, RESOLUTION_NAMES[(int) ePCM_NB_Resolution],
             pTraceIndention, TAPIDEMO_PCM_FREQ_STR[oPCM_Data.nPCM_Rate], 
             (int) pcmConf.nTimeslotRX, (int) pcmConf.nTimeslotTX));
   }
   else
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Buffer overflow, array index of 'RESOLUTION_NAMES' "
             "may be outside the bounds. (File: %s, line: %d)\n",
             __FILE__, __LINE__));
   }

   /* Now PCM communication is started - test it */
   return IFX_SUCCESS;
} /* PCM_Activate() */

/**
   Map pcm channel to phone channel.

   \param nDstCh     - target PCM channel
   \param nAddCh     - which phone channel to add
   \param fDoMapping - IFX_TRUE do mapping, IFX_FALSE do unmapping
   \param pBoard     - board handle

   \return IFX_SUCCESS if ok, otherwise IFX_ERROR
*/
IFX_return_t PCM_MapToPhone(IFX_int32_t nDstCh,
                            IFX_int32_t nAddCh,
                            IFX_boolean_t fDoMapping,
                            BOARD_t* pBoard)
{
   IFX_return_t ret = IFX_SUCCESS;
   IFX_TAPI_MAP_PHONE_t phonemap;
   IFX_int32_t fd_ch = -1;


   if ((pBoard == IFX_NULL) || (0 > nDstCh) || (pBoard->nMaxPCM_Ch < nDstCh)
       || (0 > nAddCh) || (pBoard->nMaxAnalogCh < nAddCh))
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   memset(&phonemap, 0, sizeof(IFX_TAPI_MAP_PHONE_t));

   fd_ch = Common_GetFD_OfCh(pBoard, nAddCh);

   phonemap.nPhoneCh = nDstCh;
   phonemap.nChType = IFX_TAPI_MAP_TYPE_PCM;
   if (IFX_TRUE == fDoMapping)
   {
      /* Map pcm channel to phone channel */
      TRACE(TAPIDEMO, DBG_LEVEL_LOW, ("Map phone ch %d to pcm ch %d\n",
            (int)nAddCh, (int)nDstCh));

      ret = TD_IOCTL(fd_ch, IFX_TAPI_MAP_PHONE_ADD, (IFX_int32_t) &phonemap);
      if (IFX_ERROR == ret)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
              ("Error mapping phone ch %d to pcm ch %d using fd %d. \
              (File: %s, line: %d)\n",
              (int) nAddCh, (int) nDstCh, (int) fd_ch,
              __FILE__, __LINE__));
         return IFX_ERROR;
      }
   }
   else
   {
      /* Unmap pcm channel from phone channel */
      TRACE(TAPIDEMO, DBG_LEVEL_LOW, ("Unmap phone ch %d from pcm ch %d\n",
            (int)nAddCh, (int)nDstCh));

      ret = TD_IOCTL(fd_ch, IFX_TAPI_MAP_PHONE_REMOVE, (IFX_int32_t) &phonemap);
      if (IFX_ERROR == ret)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
              ("Error unmapping phone ch %d from pcm ch %d using fd %d. \
              (File: %s, line: %d)\n",
              (int) nAddCh, (int) nDstCh, (int) fd_ch,
              __FILE__, __LINE__));
         return IFX_ERROR;
      }
   }

   return ret;
} /* PCM_MapToPhone() */


/**
   Map pcm channel to pcm channel.

   \param nDstCh     - target channel
   \param nAddCh     - which channel to add
   \param fDoMapping - IFX_TRUE do mapping, IFX_FALSE do unmapping
   \param pBoard  - board handle

   \return IFX_SUCCESS if ok, otherwise IFX_ERROR
*/
IFX_return_t PCM_MapToPCM(IFX_int32_t nDstCh,
                          IFX_int32_t nAddCh,
                          IFX_boolean_t fDoMapping,
                          BOARD_t* pBoard)
{
   IFX_return_t ret = IFX_SUCCESS;
   IFX_TAPI_MAP_PCM_t pcmmap;
   IFX_int32_t fd_ch = -1;


   if ((pBoard == IFX_NULL) || (0 > nDstCh) || (pBoard->nMaxPCM_Ch < nDstCh) ||
        (0 > nAddCh) || (pBoard->nMaxPCM_Ch < nAddCh))
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   memset(&pcmmap, 0, sizeof(IFX_TAPI_MAP_PCM_t));

   fd_ch = Common_GetFD_OfCh(pBoard, nAddCh);

   pcmmap.nDstCh = nDstCh;
   pcmmap.nChType = IFX_TAPI_MAP_TYPE_PCM;
   if (IFX_TRUE == fDoMapping)
   {
      /* Map pcm channel to pcm channel */
      TRACE(TAPIDEMO, DBG_LEVEL_LOW, ("Map pcm ch %d to pcm ch %d\n",
            (int)nAddCh, (int)nDstCh));

      ret = TD_IOCTL(fd_ch, IFX_TAPI_MAP_PCM_ADD, (IFX_int32_t) &pcmmap);
      if (IFX_ERROR == ret)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
              ("Error mapping pcm ch %d to pcm ch %d using fd %d. \
              (File: %s, line: %d)\n",
              (int) nAddCh, (int) nDstCh, (int) fd_ch,
              __FILE__, __LINE__));
         return IFX_ERROR;
      }
   }
   else
   {
      /* Unmap pcm channel from pcm channel */
      TRACE(TAPIDEMO, DBG_LEVEL_LOW, ("Unmap pcm ch %d from pcm ch %d\n",
            (int)nAddCh, (int)nDstCh));

      ret = TD_IOCTL(fd_ch, IFX_TAPI_MAP_PCM_REMOVE, (IFX_int32_t) &pcmmap);
      if (IFX_ERROR == ret)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
              ("Error unmapping pcm ch %d to pcm ch %d using fd %d. \
              (File: %s, line: %d)\n",
              (int) nAddCh, (int) nDstCh, (int) fd_ch,
              __FILE__, __LINE__));
         return IFX_ERROR;
      }
   }

   return ret;
} /* PCM_MapToPCM() */


/**
   Map pcm channel to data channel.

   \param nDstCh     - target PCM channel
   \param nAddCh     - which data channel to add
   \param fDoMapping - IFX_TRUE do mapping, IFX_FALSE do unmapping
   \param pBoard  - board handle

   \return IFX_SUCCESS if ok, otherwise IFX_ERROR
*/
IFX_return_t PCM_MapToData(IFX_int32_t nDstCh,
                           IFX_int32_t nAddCh,
                           IFX_boolean_t fDoMapping,
                           BOARD_t* pBoard)
{
   IFX_return_t ret = IFX_SUCCESS;
   IFX_TAPI_MAP_DATA_t datamap;
   IFX_int32_t fd_ch = -1;


   if (pBoard == IFX_NULL || 0 > nDstCh || pBoard->nMaxPCM_Ch < nDstCh ||
       (0 > nAddCh) || (pBoard->nMaxCoderCh < nAddCh))
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   memset(&datamap, 0, sizeof(IFX_TAPI_MAP_DATA_t));

   fd_ch = Common_GetFD_OfCh(pBoard, nAddCh);

   datamap.nDstCh = nDstCh;
   datamap.nChType = IFX_TAPI_MAP_TYPE_PCM;
   if (IFX_TRUE == fDoMapping)
   {
      /* Map pcm channel to data channel */
      TRACE(TAPIDEMO, DBG_LEVEL_LOW, ("Map data ch %d to pcm ch %d\n",
            (int)nAddCh, (int) nDstCh));

      ret = TD_IOCTL(fd_ch, IFX_TAPI_MAP_DATA_ADD, (IFX_int32_t) &datamap);
      if (IFX_ERROR == ret)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
              ("Error mapping data ch %d to pcm ch %d using fd %d. \
              (File: %s, line: %d)\n",
              (int) nAddCh, (int) nDstCh, (int) fd_ch,
              __FILE__, __LINE__));
         return IFX_ERROR;
      }
   }
   else
   {
      /* Unmap pcm channel from data channel */
      TRACE(TAPIDEMO, DBG_LEVEL_LOW, ("Unmap data ch %d from pcm ch %d\n",
            (int)nAddCh, (int) nDstCh));

      ret = TD_IOCTL(fd_ch, IFX_TAPI_MAP_DATA_REMOVE, (IFX_int32_t) &datamap);
      if (IFX_ERROR == ret)
      {
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
              ("Error unmapping data ch %d to pcm ch %d using fd %d. \
              (File: %s, line: %d)\n",
              (int) nAddCh, (int) nDstCh, (int) fd_ch,
              __FILE__, __LINE__));
         return IFX_ERROR;
      }
   }

   return ret;
} /* PCM_MapToData() */


/**
   Return free PCM channel.

   \param pBoard - pointer to board

   \return free PCM channel or NO_FREE_PCM_CH
*/
IFX_int32_t PCM_GetFreeCh(BOARD_t* pBoard)
{
   IFX_int32_t i = 0;
   IFX_int32_t free_ch = NO_FREE_PCM_CH;


   if (pBoard == IFX_NULL || IFX_NULL == pBoard->fPCM_ChFree)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   for (i = 0; i < pBoard->nMaxPCM_Ch; i++)
   {
      if (IFX_TRUE == pBoard->fPCM_ChFree[i])
      {
         free_ch = i;
         break;
      }
   }

   return free_ch;
} /* PCM_GetFreeCh() */


/**
   Free mapped PCM channel.

   \param nPCM_Ch - pcm channel number
   \param pBoard - pointer to board

   \return IFX_SUCCESS - pcm channel is freed, otherwise IFX_ERROR
*/
IFX_return_t PCM_FreeCh(IFX_int32_t nPCM_Ch, BOARD_t* pBoard)
{
   if (pBoard == IFX_NULL || 0 > nPCM_Ch || pBoard->nMaxPCM_Ch < nPCM_Ch ||
       IFX_NULL == pBoard->fPCM_ChFree)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   pBoard->fPCM_ChFree[nPCM_Ch] = IFX_TRUE;

   return IFX_SUCCESS;
} /* PCM_FreeCh() */


/**
   Reserve pcm channel as mapped.

   \param nPCM_Ch - pcm channel number
   \param pBoard - pointer to board

   \return IFX_SUCCESS - pcm channel is reserved, otherwise IFX_ERROR
*/
IFX_return_t PCM_ReserveCh(IFX_int32_t nPCM_Ch, BOARD_t* pBoard)
{
   if (pBoard == IFX_NULL || 0 > nPCM_Ch || pBoard->nMaxPCM_Ch < nPCM_Ch ||
       IFX_NULL == pBoard->fPCM_ChFree)
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_ERROR;
   }

   if (IFX_FALSE == pBoard->fPCM_ChFree[nPCM_Ch])
   {
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("PCM ch %d, already reserved. (File: %s, line: %d)\n",
             (int) nPCM_Ch, __FILE__, __LINE__));
      return IFX_ERROR;
   }

   pBoard->fPCM_ChFree[nPCM_Ch] = IFX_FALSE;

   return IFX_SUCCESS;
} /* PCM_ReserveCh() */


/**
   Start connection on PCM, but basically activate PCM.

   \param pPhone - pointer to phone
   \param pProgramArg - pointer to program arguments
   \param nPCM_Ch - pcm channel number
   \param pBoard - pointer to board

   \return IFX_TRUE if activated, otherwise IFX_FALSE
*/
IFX_boolean_t PCM_StartConnection(TAPIDEMO_PORT_DATA_t* pPortData,
                                  PROGRAM_ARG_t* pProgramArg,
                                  BOARD_t* pBoard, CONNECTION_t* pConn)
{
   IFX_boolean_t fWideBand = IFX_FALSE;

   if ((pBoard == IFX_NULL) || (pProgramArg == IFX_NULL) ||
       (pConn == IFX_NULL) || (IFX_NULL == pPortData))
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return IFX_FALSE;
   }

   if (pProgramArg->oArgFlags.nUsePCM_WB)
   {
      fWideBand = IFX_TRUE;
   }

   if ((pProgramArg->oArgFlags.nPCM_Master)
       || (pProgramArg->oArgFlags.nPCM_Slave))
   {
      if (pConn->nType == PCM_CALL)
      {
         PCM_SetActivation(pPortData,
                           PCM_GetFD_OfCh(pConn->nUsedCh, pBoard), IFX_TRUE,
                           pConn->oPCM.nTimeslot_RX, pConn->oPCM.nTimeslot_TX,
                           fWideBand);
      } /* if (pConn->nType == PCM_CALL) */
      else if (pConn->nType == LOCAL_PCM_CALL)
      {
         PCM_SetActivation(pPortData,
                           PCM_GetFD_OfCh(pConn->nUsedCh, pBoard), IFX_TRUE,
                           pConn->oPCM.nTimeslot_RX, pConn->oPCM.nTimeslot_TX,
                           fWideBand);
      } /* else if (pConn->nType == LOCAL_PCM_CALL) */
      else if (pConn->nType == LOCAL_BOARD_PCM_CALL)
      {
         /* For the right timeslot pair, board must be used on which
            caller phone is connected. */
         PCM_SetActivation(pPortData,
                           PCM_GetFD_OfCh(pConn->nUsedCh, pBoard), IFX_TRUE,
                           pConn->oPCM.nTimeslot_RX, pConn->oPCM.nTimeslot_TX,
                           fWideBand);
      } /* else if (pConn->nType == LOCAL_BOARD_PCM_CALL) */
      else if (pConn->nType == FXO_CALL)
      {
         PCM_SetActivation(pPortData,
                           PCM_GetFD_OfCh(pConn->nUsedCh, pBoard),IFX_TRUE,
                           pConn->oPCM.nTimeslot_RX, pConn->oPCM.nTimeslot_TX,
                           fWideBand);

         COM_MOD_FXO(("FXO_TEST:PCM_IF_PCM_ACTIVE:%d:%d",
                      (int) pConn->oPCM.nTimeslot_RX, pConn->nUsedCh));
      } /* else if (pConn->nType == FXO_CALL) */
      else
      {
         /* unsupported connection type */
         return IFX_FALSE;
      }
      /* PCM is activated for this connection */
      pConn->fPCM_Active = IFX_TRUE;
      return IFX_TRUE;
   } /* if */

   return IFX_FALSE;
} /* PCM_StartConnection() */


/**
   Stop connection on PCM, but basically deactivate PCM.

   \param pPhone        - pointer to phone
   \param pProgramArg   - pointer to program arguments
   \param pBoard        - pointer to board
   \param pConn         - pointer to connection

   \return IFX_TRUE if deactivated, otherwise IFX_FALSE
*/
IFX_boolean_t PCM_EndConnection(TAPIDEMO_PORT_DATA_t* pPortData,
                                PROGRAM_ARG_t* pProgramArg,
                                BOARD_t* pBoard, CONNECTION_t* pConn)
{
   IFX_boolean_t fWideBand = IFX_FALSE;

   if ((pBoard == IFX_NULL) || (pProgramArg == IFX_NULL) ||
       (pConn == IFX_NULL) || (IFX_NULL == pPortData))
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
            ("Invalid input argument(s). (File: %s, line: %d)\n",
             __FILE__, __LINE__));
      return IFX_FALSE;
   }

   /* for FXO calls pPhone is not used and is equal IFX_NULL */
   if (FXO_CALL != pConn->nType)
   {
      if (PORT_FXS != pPortData->nType || IFX_NULL == pPortData->uPort.pPhopne)
      {
         /* Wrong input arguments */
         TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
               ("Invalid input argument(s). (File: %s, line: %d)\n",
                __FILE__, __LINE__));
         return IFX_FALSE;
      }
      if ((IFX_TRUE == pPortData->uPort.pPhopne->fWideBand_PCM_Cfg) ||
          pProgramArg->oArgFlags.nUsePCM_WB)
      {
         fWideBand = IFX_TRUE;
      }
   }
   else
   {
      if (pProgramArg->oArgFlags.nUsePCM_WB)
      {
         fWideBand = IFX_TRUE;
      }
   }

   if ((pProgramArg->oArgFlags.nPCM_Master) ||
       (pProgramArg->oArgFlags.nPCM_Slave))
   {
      if (pConn->nType == PCM_CALL)
      {
         /* check if PCM is active */
         if (IFX_FALSE != pConn->fPCM_Active)
         {
            PCM_SetActivation(pPortData,
                              PCM_GetFD_OfCh(pConn->nUsedCh, pBoard), IFX_FALSE,
                              pConn->oPCM.nTimeslot_RX, pConn->oPCM.nTimeslot_TX,
                              fWideBand);
         }
      }
      else if (pConn->nType == LOCAL_PCM_CALL)
      {
         /* check if PCM is active */
         if (IFX_FALSE != pConn->fPCM_Active)
         {
            PCM_SetActivation(pPortData,
                           PCM_GetFD_OfCh(pConn->nUsedCh, pBoard), IFX_FALSE,
                           pConn->oPCM.nTimeslot_RX, pConn->oPCM.nTimeslot_TX,
                           fWideBand);
         }
      }
      else if (pConn->nType == LOCAL_BOARD_PCM_CALL)
      {
         /* check if PCM is active */
         if (IFX_FALSE != pConn->fPCM_Active)
         {
            PCM_SetActivation(pPortData,
                           PCM_GetFD_OfCh(pConn->nUsedCh, pBoard), IFX_FALSE,
                           pConn->oPCM.nTimeslot_RX, pConn->oPCM.nTimeslot_TX,
                           fWideBand);
         }
      }
      else if (pConn->nType == FXO_CALL)
      {
         /* check if PCM is active */
         if (IFX_FALSE != pConn->fPCM_Active)
         {
            PCM_SetActivation(pPortData,
                           PCM_GetFD_OfCh(pConn->nUsedCh, pBoard), IFX_FALSE,
                           pConn->oPCM.nTimeslot_RX, pConn->oPCM.nTimeslot_TX,
                           fWideBand);

            COM_MOD_FXO(("FXO_TEST:PCM_IF_PCM_DEACTIVE:%d:%d", 
                         (int) pConn->oPCM.nTimeslot_RX, (int) pConn->nUsedCh));
         }
      }
      else
      {
         /* unsupported connection type */
         return IFX_FALSE;
      }
      /* PCM is deactivated for this connection */
      pConn->fPCM_Active = IFX_FALSE;
      /* if at least one timeslot is set */
      if (0 <= pConn->oPCM.nTimeslot_RX && 0 <= pConn->oPCM.nTimeslot_TX)
      {
         /* only master can free timeslots */
         if (pProgramArg->oArgFlags.nPCM_Master)
         {
            if (PCM_CALL == pConn->nType ||
                FXO_CALL == pConn->nType)
            {
               PCM_FreeTimeslots(pConn->oPCM.nTimeslot_RX);
            }
            /* for PCM_LOCAL_CALL and LOCAL_BOARD_PCM_CALL only master peer
               can free timeslots */
            else if ((LOCAL_PCM_CALL == pConn->nType ||
                      LOCAL_BOARD_PCM_CALL == pConn->nType) &&
                     (IFX_TRUE != pConn->oConnPeer.oLocal.fSlave))
            {
               PCM_FreeTimeslots(pConn->oPCM.nTimeslot_RX);
               PCM_FreeTimeslots(pConn->oPCM.nTimeslot_TX);
            }
         } /* PCM master */
      } /* if at least one timeslot is set */
      if (FXO_CALL != pConn->nType)
      {
         pPortData->uPort.pPhopne->fWideBand_PCM_Cfg = IFX_FALSE;
         /* for FXO timeslot number is reseted in FXO_EndConnection() */
         pConn->oPCM.nTimeslot_RX = TIMESLOT_NOT_SET;
         pConn->oPCM.nTimeslot_TX = TIMESLOT_NOT_SET;
      }
      return IFX_TRUE;
   } /* if */

   return IFX_FALSE;
} /* PCM_EndConnection() */


/**
   Get file descriptor of device for pcm channel.

   \param nPCM_Ch - pcm channel
   \param pBoard  - board handle

   \return device connected to this channel or NO_DEVICE_FD if none
*/
IFX_int32_t PCM_GetFD_OfCh(IFX_int32_t nPCM_Ch, BOARD_t* pBoard)
{
   if ((pBoard == IFX_NULL) || (0 > nPCM_Ch) || (pBoard->nMaxPCM_Ch < nPCM_Ch))
   {
      /* Wrong input arguments */
      TRACE(TAPIDEMO, DBG_LEVEL_HIGH,
           ("Invalid input argument(s). (File: %s, line: %d)\n",
            __FILE__, __LINE__));
      return NO_DEVICE_FD;
   }

   return Common_GetFD_OfCh(pBoard, nPCM_Ch);
} /* PCM_GetFD_OfCh() */



